;******************************************
; Test file for the "6502 Editor, Assembler and Debugger IDE"
;******************************************
; This assembler code example shall show the use the 6502 assembler regarding:
;
; - pseudo operators
; - definition of labels / symbols
; - definition of constants
; - location / program counter definitions
; - op-codes mnemonics
; - addressing modes
; - potential errors in writing the source code
;
;******************************************
; Pseudo operators:
;******************************************
;	.nlst	Rest of assembler source code shall NOT be placed to the list file (.lst)
;		    Warning: pseudo operator shall NOT start in the first column
;		    of the source code line. If yes, it will be interpreted as a label.
;		    This pseudo operator will only be evaluated, when the
;		    iflag parameter in function call AssembleClass.Assemble() is set to FALSE.
;	.list	Rest of assembler source code shall be placed to the list file (.lst)
;		    See comment above.
;	.byte	Place a byte constant to the next program location address:
;		    Possible byte definitions:
;			.byte %0101,%10100101 	; binary number
;			.byte 022,@22		; octal numbers - two forms
;			.byte 22,44,66,88	; decimal number
;			.byte $22,$ff,$FF	; hex - upper/lower case
; 		    An ASCII string can be defined as follows:
;			.byte "abcd\t\n",0 ; tab, new line escaped and string terminating NULL
;		    A sequence of single ASCII characters can be defined as follows:
;			.byte 'a,'b ;
;		    A sequence of byte definition shall never have spaces in between, e.g.
;			.byte $80, $81 <-- This will be interpreted as "0x80 0x00 0x81"
;			.byte $80 ,$81 <-- This will be interpreted as "0x80", without "0x81"
;	.word	Place a word constant (2 bytes) into the next loation address
;		    Order is: low byte first - high byte word second
;		    Possible word definitions:
;			.word %0101,%0101101010011010 	; binary number
;			.word 022,@22		; octal numbers - two forms
;			.word 241,45,65		; decimal number
;			.word $2145,$ff0b,$FF8C	; hex - upper/lower case
;			.word *			; place actual location address value to memory;
;		    A sequence of .word definition shall never have spaces in between
;	.dbyt	Place a word constant (2 bytes) into the next location address
;		    Order is: high byte first - low byte word second
;		    Definitions identical to above .word pseudo operator, e.g.
;			.dbyt 2241,4586,6500	; decimal number
;			.dbyt *			; place actual location address value to memory
;		    A sequence of .dbyt definition shall never have spaces in between
;
;	Note: all pseudo operators can also be written in upper case letters, like:
;		.NLST
;		.LIST
;		.BYTE
;		.WORD
;		.DBYT
;******************************************
; Warning: there MUST always be a CRLF at the end of the last source code line !!
; If not, then the last source code record will not be considered by the assembler.
; Best to always place a semicolon at the end of the source file.
;******************************************
;******************************************
; Label / Symbol Definitions:
;******************************************
;	LAB1	lda #5		; Label with op-code in one line
;	LAB1:			; Label without op-code in a line, op-code follows next line
;		lda #5
;******************************************
; Label / Symbol Usage:
;******************************************
;	LAB1	lda #5		; Label with op-code in one line
;		bne LAB1	; Jump to label, defined earlier
;		bmi LAB2	; Jump to label, defined later
;		ora #$08
;	LAB2	lda #ff
;
; The following labels/symbols are not allowed, test it by removing the semicolon
;
;A		bcs LAB1	; Error: Label named A is reserved for Accumulator
;X		bcs LAB1	; Error: Label named X is reserved for X register
;Y		bcs LAB1	; Error: Label named Y is reserved for Y register
;a		bcs LAB1	; Error: Label named a is reserved for Accumulator
;x		bcs LAB1	; Error: Label named x is reserved for X register
;y		bcs LAB1	; Error: Label named y is reserved for Y register
;******************************************
; Constants definition examples
; Constants are defined by a " = " string (spaces before and after the '=' character)
; A defintion with 'adc=$27' will result in an error -> invalid operation code
;******************************************
adc = $27 ; test, that a constant definition is not interpretet as opcode, e.g. "ADC"
x1 = 10 ;
x2 = $15 ;
aa = %01011011
bb = $BB
adz = 0
ad = $1000
ad1 = 1000
B = $77 ;space to comment subfield
C = $33	;  tab to comment subfield
DEFGHIjkl = $FFEE ; length of label / symbol name shall be less than 20 characters
D = DEFGHIjkl
A1 = $10
A2 = $15F7

;******************************************
; Pseudo operation code examples
;******************************************
;*= $0400 ; location counter adjust --> NEVER do this, because '*' is interpreted as a symbol
; You have to place the '*' to a column larger than 0 !!
; Check it out by removing the colon
START	*= $0400	; This is correct !!!!
	jmp $0400
	.byte %0101,%01011010 ; binary number
	.byte 022,@22 ; octal numbers - two forms
	.byte 22 ; decimal number
	.byte 22,44,66,88
	.byte $22,$fa,$FA ; hex - upper/lower case
	.byte 'a,'b ; single ASCII characters, no space in between
	.byte "abcd\t\n",0 ;   ASCII string, tab, new line with string terminating NULL
	.byte <D ; low byte of $FFEE
	.byte >D ; high byte $FFEE

	.word aa,bb,C,B
	.word %0101,%0101101010011010
	.word aa+B ; addition
	.word aa-B ; subtraction
	.word aa*B ; multiplication
	.word B/aa ; division
	.word C%B ; modulo
	.word B^C ; exclusive OR
	.word C
	.word ~C ; one's complement, i.e. Bitwise NOT
	.word B&C ; logical AND
	.word aa|B ; logical OR
	.word <D ; low byte of $FFEE
	.word >D ; high byte of $FFEE
	.word * ; current location
	.word aa,B,C
	.word B*[aa+C] ; one level of parenthesis
	.word D ; low byte - high byte word
	.word D/256,D%256
	.word D, B ; this is wrong, instead use the following:
	.word D,B
locat	.word *

locat1	.dbyt *
;	.dbyt D, B ; this is wrong because of space before 'B', instead use the following:
	.dbyt D,B
	.dbyt B ; high byte - low byte word

;******************************************
; Location / Program Counter definition examples
;******************************************
	*=$0400
	*=4*256+16
	*=3*256*2-1
	*=*+$5
	*=*+215+D
;*=$0400	; remove the semicolon, which results in an error at opcode lda,
		; because understood as a symbol
	lda #25
	adc #25
;******************************************
; Using the BRK opcode (Break).
; When using this opcode, then there MUST be a valid vector at IRQ address FFFE-FFFF
; In addition, the IRQ vector shall point to a valid IRQ routine.
; Only when this is implemented correctly, then the BRK opcode will successfully
; be performed by the debugger !
; It shall be noted, that the BRK will only be performed, when the I-flag
; in the status register is set to 0.
; The below source code shows a correct implementation example.
;******************************************
;	cli
;	brk		; here my BRK opcode
;	other opcodes
;irq	lda #$10	; this is the function, which shall be performed at an IRQ
;	sta $0000
;	rti
;
; And here we define the 3 valid NMI, RST and IRQ vectors.
;
;	*=$FFFA
;	.word  nmi	; NMI
;	.word  start	; RST
;	.word  irq	; IRQ
;******************************************
; Some tests
;******************************************
Start1	*=$0400
	lda #25
	adc #25
	sta $0280
	sta $0287
	sta $0288
	sta $028F
	sta $ffff
;******************************************
; Addressing Mode Examples for 6502. Not for 65C02.
;******************************************
	adc #$56	; Immediate
	adc #98		; Immediate
	adc =$56	; Immediate
	adc =98		; Immediate
	cmp #'R'	; Immediate: compare with ASCII R

	adc $568a	; Absolute
	adc 5687	; Absolute
	adc $0020	; translates automatically to zero page
	adc 0255	; translates automatically to zero page, operand evaluated as octal value
	adc @255	; translates automatically to zero page, operand evaluated as octal value
	adc $200	; translates automatically to 16 bit addressing
	adc 256		; translates automatically to 16 bit addressing
	adc A2		; Absolute
	jsr MySub	; Absolute
;	jsr $0100	; Absolute
;	jmp abcd1	; Absolute
;	jmp $0100	; Absolute

	adc $7F		; Zero Page
	adc 45		; Zero Page
	adc A1		; Zero Page

	asl A		; Accumulator
	asl a		; Accumulator
	lsr a		; Accumulator
	rol A		; Accumulator
	ror a		; Accumulator --> only available in controllers delivered after 1976

	clc		; Implied
	pha		; Implied
	pla		; Implied
; --> Note: phx, plx, phy, ply not available in 6502, only in 65C02

	cmp ($1d,X)	; Indirect X
	cmp (25,x)	; Indirect X

	cmp (A1),y	; Indirect Y
	cmp (A1),Y	; Indirect Y

	cmp A1,X	; Zero Page X
	adc A1,X	; Zero Page X
	adc $25,X	; Zero Page X
	adc 25,X	; Zero Page X
; --> Note: Zero Page Y addressing mode not available in 6502, only in 65C02

	adc $1d7f,X	; Absolute X
	cmp A2,x	; Absolute X
; --> Note: (ABS, X) indirect X addressing mode not available in 6502, only in 65C02

	adc $1d7f,Y	; Absolute Y
	cmp A2,y	; Absolute Y

abcd1
	bcs abcd1	; Relative
	bne abcd2	; Relative
abcd3	bvs abcd2	; Relative
	bmi abcd1	; Relative
abcd2	bpl abcd3	; Relative
; --> Note: bbr and bbs not available in 6502, only in 65C02

;	jmp (abcd1)	; Indirect
;	jmp ($0010)	; Indirect
; --> Note: 65C02 offers more indirect addressing codes, which are not available in 6502

;******************************************
; Test of all op-codes and addressing modes for 6502. Not for 65C02
;******************************************
;------------------------------------------
abcd4	adc #$75	; immediate hex
	adc =$75	; immediate hex
	adc =75		; immediate decimal
	adc #75		; immediate decimal
	adc #%00100101	; immediate binary
	adc #@22	; immediate octal
	adc #022	; immediate octal
	adc $20		; translates automatically to zero page
	adc $0020	; translates automatically to zero page
	adc $0200	; translates automatically to 16 bit addressing
	adc ad		; abs
	adc adz		; z-page
	adc (adz,x)	; indx
	adc (adz),y	; indy
	adc adz,x	; z-page x
	adc ad,x	; abs x
	adc ad,y	; abs y
;------------------------------------------
	and #$05	; imm
	and ad		; abs
	and adz		; z-page
	and (adz,x)	; indx
	and (adz),y	; indy
	and adz,x	; z-page x
	and ad,x	; abs x
	and ad,y	; abs y
;------------------------------------------
	asl ad		; abs
	asl adz		; z-page
	asl a		; accumulator
	asl adz,x	; z-page x
	asl ad,x	; abs x
;------------------------------------------
	bcc lab100	; relative
lab100	bcs lab101	; relative
lab101	beq lab102	; relative
lab102:
;------------------------------------------
	bit ad		; abs
	bit adz		; z-page
;------------------------------------------
	bmi lab103	; relative
lab103	bne lab104	; relative
lab104	bpl lab105	; relative
;------------------------------------------
lab105:
	brk		; implied
;------------------------------------------
	bvc lab106	; relative
lab106	bvs lab107	; relative
lab107	clc		; implied
	cld		; implied
	cli		; implied
	clv		; implied
;------------------------------------------
	cmp #$05	; imm
	cmp ad		; abs
	cmp adz		; z-page
	cmp (adz,x)	; indx
	cmp (adz),y	; indy
	cmp adz,x	; z-page x
	cmp ad,x	; abs x
	cmp ad,y	; abs y
;------------------------------------------
	cpx #$05	; imm
	cpx ad		; abs
	cpx adz		; z-page
;------------------------------------------
	cpy #$05	; imm
	cpy ad		; abs
	cpy adz		; z-page
;------------------------------------------
	dec ad		; abs
	dec adz		; z-page
	dec adz,x	; z-page x
	dec ad,x	; abs x
;------------------------------------------
	dex		; implied
	dey		; implied
;------------------------------------------
	eor #$05	; imm
	eor ad		; abs
	eor adz		; z-page
	eor (adz,x)	; indx
	eor (adz),y	; indy
	eor adz,x	; z-page x
	eor ad,x	; abs x
	eor ad,y	; abs y
;------------------------------------------
	inc ad		; abs
	inc adz		; z-page
	inc adz,x	; z-page x
	inc ad,x	; abs x
;------------------------------------------
	inx		; implied
	iny		; implied
;------------------------------------------
lab108:
	jmp (ad)	; indirect
	jmp lab108	; absolute
	jsr lab108	; absolute
;------------------------------------------
	lda #$05	; imm
	lda #2*[$05+B]	; imm
	lda ad		; abs
	lda adz		; z-page
	lda (adz,x)	; indx
	lda (adz),y	; indy
	lda adz,x	; z-page x
	lda ad,x	; abs x
	lda ad,y	; abs y
;------------------------------------------
	ldx #$05	; imm
	ldx ad		; abs
	ldx adz		; z-page
	ldx ad,y	; abs y
	ldx adz,y	; z-page y
;------------------------------------------
	ldy #$05	; imm
	ldy ad		; abs
	ldy adz		; z-page
	ldy adz,x	; z-page x
	ldy ad,x	; abs x
;------------------------------------------
	lsr ad		; abs
	lsr adz		; z-page
	lsr a		; accumulator
	lsr adz,x	; z-page x
	lsr ad,x	; abs x
;------------------------------------------
	nop		; implied
;------------------------------------------
	ora #$05	; imm
	ora ad		; abs
	ora adz		; z-page
	ora (adz,x)	; indx
	ora (adz),y	; indy
	ora adz,x	; z-page x
	ora ad,x	; abs x
	ora ad,y	; abs y
;------------------------------------------
	pha		; implied
	php		; implied
	pla		; implied
	plp		; implied
;------------------------------------------
; --> Note: RMB operation not available in 6502, only in 65C02
;------------------------------------------
	rol ad		; abs
	rol adz		; z-page
	rol a		; accumulator
	rol adz,x	; z-page x
	rol ad,x	; abs x
;------------------------------------------
; --> Note: ROR operation only available in 6502, manufactured after 1976,
;	see MOS MCS6502 Programming Manual, chapter 10.4 ROR - ROTATE RIGHT, page 150
	ror ad		; abs
	ror adz		; z-page
	ror a		; accumulator
	ror adz,x	; z-page x
	ror ad,x	; abs x
;------------------------------------------
	rti		; implied
	rts		; implied
;------------------------------------------
	sbc #$05	; imm
	sbc ad		; abs
	sbc adz		; z-page
	sbc (adz,x)	; indx
	sbc (adz),y	; indy
	sbc adz,x	; z-page x
	sbc ad,x	; abs x
	sbc ad,y	; abs y
;------------------------------------------	
	sec		; implied
	sed		; implied
	sei		; implied
; --> Note: SMB operation not available in 6502, only in 65C02
;------------------------------------------
	sta ad		; abs
	sta adz		; z-page
	sta (adz,x)	; indx
	sta (adz),y	; indy
	sta adz,x	; z-page x
	sta ad,x	; abs x
	sta ad,y	; abs y
;------------------------------------------
	stx ad		; abs
	stx adz		; z-page
	stx adz,y	; z-page y
;------------------------------------------
	sty ad		; abs
	sty adz		; z-page
	sty adz,x	; z-page x

;------------------------------------------
	tax		; implied
	tay		; implied
	tsx		; implied
	txa		; implied
	txs		; implied
	tya		; implied
; --> Note: TRB and TSB operation not available in 6502, only in 65C02
;******************************************
; End of Test of all opcodes and addressing modes
;******************************************

;******************************************
; Further source code examples and potential errors
;******************************************
	.word C
;	.nlst	--> This is correct, delete semicolon to test
; .nlst		--> This is correct, delete semicolon to test
;.nlst		--> This is wrong, delete semicolon to test
 cld
 lda #5
 adc #$FD
 sbc #3
 lda #5
 sec
 sbc #6
 lda =aa ; immediate addressing
; *=*+$5
 lda #aa ; immediate addressing, alternate
kukuk lda D ; direct addessing
 LDA aa ; page zero addressing, aa < 256
a1 = 512
a2 = 500
 lda a1-a2 ; also page zero
; asl A ; accumulator addressing
; AsL a ; accumulator addressing also
; brk ; implied addressing
hallo lda (aa,X) ; indirect,X addressing
 lda (aa),Y ; indirect,Y addressing
 lda aa,X ; zero page,X addressing
 lda D,X ; absolute,X addressing
 lda D,Y ; absolute,Y addressing
 bcc *-$10 ; relative addressing
 clc
 bcc next ; relative addressing
next bcc hallo
 jmp (D) ; indirect addressing
 ldx aa,Y ; zero page,Y addressing
 ldx aa,y ; alternate index name
; .nlst

MySub	stx ad		; abs
	stx adz		; z-page
	stx adz,y	; z-page y
	rts

irq	lda #$10	; this is the function, which shall be performed at an IRQ
	sta $0000
	rti
;
; And here we define the 3 valid NMI, RST and IRQ vectors.
;
	*=$FFFE
;	.word  nmi	; NMI
;	.word  start	; RST
	.word  irq	; IRQ
;******************************************
; Warning: there MUST always be a CRLF at the end
; of the last source code instruction line !!
; If not, then the last instruction line will not be considered by the Assembler.
;******************************************
