;
; My_KIM_V1.asm
; Version generated: 22.02.2021
;
; ===============================================================================
;
;
;
;		666666   555555   333333   000000
;		6        5             3   0    0
;		6        5             3   0    0
;		666666   555555   333333   0    0
;		6    6        5        3   0    0
;		6    6        5        3   0    0
;		666666   555555   333333   000000
;
;
;		         000000   000000   333333
;		         0    0   0    0        3
;		------   0    0   0    0        3
;		------   0    0   0    0   333333
;		------   0    0   0    0        3
;		         0    0   0    0        3
;		         000000   000000   333333
;
;
;
;
; COPYRIGHT
; MOS TECHNOLOGY, INC
; DATE OCT 18 1975 REV D
;
; 6530-003 IS AN AUDIO CASSETTE TAPE
; RECORDER EXTENSION OF THE BASIC KIM MONITOR
;
; IT FEATURES TWO BASIC ROUTINES
; 	DUMPT - STORE MEM ONTO AUDIO TAPE	Start at $E000
; 	LOADT - LOAD MEM FROM AUDIO TAPE	Start at $E069
;
; DUMPT: Store data / program from memory to audio tape
;	SAL		LSB OF START ADDRESS     $7FF4
;	SAH		MSB OF START ADDRESS     $7FF5
;	EAL		LSB OF END ADDRESS       $7FF6	!! One byte more then the last byte to store
;	EAH		MSB OF END ADDRESS       $7FF7	!! One byte more then the last byte to store
;	ID = $01-$FE	Valid ID range           $7FF8
;	Note: ID = $00 or ID = $FF SHOULD NOT BE USED, however $FF is used by MS BASIC
;
; LOADT: Load data / program from audio tape to memory
;	SAL		LSB OF START ADDRESS     $7FF4
;	SAH		MSB OF START ADDRESS     $7FF5
;	ID		ID value in	         $7FF8
; 	ID = $00	Ignore ID and use Start Address on Tape
; 	ID = $FF	Ignore ID and use SAL/SAH content as Start Address
; 	ID = $01-$FE	Load program with exactly this ID and use Start Address on Tape
;
; Note: "Ignore ID" means: Load first found program on tape !
; Therefore the nominal case is ID = $00: Ignore ID and use Start Address on Tape
;
; Note: See KIM-1 User Manual, page Appendix E1, Audio Tape Format
; ----------------------------------------------------------------
;  "Data is transmitted to the tape recorder in the form of serial
;  ASCII encoded characters (seven data bits plus Parity bit). Data
;  retrieved from the memory is converted into this form by separating each
;  byte into two half bytes. The half bytes are then converted into their
;  ASCII equivalents."
;
; This means, that each data byte is transmitted as two ASCII characters,
; hence the data rate is doubled. Valid ASCII characters are defined as
; bytes between 0=$30 to 9=$39 and A=$41 to F=$46.
; In addition, the tape dumping format is using further control characters, which are:
; - SYNC	$16	(ASCII SYN)
; - START	$2A	(* character)
; - EOT		$04	(ASCII EOT, End of Transmission)
;
; Because the physical data rate is 133 bit/s = 16 Byte/sec, or 1 KByte in 64 seconds,
; it will take slightly more than 2 minutes to save 1 KByte of RAM onto tape.
; Therefore the effective data rate is 67 bit/s or 8.4 Byte/s or 34 seconds for
; one memory page ! The ultimate horror is to save the entire 32 KByte RAM,
; which lasts more than 64 minutes !
;
; Special note:
; -------------
; Loading Microsoft BASIC, Vers. KB9 from 1977 with 8 KByte will take 16 minutes !!
; However, we are loading MS BASIC with subroutine RFD via TTY (UART),
; which only takes less than 3 minutes !
;
; Additional note:
; ----------------
; The KIM-1 User Manual on page E2, Figure E1 "Audio Tape Format" is not fully
; correct, because the drawing shows the output signal of the LM311 comparator.
; However, this signal is fed to the NAND gate U26 (pin 13) and the output pin of
; this gate is fed to the 6502 port PB7. The KIM-1 detection software has therefore to
; analyse the inverted signal, which means:
;
; - the high frequency of 3700 Hz will produce at PB7 a low level signal
; - the low frequency of 2400 Hz will produce at PB7 a high level signal
;
; Therefore the signal at the output of the NAND gate U26 (pin 11) or at the 6502
; port B7 input is as follows:
;
;        f=3700 Hz  f=2400 Hz
;   --              --------
;    |              |      |         Logical 0 Bit
;    ----------------      ----
;
;     f=3700 Hz   f=2400 Hz
;   --       ---------------
;    |       |             |         Logical 1 Bit
;    ---------             ----
;
; Also in the My-KIM design I use an inverter behind the LM311 output, which is an 74LS04, called IC1F
; on the "Tape / TTY Interface Board". The output of this inverter is connected to
; input pin PC6 of the 8255 PIA.
;
; ===============================================================================
;
; ------------------------------------------------------
; Changes made with respect to original KIM-1 software.
; ------------------------------------------------------
;
; Memory Map of My-KIM:
;
;	RAM:	$0000 - $7FFF	32 KByte (128 pages), allows to load Microsoft BASIC, Ver. KB9, 1977
;	ROM:	$E000 - $FFFF	8 KByte (32 pages)
;	8255:	I/O IC, Base Address = $C000
;		Port A address = $C000, used for Keyboard, 7-Seg. LEDs and TTY/KB bridge
;		Port B address = $C001, used for 74LS145 Row and 7-Seg. LED Selector
;		Port C address = $C002, used for Tape In/Out, TTY (UART) In/Out
;		8255 Control Word address = $C003
;	
; The IC 8255, an Intel Programmable Peripheral Interface, offers three 8-bit ports.
; --> Important: The 8255 can only set an entire port to output or to input.
; Mixing the pins of a port is not possible, but was possible in 6530 ICs !
;
; --> Very important !
; Whenever changing the control word of the 8255, i.e. changing the port directions,
; all output registers will be reset to 0 !!
; This is a very nasty thing, because we have to change the control word,
; for Port A from input to output and back for reading the Keyboard
; and driving the 6 LED displays. See further comments below in subroutine SCAND.
; It should be mentioned, that this is NOT valid for the Bit Set/Reset (BSR) function,
; where the bit 7 of the control word is 0 !
;
; Only for port C we can set a group of 4 pins (lower/higher nibble)
; to either input or output. It is therefore necessary to move the
; tape and TTY inputs and outputs lines to port C,
; and thus changing some software sequences.
;
; For ports A and B we can not set individually an output pin to low or high level.
; We only can write a complete byte to the port. This feature is only possible for port C.
;
; In the 8255 data sheet it is written, that the port C output pins can individually
; be set to 0 or 1 by the single "Bit Set/Reset" feature = BSR, citation:
; "... when Port C is beeing used as status/control for port A or B ...".
; This wording is missleading, because the BSR feature is also available
; in Mode 0 (Basic Input/Output).
; We therefore can directly make use of this feature in the tape and TTY output routines,
; where we have to set the output pins separately to low or high.
;
; In addition, My-KIM does not have a timer, as used in KIM-1 (6530-002) for the tape routines and
; Note: the measurement of the length of the start bit for TTY operation is
; already performd by software in KIM-1.
;
; The tape timing functions are therefore also realised by software,
; which is possible, because the 6502 controller is driven by a crystal.
; See subroutine WAIT at end of code.
;
; In KIM-1 the 6530-002 port usage is as follows:
; -----------------------------------------------
;
;	Port A of 6530-002:
;	- Case 1: Nominal Case, PA0 - PA7 working as input
;	          * PA0 - PA6 check the keys
;	          * PA0 in combination with ROW3 of 74LS145 (pin 4) checks for TTY/KB bridge
;	          * PA7 reads TTY input, if PB5 is low
;	- Case 2: PA0 - PA6 working as output, PA7 working as input
;	          * This case is for driving the LED segments in function SCAND
;	          * PA0 - PA6 for driving 7-Segment LEDs
;	          * When PA0 is set to output, ROW3 will NEVER be set to 0,
;		    which would be a collision!
;	          * See function SCAND, where PA0 - PA6 are temporarily set to output.
;	          * PA7 reads TTY input, if PB5 is low
;
;	Port B of 6530-002:
;	- PB0 - PB4 always output: PB1-PB4 are driving the 74LS145 1 out of 10 selector IC
;	- PB5 always output to enable/disable TTY and Tape reading / writing
;	  	* PB5 = 0 --> allows to read from TTY via PA7 and Tape via PB7
;	  	* PB5 = 1 --> allows to write to TTY via PB0 and Tape via PB7
;	- PB6 physically not existing
;	- PB7 either input or output, reading or writing the tape
;
; In My-KIM we use the three ports of the 8255 IC as follows:
; -----------------------------------------------------------
;
;	Port A:
;	- Case 1: PA0 - PA7 working as input (check keys and bridge TTY/KB selection)
;	- Case 2: PA0 - PA7 working as output (for driving 7-Segment LEDs)
;	          a) Do not use PA7 !
;		  b) Potential collision when PA0 is connected to ROW3 of 74145,
;		     which is the TTY/KB selection bridge
;		     (see page 18 of KIM-1 User Manual):
;	             When PA0 is set to output, ROW3 of 74145 (pin 4) will NEVER
;		     be set to 0, see function SCAND, where PA0 - PA6 are
;		     temporarily set to output.
;
;	Port B:
;	- PB0 - PB7 working as output (PB1-PB4 are driving the 74LS145 IC, acting as key row selector)
;	  PB0, PB5 - PB7 can be used as output for other purposes, if required.
;	  However, this is difficult, because we cannot set a port line directly
;	  to either low or high. We always have to write the whole byte to the port.
;	  Individual output port pin settings can only be done for port C, see 8255 manual.
;
;	Port C:
;	- PC0 - PC3 working as output, PC0 = Tape output, PC1 = TTY output
;	- PC4 - PC7 working as input, PC6 = Tape input, PC7 = TTY input
;	- PC2 - PC5 not used
;
; In summary we have two cases for setting the 8255 port A and B directions:
;
; Case 1: 8255 Control Word = 0x98 (nominal case)
;	Port A is input -> read keys
;	Port B is output -> driving the 74LS145 selector
;	Port C: PC0-PC3 are output, PC4-PC7 are input (for TTY/Tape)
;
; Case 2: 8255 Control Word = 0x88 (special case)
;	Port A is output -> driving the 7-segment LEDs
;	Port B is output -> driving the 74LS145 selector
;	Port C: PC0-PC3 are output, PC4-PC7 are input (for TTY/Tape)
;
; Please remember: when changing the 8255 control word,
; the output lines of all output ports will be reset to 0 !!!
;
; The change from the nominal Case 1 to the special Case 2 and back is performed only
; in function SCAND. In this function the ROW3 output of 74LS145 (pin 4) will
; never be set to low level for checking the bridge TTY/KB, which would be a collision !
;
; After CPU reset, the 8255 ports will be set to Case 1 in subroutine INITS.
; In subroutine SCAND we temporarily set the 8255 ports to Case 2 for driving the LEDs.
; Before leaving this subroutine, the 8255 ports are set back to Case 1.
;
; We keep port names for compatibility with KIM-1 code, which are basically not changed,
; only added port C (Symbol SCD and PCDD)
;
SAD	= $C000		; 8255 Port A
SBD	= $C001		; 8255 Port B
SCD	= $C002		; 8255 Port C --> port C is added for Tape and TTY interface
PADD	= $C003		; 8255 Port Control --> Write to here the 8255 control word
PBDD	= PADD		; 8255 Port Control, only to keep compatibility
PCDD	= PADD		; 8255 Port Control, only to keep compatibility
CW8255_1 = $98		; 8255 Control Word Case 1 (Nominal: Reading KIM-1 keyboard)
			; Port A is input, Port B is output,
			; Port C: PC0-PC3 are output, PC4-PC7 are input
CW8255_2 = $88		; 8255 Control Word Case 2 (Special: Driving 7-Seg. LEDs)
			; Port A is output, Port B is output,
			; Port C: PC0-PC3 are output, PC4-PC7 are input
;
; We do not have a 6530-002 timer. We therefore do not need these symbols.
; Audio tape timer functions are realised by software, see new subroutine WAIT.
;
;CLK1T	= $1744		; WRITE DIVIDE BY 1 TIME
;CLK8T	= $1745		; WRITE DIVIDE BY 8 TIME
;CLK64T	= $1746		; WRITE DIVIDE BY 64 TIME
;CLKKT	= $1747		; WRITE DIVIDE BY 1024 TIME
;CLKRDI	= $1747		; READ TIMEOUT BIT
;CLKRDT	= $1746		; READ TIME
;
; ############## CHANGE BEGIN #########################
	*=$00EC		; #### Changed from $00EF to $00EC,
			; see added SPPL/H and TMPY below
; ############## CHANGE END ###########################
;
;	CPU REGISTER SAVE AREA IN PAGE 0
;
PCL	*=*+1		; PROGRAM COUNTER LOW
PCH	*=*+1		; PROGRAM COUNTER HIGH
PREG	*=*+1		; STATUS REGISTER
SPUSER	*=*+1		; STACK POINTER of user program
ACC	*=*+1		; ACCUMULATOR
YREG	*=*+1		; Y INDEX
XREG	*=*+1		; X INDEX
;
;	KIM FIXED AREA IN PAGE 0
;	Never change the RAM location sequence of the variables INL to POINTH.
;	See comments in subroutine SCAND.
;
; ############## CHANGE BEGIN #########################
; added here
SPPL	*=*+1		; String printing pointer low
SPPH	*=*+1		; String printing pointer high
; ############## CHANGE END ###########################
CHKHI	*=*+1		; CHECKSUM FOR PAPERTAPE LOAD/DUMP
CHKSUM	*=*+1		; CHECKSUM FOR PAPERTAPE LOAD/DUMP
INL	*=*+1		; INPUT BUFFER LOW for TTY mode
INH	*=*+1		; INPUT BUFFER HIGH for TTY mode
POINTL	*=*+1		; ADDRESS LOW ON LED DISPLAY
POINTH	*=*+1		; ADDRESS HI ON LED DISPLAY
TEMP	*=*+1		; TEMPORARY ACCUMULATOR STORAGE
TMPX	*=*+1		; TEMPORARY X-REGISTER STORAGE
; ############## CHANGE BEGIN #########################
; added here
TMPY	*=*+1		; TEMPORARY Y-REGISTER STORAGE
; ############## CHANGE END ###########################
CHAR	*=*+1		; USED FOR RETRIEVING CHARACTER FROM TTY
MODE	*=*+1		; HOLDS ACTUAL Keyboard MODE ($01=ADDRESS MODE, $00=DATA MODE)
;
; At this point, the address pointer shall show $00FF
; If this is not the case, we have a severe error !!!!!!
;
; My-KIM FIXED RAM AREA IN PAGE 127 (32 KByte RAM: $0000 - $7FFF)
; In KIM-1 it was begin address $17E7 
; Note: we added here the variable ID_BAS and WAIT_TIME
;
	*=$7FE6
;
CHKL	*=*+1		; CHECKSUM LOW FOR Tape, PAPERTAPE LOAD/DUMP and TTY Load/Dump
CHKH	*=*+1		; CHECKSUM HIGH FOR Tape,PAPERTAPE LOAD/DUMP and TTY Load/Dump
SAVX	*=*+3		; USED FOR TEMPORARY SAVINGS
VEB	*=*+6		; VOLATILE EXECUTION BLOCK (executed in DUMPT4 and below LOAD13)
CNTL30	*=*+1		; TTY DELAY LOW (MEASURED TTY START BIT DURATON)
CNTH30	*=*+1		; TTY DELAY HIGH -> CNTx30 = TYPICALLY 30 CHARS PER SECOND
TIMH	*=*+1		; USED IN TTY DELAY SUBROUTINE
SAL	*=*+1		; LOW STARTING ADDRESS for tape dump / load routines
SAH	*=*+1		; HI STARTING ADDRESS for tape dump / load routines
EAL	*=*+1		; LOW ENDING ADDRESS !! ONE MORE BYTE THAN THE LAST BYTE !!!!
EAH	*=*+1		; HI ENDING ADDRESS  !! ONE MORE BYTE THAN THE LAST BYTE !!!!
ID	*=*+1		; ID for tape routines
; ############## CHANGE BEGIN #########################
; added here
WAIT_TIME *=*+1		; Delay time byte for subroutine WAIT
; ############## CHANGE END ###########################
;
; The variable ID shall show address $7FF9,
; because we continue below with RAM interrupt vectors at $7FFA
;
;	INTERRUPT VECTORS in RAM
;	Note: NMIV and IRQV is set in function INITS to $EC00 (SAVE symbol),
;	see KIM-1 User Manual, page 45-46,
;	(chapter 6. Terminate a Program, chapter 7. Single Step Program Execution)
;
	*=$7FFA
NMIV	*=*+2		; STOP VECTOR (used when stop key "ST" is pressed or in single step mode)
RSTV	*=*+2		; RESET-VECTOR, DEFINED IN ROM WITH $EC22 (KIM ENTRY VIA RESET)
IRQV	*=*+2		; IRQ-VECTOR (also used by BRK OP-CODE)
;
; ---------------------------------------------------------------- PAGE 5 -------
;
; ======================================================================
; Below the original source code of MS BASIC for KIM-1 for commands:
;  "SAVE"	to save a written BASIC program from KIM-1 RAM onto audio tape
;  "LOAD"	to load a written BASIC program from audio tape to KIM-1 RAM
;
; See attached File: "Microsoft BASIC KB9 Source-Code.txt"
;
; When we would like to use the BASIC audio tape SAVE and LOAD functionality,
; we have to change all the audio tape vectors and addresses, which are:
;
;					Address in	Address in	Called by
;	Symbol	Task			KIM-1		My-KIM		MS BASIC at
;	---------------------------------------------------------------------------
;	DUMPT	Start address for	$1800		$E000		$275C
;		tape dump
;	LOADT	Start address for	$1873		$E073		$27A3
;		tape load
;	SAL/H	Data start address	$17F5/6		$7FF5/6
;	EAL/H	Data end address	$17F7/8		$7FF7/8
;	ID	The tape ID address	$17F9		$7FF9
;	VEB+1	Volat. Exec. Block + 1	$17ED		$7FED
;	VEB+2	Volat. Exec. Block + 2	$17EE		$7FEC
;
; These changes (patches) will be performed after having loaded BASIC into memory
; right behind the label "BASIC_PATCHES".
;
; ======================================================================
; Here the original source code of MS BASIC for KIM-1 for command "SAVE"
; to save a BASIC program from BASIC memory to audio tape.
; ======================================================================
;
;	SUBTTL LOAD AND SAVE SUBROUTINES.
;	
;	IFE	REALIO-1,<			; KIM CASSETTE I/O	
;	273C BA		TSX		SAVE:	TSX		; SAVE STACK POINTER
;	273D 86 12	STX	$12		STX	INPFLG
;	273F A9 37	LDA	#$37		LDAI	STKEND-256-200	; STKEND=511 -> Accu = 55 = $37
;	2741 85 F2	STA	$F2		STA	^O362 ($F2)	; SETUP DUMMY STACK FOR KIM MONITOR
;						; Warning: we shall never use the location
;						; XREG=$F2 in the audio tape or TTY dump routine,
;						; because BASIC is using it as save address
;						; for the BASIC current stack pointer value
;	2743 A9 FE	LDA	#254		LDAI	254		; MAKE ID BYTE EQUAL TO FF HEX
;						; Here Bill made an error !
;						; The documentation states that the tape
;						; ID is $FF, but in the code the
;						; ID is $FE
;	2745 8D F9 17	STA	$17F9		STA	^O13771 ($17F9) ; STORE INTO KIM ID
;	2748 A5 78	LDA	$78		LDWD	TXTTAB		; START DUMPING FROM TXTTAB
;	274A A4 79	LDY	$79
;	274C 8D F5 17	STA	$17F5		STWD	^O13765 ($17F5) ; SETUP SAL,SAH
;	274F 8C F6 17	STY	$17F6
;	2752 A5 7A	LDY	$7A		LDWD	VARTAB		; STOP AT VARTAB
;	2754 A4 7B	LDY	$7B
;	2756 8D F7 17	STA	$17F7		STWD	^O13767 ($17F7) ; SETUP EAL,EAH
;	2759 8C F8 17	STY	$17F8
;	275C 4C 00 18	JMP	$1800		JMP	^O14000 ($1800)	; 
;	275F A6 12	LDX	$12	RETSAV:	LDX	INPFLG	; RESORE THE REAL STACK POINTER
;	2761 98  	TXS			TXS
;	2762 A9 70	LDA	#$70		LDWDI	TAPMES		; SAY IT WAS DONE
;	2764 A0 27	LDY	#$27
;	2766 4C 18 2A	JMP	$2A18		JMP	STROUT		; Null terminated string output
;	2769 4C 4F 41	.BYTE	"LOADED",0	GLOAD:	DT"LOADED"
;	276C 44 45 4D
;	276F 00					0
;	2770 53 41 56	.BYTE	"SAVED"		TAPMES: DT"SAVED"
;	2773 45 44
;	2775 0D 0A					ACRLF
;	2757 00					0
;
; ======================================================================
; Here the original source code of MS BASIC for KIM-1 for command "LOAD"
; to load a BASIC program from the audio tape to the BASIC memory.
; ======================================================================
;
;			PATSAV: BLOCK 20 (Place 20 NULL characters)
;	278C A5 78	LDA	$78	LOAD:	LDWD	TXTTAB		; START LOADING IN AT TXTTAB
;	278E A4 79	LDY	$79
;	2790 8D F5 17	STA	$17F5		STWD	^O13765 ($17F5/6) ; SETUP SAL,SAH
;	2793 8C F6 17	STY	$17F6
;	2796 A9 FF	LDA	#$FF		LDAI	255		; Define Tape ID=$FF
; Defining ID=$FF means for KIM-1: Ignore ID and use SAL/SAH content as Start Address
;	2798 8D F9 17	STA	$17F9		STA	^O13771 ($17F9)
;	279B A9 A6	LDA	#$A6		LDWDI	RTLOAD		; Return address from LOAD
;	279D A0 27	LDY	#$27
;	279F 85 01	STA	$01		STWD	^O1		; SET UP RETURN ADDRESS FOR LOAD
;	27A1 84 02	STY	$02
;	27A3 4C 73 18	JMP	$1873	JMP	^O14163 ($1873)		; GO READ THE DATA IN
;	27A6 A2 FF	LDX	#$FF	RTLOAD: LDXI	STKEND-256	; RESET THE STACK
;	27A8 9A		TXS			TXS
;	27A9 A9 49	LDA	#$49		LDWDI	READY
;	27AB A0 23	LDY	#$23
;	27AD 85 01	STA	$01		STWD	^O1
;	27AF 84 02	STY	$02
;	27B1 A9 60	LDA	#$60		LDWDI	GLOAD		; TELL HIM IT WORKED
;	27B3 A0 27	LDY	#$27
;	27B5 20 18 2A	JSR	$2A18		JSR	STROUT
;	27B8 AE ED 17	LDX	$17ED		LDXY	^O13755 ($17ED)	; GET LAST LOCATION
;	27BB AC EE 17	LDY	$17EE
;	27BE 8A  	TXA			TXA		; ITS ONE TOO BIG
;	27BF D0 01	BNE	$01		BNE	DECVRT		; DECREMENT [X,Y]
;	27C0 EA		NOP			NOP
;	27C1 EA		NOP		DECVRT: NOP
;	27C2 86 7A	STX	$7A		STXY	VARTAB		; SETUP NEW VARIABLE LOCATION
;	27C4 84 7B	STY	$7B
;	27C6 4C EE 23	JMP	$23EE		JMP	FINI>		; RELINK THE PROGRAM
;
; ------------------------------------
; End of original MS BASIC source code
; ------------------------------------
; *******************************************************
; Dump Data / Program from Memory to Audio Tape (DUMPT / DUMPT_BASIC):
; - KIM-1 keyboard mode: After completion of dump, the LED display shows "0000 xx"
; - TTY mode: After completion of dump, the address "0000 xx" is printed
; - BASIC mode: After completion of dump, the program returns to BASIC input mode
; *******************************************************
; Load Data / Program from Audio Tape to Memory (LOADT / LOADT_BASIC):
; - KIM-1 keyboard mode: After successful completion of load, the LED display shows "0000 xx"
; - KIM-1 keyboard mode: After a load failure, the LED display shows "FFFF xx"
; - TTY mode: After successful completion of load, the address "0000 xx" is printed
; - TTY mode: After a load failure, the address "FFFF xx" is printed
; - BASIC mode: After successful completion of load, the program returns to BASIC input mode
; *******************************************************
;
	*=$E000		; In KIM-1 this was address $1800
;
; ############## CHANGE BEGIN #########################
;
; We have modified the starting point for audio tape dumping to two different versions:
;
; - one for Keyboard / TTY teletype mode	--> DUMPT
; - one for the version, having loaded MS BASIC	--> DUMPT_BASIC
;
; In the BASIC case, the audio tape dumping procedure will immediately return to the
; MS BASIC specified RETSAV address symbol, which is $275F.
; To enable this, we patch the address of DUMPT_BASIC to the MS BASIC binary code,
; see code after symbol LBASIC (Load BASIC).
;
DUMPT	JSR	DUMPT_BODY
	JMP     START	; Back to KIM-1 operating system
;
DUMPT_BASIC:
	JSR	DUMPT_BODY
	JMP     $275F	; Back to BASIC via symbol RETSAV (RETurn from SAVe)
;
DUMPT_BODY:
;
; ############## CHANGE END ###########################
;
;	INIT VOLATILE EXECUTION BLOCK
;
	LDA     #$AD    ; Load "LDA" absolute opcode
	STA     VEB	; and store in volatile execution block (VEB)
	JSR     INTVEB	; fill volatile execution block with start address and RTS opcode
;
; A note at the beginning.
; The tape dump code uses the following subroutines:
;
; - OUTCHT	Send a byte directly without converting it to two ASCII
; - OUTBT	Send a byte as two ASCII bytes without doing a checksum summation
; - OUTBTC	Send a byte as two ASCII bytes with doing a checksum summation
;
; Send 100 SYNC bytes to the tape
;
	LDX     #$64	; send 100 CHARS
DUMPT1  LDA     #$16	; This is the SYNC CHAR
	JSR     OUTCHT
	DEX
	BNE     DUMPT1
;
; Send START byte
;
	LDA     #$2A	; START CHAR
	JSR     OUTCHT
;
; Send ID byte
;
	LDA     ID	; OUTPUT ID
	JSR     OUTBT
;
; Send start address
;
	LDA     SAL	; OUTPUT START ADDRESS
	JSR     OUTBTC
	LDA     SAH
	JSR     OUTBTC
;
; Send the data bytes
;
DUMPT2  LDA     VEB+1	; COMPARE FOR LAST DATA BYTE
	CMP     EAL
	LDA     VEB+2
	SBC     EAH
	BCC     DUMPT4
;
	LDA     #$2F	; OUTPUT END OF DATA CHAR
	JSR     OUTCHT
	LDA     CHKL	; Output checksum Bytes (low and high)
	JSR     OUTBT
	LDA     CHKH
	JSR     OUTBT
;
; Send two End Of Transmission (EOT) bytes
;
	LDX     #$02	; 2 CHARS
DUMPT3  LDA     #$04	; EOT-CHAR
	JSR     OUTCHT
	DEX
	BNE     DUMPT3
; ---------------------------------------------------------------- PAGE 6 -------
DUMPT5	LDA     #$00	; Display 0000, This label is used by Hypertape, see below
	STA     POINTL	; FOR NORMAL EXIT
	STA     POINTH
; ############## CHANGE BEGIN #########################
	RTS
;	JMP     START	; Back to KIM-1 operating system
; ############## CHANGE END ###########################
;
DUMPT4  JSR     VEB	; Load the byte to send to tape into accumulator
	JSR     OUTBTC	; Send the byte as two ASCII bytes including checksum summation
	JSR     INCVEB	; Increment pointer, from where to read the next byte
	JMP     DUMPT2	; Back to DUMPT2, to check if it is the last byte to send
;
; *******************************************************
; Load Data / Program from Audio Tape to Memory
; On success, the LED display shows "0000 xx"
; In case of checksum error, the LED display shows "FFFF xx"
; *******************************************************
;
; We have modified the starting point for audio tape loading to two different versions:
;
; - one for Keyboard / TTY teletype mode	--> LOADT
; - one for the version, having loaded MS BASIC	--> LOADT_BASIC
;
; The BASIC documentation states the following:
;
; "To load a program from tape prepare the cassette just as though a "LOAD" command
;  where about to be issued to the KIM-1 monitor. Then type "LOAD".....
;  If the load was successful "0000" will be displayed and simple typing "G"
;  will return to BASIC. Otherwise patch locations 0001 HEX and 0002 HEX to
;  contain their previous values (what they were before the load -
;  xx HEX and yy HEX for KB9) and do a "G" to the bad load address (BLOAD)
;  specified for your version of BASIC"
;
; In the "good load" case, everything is fine, but although reading the text more than
; 10 times I do not understand, what Bill is trying to express with
; "patch" and BLOAD address. In particual, what are the xx and yy Hex value to patch.
; Also, why do I have to patch the loactions $0001 and $0002, when I have to
; start with the BLOAD address, which is $2523 ?
;
; I tried to figure out, what the situation in BASIC is:
; - Before typing "LOAD", the content of $0000-2 is: 4C 48 23 --> JMP $2348
; - After typing "LOAD", the content of $0000-2 is:  4C A6 27 --> JMP $27A6
;
; What is clear, is the following: When working in the KIM-1 keyboard mode
; and reading the audio cassette, the result is as follows:
;
; - Reading the cassette successful --> KIM-1 shows on the display the address $0000
; - Reading the cassette with error --> KIM-1 shows on the display the address $FFFF
;
; Based on this I implemented the following procedure for loading a BASIC program:
; - When POINTL/H is $0000, then load was successful
; - When POINTL/H is $FFFF, then load was not successful
;
; In the BASIC case, the audio tape loading procedure will immediately return to the
; MS BASIC specified symbol GLOAD = $0000 (Return from Good Load), because
; BASIC has placed at address $0000 the following code:
;
;	4C A6 27	JMP $27A6
;
TAB     .WORD	LOAD12
; ############## CHANGE BEGIN #########################
LOADT	JSR	LOADT_BODY
	JMP     START   ; DISPLAY status of Load
			; either $0000 for 'good load' or $FFFF for 'bad load'
;
LOADT_BASIC:
	JSR	LOADT_BODY
	LDA	POINTL		; branch if display is not 0000
	BNE	LOADT_BASIC1
	JMP     $0000		; Good Load --> Jump back to BASIC
;
; Bad load return to BASIC
;
LOADT_BASIC1:
	LDA	#$48	; patch locations 0001 ad 0002 with the address
	STA	$0001	; before typing "LOAD"
	LDA	#$23
	STA	$0002
	JMP	$2523	; Now jump to "Bad Load" address
;
; ------------------------------------------------------------------------
; Modified the audio tape load procedure to a subroutine with RTS at the end.
; ------------------------------------------------------------------------
;
LOADT_BODY:
	LDA     #$8D    ; $8D = STA --> Init VOLATILE EXECUTION BLOCK = VEB
; ############## CHANGE END ###########################
	STA     VEB     ; Block starting with "STA" absolute opcode
	JSR     INTVEB	; fill volatile execution block with start address and RTS opcode
;
; Update VEB content
;
	LDA     #$4C    ; Store "JMP" Op-Code
	STA     VEB+3
	LDA     TAB	; Store address where to jump to, low byte
	STA     VEB+4
	LDA     TAB+1	; Store address where to jump to, high byte
	STA     VEB+5
;
; We begin with trying to receive a correct SYNC byte.
; When a SYNC byte has correctly been detected, then we continue
; to receive at least 10 additional SYNC bytes.
; 
SYNC    LDA     #$FF	; Set the default bits for the SYNC byte
	STA     SAVX	; and save it
;
SYNC1   JSR     RDBIT   ; Get a new bit from tape (in bit position 7 of accumulator)
	LSR     SAVX	; Shift the byte in memory to the right, bit 7 is now 0
	ORA     SAVX	; OR the byte with the new bit in accumulator bit position 7
	STA     SAVX	; Save the new byte
	LDA     SAVX    ; Retrieve the new byte, Why this ? This is not neccessary !
	CMP     #$16    ; Is it a SYNC byte ?
	BNE     SYNC1	; Branch if not
;
; Now receive and detect at least 10 furtherSYNC bytes.
;
	LDX     #$0A    ; TEST FOR 10 SYNC characters
SYNC2   JSR     RDCHT	; read a character (i.e. a byte) from audio tape
	CMP     #$16    ; SYNC-CHAR ?
	BNE     SYNC	; If not, start from the begin for detecting a SYNC byte
	DEX
	BNE     SYNC2
;
; Success, 10 SYNC character have been correctly detected !
; Receive the remaining SYNC bytes and wait for the START byte = $2A
;
LOADT4  JSR     RDCHT   ; Look for START byte, which is $2A
	CMP     #$2A 
	BEQ     LOAD11  ; Yes, it is $2A, branch and continue at LOAD11
	CMP     #$16    ; If not, is it still a SYNC character ?
; ---------------------------------------------------------------- PAGE 7 -------
	BNE     SYNC    ; No -> start from the top for detecting a SYNC byte
	BEQ     LOADT4  ; Yes, it is a SYNC byte,
;			; branch to wait for the START byte
;
; Success, START byte of $2A has been detected !
; Continue to read the ID byte.
; We now read bytes, which are stored as two ASCII characters on tape, call RDBYT
;
LOAD11	JSR     RDBYT   ; READ ID
	CMP     ID      ; REQUESTED ID ?
	BEQ     LOADT5  ; Load program with this ID, USE ADDR ON TAPE
	LDA     ID
	CMP     #$00    ; IGNORE ID ?
	BEQ     LOADT5  ; Yes IGNORE ID, USE ADDR ON TAPE
	CMP     #$FF    ; IGNORE START ADDRESS ?
	BEQ     LOADT6  ; YES -> take start address in SAL and SAH
	BNE     LOADT   ; NEXT PROGRAM, jump ALWAYS ->
;
LOADT5	JSR     RDBYT   ; GET Start address from tape and save it in VEB+1-2
	JSR     CHKT
	STA     VEB+1
	JSR     RDBYT
	JSR     CHKT
	STA     VEB+2
	JMP     LOADT7
;
LOADT6  JSR     RDBYT   ; GET Start address FROM TAPE but ignore it
	JSR     CHKT
	JSR     RDBYT
	JSR     CHKT
;
; This is the loop to read all data / program bytes from the tape.
; Note: each data byte is 'glued' together by two ASCII character bytes to be read from tape.
; If the first ASCII character byte is $2F (which is not an ASCII character),
; then this is the end of the data bytes.
; We then proceed to read the two checksum data bytes.
;
LOADT7  LDX     #$02
LOAD13  JSR     RDCHT
	CMP     #$2F    ; LAST CHAR ?
	BEQ     LOADT8  ; YES ->
	JSR     PACKT   ; CONVERT TO HEX
	BNE     LOADT9  ; Y=1, NON-HEX CHAR
	DEX             ; 2 CHARS ?
	BNE     LOAD13  ; NO -> NEXT ONE
;
	JSR     CHKT    ; COMPUTE CHECKSUM
	JMP     VEB	; This will store the received data byte to the required address,
			; specified in VEB+1,2
LOAD12  JSR     INCVEB  ; INCREMENT DATAPOINTER, i.e. the store address
	JMP     LOADT7
;
; Now read the two checksum bytes
;
LOADT8  JSR     RDBYT   ; End of data, now COMPARE CHECKSUM
	CMP     CHKL
	BNE     LOADT9
	JSR     RDBYT
	CMP     CHKH
	BNE     LOADT9	; --> Bad Load
	LDA     #$00	; Good Load
	BEQ     LOAD10  ; Success EXIT, jump ALWAYS ->
LOADT9  LDA     #$FF
LOAD10  STA     POINTL
; ---------------------------------------------------------------- PAGE 8 -------
	STA     POINTH
; ############## CHANGE BEGIN #########################
;	JMP     START   ; DISPLAY VALUES
	RTS
; ############## CHANGE END ###########################
;
; ---------------------------------------------------------------- PAGE 9 -------
; *******************************************************
; SUBROUTINES FOLLOWS
; *******************************************************
;
; SUB TO MOVE Start address TO VEB+1,2
;
INTVEB  LDA     SAL
	STA     VEB+1
	LDA     SAH
	STA     VEB+2
	LDA     #$60    ; CODE FOR RTS
	STA     VEB+3
	LDA     #$00
	STA     CHKL
	STA     CHKH
	RTS
;
; COMPUTE CHECKSUM FOR TAPE Dump and Load
; Checksum is saved in CHKL/CHKH
; Subroutine uses Y-register to save accumulator
;
CHKT    TAY
	CLC
	ADC     CHKL
	STA     CHKL
	LDA     CHKH
	ADC     #$00
	STA     CHKH
	TYA
	RTS
;
; OUTPUT ONE BYTE as two ASCII values:
; - Use OUTBTC to add the byte value to the checksum.
; - Use OUTBT -- NOT -- to add the byte value to the checksum.
;
OUTBTC  JSR     CHKT
OUTBT   TAY
	LSR	A
	LSR	A
	LSR	A
	LSR	A
	JSR     HEXOUT  ; OUTPUT MSD (high nibble)
	TYA
	JSR     HEXOUT  ; OUTPUT LSD (low nibble)
	TYA
	RTS
;
; Convert LSD of accumulator content to ASCII and send this ASCII to tape.
; The value in accu after the AND operation is 0b0000xxxx.
;
; Convert the xxxx low nibble to:
; - xxxx = 0 - 9 --> $30 - $39
; - xxxx = A - F --> $41 - $46
;
HEXOUT  AND     #$0F	; mask the low nibble of the byte we want to send
	CMP     #$0A
	CLC
	BMI     HEX1	; if xxxx = $0 - $9, then branch
	ADC     #$07	; when xxxx = $A - $F, add 7
; ---------------------------------------------------------------- PAGE 10 ------
HEX1    ADC     #$30	; always add $30
;
; Output one Byte to audio tape, least significant bit first.
; Uses subs ONE + ZRO
;
; If bit is 0 --> 2/3 = 3700 Hz and 1/3 = 2400 Hz
; If bit is 1 --> 1/3 = 3700 Hz and 2/3 = 2400 Hz
;
; Signal Level at 
;
;        f=3700 Hz  f=2400 Hz
;   --              --------
;    |              |      |         Logical 0 Bit
;    ----------------      ----
;
;     f=3700 Hz   f=2400 Hz
;   --       ---------------
;    |       |             |         Logical 1 Bit
;    ---------             ----
;
OUTCHT  STX     SAVX
	STY     SAVX+1
	LDY     #$08    ; STARTBIT
CHT1    JSR     ONE	; Output of 3700 Hz for 1/3 bit time
	LSR     A       ; GET DATA BIT
	BCS     CHT2	; Branch, if bit in Carry is 1 --> Set 2/3 to 2400 Hz
	JSR     ONE	; Output of 3700 Hz for 2/3 bit time
	JMP     CHT3
CHT2    JSR     ZRO	; Output of 2400 Hz for 2/3 bit time
CHT3    JSR     ZRO	; Output of 2400 Hz for 3/3 bit time
	DEY             ; ALL BITS ?
	BNE     CHT1    ; NO ->
	LDX     SAVX
	LDY     SAVX+1
	RTS
;
; Ooutput a 1 to tape at port PC0
; 9 PULSES 135 MICROSEC EACH (ca. 3700 Hz)
; It should be 138 us (f = 3623 Hz), based on KIM-1 User Manual, page E3
;
ONE     LDX     #$09
	PHA			; SAVX A
; ############## CHANGE BEGIN #########################
;
; Set wait time
; Set Port C0 = 1 (Bit Set Format of 8255 = 0x01. Write this to the 8255 control register)
; Then wait for 138 micro seconds
; Set Port C0 = 0 (Bit Set Format of 8255 = 0x00. Write this to the 8255 control register)
; Then wait for 138 micro seconds
;
; Because of LDA #$xx and STA PCDD we have to subtract 6 us from WAIT_TIME value.
; We ignore the DEX-BNE in the loop.
; Details see subroutine WAIT at end of source code.
;
	LDA	#22		; 132 us => (132 - 23) / 5 = 22 --> 132 us + 6 us = 3623 Hz 
	STA	WAIT_TIME
ONE1	LDA	#$01		;				--> 2 us
	STA	PCDD		; Port C0 = 1			--> 4 us	
	JSR	WAIT		; Saves A and X register	--> 132 us
	LDA	#$00		;				--> 2 us
	STA	PCDD		; Port C0 = 0			--> 4 us	
	JSR	WAIT		; Saves A and X register	--> 132 us
	DEX
	BNE     ONE1		; All pulses done ?
;
; ############## CHANGE END ###########################
	PLA			; RESTORE A
	RTS
;
; Output a 0 to tape at port PC0
; 6 PULSES 208 MICROSEC EACH (ca. 2400 Hz)
; It should be 207 us (f = 2415 Hz), based on KIM-1 User Manual, page E3
;
ZRO     LDX     #$06
	PHA			; SAVX A
; ############## CHANGE BEGIN #########################
;
; Set wait time
; Set Port C0 = 1 (Bit Set Format of 8255 = 0x01. Write this to the 8255 control register)
; Then wait for 207 micro seconds
; Set Port C0 = 0 (Bit Set Format of 8255 = 0x00. Write this to the 8255 control register)
; Then wait for 207 micro seconds
;
; Because of LDA #$xx and STA PCDD we have to subtract 6 us from WAIT_TIME value
; We ignore the DEX-BNE in the loop.
; Details see subroutine WAIT at end of source code.
;
	LDA	#36		; 201 us => (201 - 23) / 5 = 36 --> 199 us + 6 us = 2439 Hz
	STA	WAIT_TIME
ZRO1	LDA	#$01		;				--> 2 us
	STA	PCDD		; Port C0 = 1			--> 4 us	
	JSR	WAIT		; Saves A and X register	--> 199 us
	LDA	#$00		;				--> 2 us
	STA	PCDD		; Port C0 = 0			--> 4 us
	JSR	WAIT		; Saves A and X register	--> 199 us
	DEX
	BNE     ZRO1		; All pulses done ?
; ############## CHANGE END ##########################
	PLA			; RESTORE A
	RTS
;
; INCREMENT VEB+1,2
;
INCVEB  INC     VEB+1
	BNE     INCVE1
	INC     VEB+2
INCVE1  RTS
;
; SUB TO READ BYTE FROM TAPE
;
RDBYT   JSR     RDCHT
	JSR     PACKT
RDBYT2  JSR     RDCHT
	JSR     PACKT
	RTS
;
; Pack ASCII in accumulator into SAVX as Hex data
; Valid ASCIIs are:
; * 0 - 9 --> $30 - $39
; * A - F --> $41 - $46
;
PACKT   CMP     #$30    ; ASCII ?
	BMI     PACKT3  ; NO -> invalid ASCII
	CMP     #$47
	BPL     PACKT3  ; NO -> invalid ASCII
	CMP     #$40    ; > '9' ?
	BMI     PACKT1  ; NO ->
	CLC
	ADC     #$09
PACKT1  ROL	A
	ROL	A
	ROL	A
	ROL	A
	LDY     #$04
PACKT2  ROL	A
	ROL     SAVX
	DEY
	BNE     PACKT2
	LDA     SAVX	; at this point y already is 0 (= valid hex char)
	LDY     #$00    ; SET THE ZERO-FLAG
; ---------------------------------------------------------------- PAGE 12 ------
	RTS		; Y=0 VALID HEX
PACKT3  INY     	; Y=1 -> INVALID HEX CHAR
	RTS
;
; Get one byte from tape and return it in accumulator.
; The first bit of the byte we receive is the least significant one.
; We therefore shift the 8 bits into the byte from the left to the right.
; - Use SAVX+1 to intermediately save the byte.
; - Use SAVX+2 to save X-register.
; There is no need to set SAVX+1 at the beginning to $00 or $FF.
;
RDCHT   STX     SAVX+2	; Save X-register
	LDX     #$08	; Read 8 bits
RDCHT1  JSR     RDBIT	; Get next bit from tape, return it in sign bit of accumulator
;			  All other bits in accumulator are 0
	LSR     SAVX+1	; Right shift of the intermediately saved byte, bit 7 is now 0
	ORA     SAVX+1	; OR the accu content (bit 7) with the intermediate byte content
	STA     SAVX+1	; save the new accu content
	DEX
	BNE     RDCHT1	; Branch to read next bit
	LDA     SAVX+1	; Load the read byte once more into the accu.
			; Question: the byte was already in accu, why this LDA once more ?
	LDX     SAVX+2	; Restore X-register
	RTS
;
; The below subroutine RDBIT gets one bit from audio tape.
; My-KIM: Tape input signal is at port C6.
; The subroutine returns the read bit in the sign bit (bit 7) of the Accumulator
;
; KIM-1 is using the timer in the 6530 IC, which is not available here.
; We do the timing by software, implemented by function WAIT, working very accurate.
;
; Signal level at Port C6 is shown below.
; The logical bit level will be measured at point (*).
; In KIM-1 it is the output of U26 pin 11 open collector NAND Gate,
; driven by LM311 comparator.
;
;            |<--------- 7.542 ms -------->|	1/3 = 2.514 ms 
;
;            |<------ 1 Bit Logic 0 ------>|
;            |    1/3   |   2/3  |   3/3   |
;
; +5V  -------                   -----------
;            |   3700 Hz     *   | 2400 Hz |	* = here we read the bit status
;  0V        ---------------------         ----->
;
;
;            |<------ 1 Bit Logic 1 ------>|
;            |    1/3   |   2/3  |   3/3   |
;
; +5V  -------          --------------------
;            | 3700 Hz  |    *   2400 Hz   |	* = here we read the bit status
;  0V        ------------                  ----->
;
;
; ############## CHANGE BEGIN #########################
;
RDBIT	LDA	SCD	; First look for high level signal, i.e. last 3/3 bit time
	AND	#$40	; mask bit 6 (PC6)
	BEQ	RDBIT	; loop as long as input bit 6 is 0
;
RDBIT1	LDA     SCD	; Wait for end of previous bit = transition high level -> low level
	AND	#$40	; mask bit 6 (PC6)
	BNE     RDBIT1	; loop as long as input bit 6 is 1
;
; Wait for 1.5 * 1/3 period = 3.726 ms (half a bit time)
; We do this by calling 3 times subroutine WAIT with:
;
;	- 1. call = 1.298 milli seconds
;	- 2. call = 1.298 milli seconds		Sum = 2.596 ms
;	- 3. call = 1.130 milli seconds		Sum = 3.726 ms
;
	LDA	#255		; Value is (1298 - 23) / 5 = 255, see function WAIT
	STA	WAIT_TIME
	JSR	WAIT		; 1.298 milli second
	JSR	WAIT		; 1.298 milli second
	LDA	#221		; Value is (1130 - 23) / 5 = 221
	STA	WAIT_TIME
	JSR	WAIT		; 1.130 milli second
;
; Now read the bit status at Port PC6
;
	LDA	SCD
;
; Shift the read bit in accu by 1 bit to the left to have it in sign bit (bit 7).
; Set all other bits to 0
;
	ASL	A
	AND     #$80    ; MASK ALL EXCEPT SIGN
	RTS
; ############## CHANGE End #########################
; ---------------------------------------------------------------- PAGE 13 ------
;
; *******************************************************
; DIAGNOSIC PLLCAL TEST FOR NE565
; *******************************************************
;
; PLLCAL OUTPUT 166 MICROSEC PULSE STRING at port PC0 = Tape output line.
; Should be 172 us, because of: (138 us + 207 us) / 2.
; This is a FSK (Frequency Shift Keying) mid frequency of 2.907 KHz.
; Because of LDA #$0x and STA PCDD we have to subtract 5 us.
; But we ignore the 3 us at the end of the loop.
; We therefore have an effective FSK mid frequency of 2.865 KHz,
; because of 1/(2 x (0.000168+0.000005) + 0.000003).
; And this is exactly, what my oscilloscope is shown !
;
; How to use this calibration frequency:
; --------------------------------------
; With this calibration frequency the free running oscillator of the LM565 PLL
; can be adjusted to the correct frequency. Perform the following steps,
; where reference is made to the Tape / TTY Interface board and it parts:
;
; - connect the tape output signal at JP1 pin 1 with
;   the tape input signal line, which is at JP1 pin 2
; - set the tape output signal jumper to High audio output signal
; - place an oscilloscope probe to pin 12 of the inverter IC1 74LS04 or
;   measure this inverter output with a multimeter
; - start the PLLCAL endless running procedure, see below
; - now turn the 5K potentiometer R18 from the most left to the most right side
; - you can also turn the 5K potentiometer R18 from the most right to the most left side
; - you should see a signal level transition from either high to low or vice versa
;   at specific potentiometer positions
; - depending on how you are turning the potentiometer, you will see slightly the
;   same position, where the signal level changes
; - the best position of the potentiometer is exactly in between these two positions
; - using this 5K potentiometer it is nearly in the mid position
;
; With this position you have the correct free running PLL oscillator frequency
; of 2755 Hz for the detection of the FSK signal coming from the tape.
; This is slightly below the frequency, generated by the test procedure PLLCAL (2865 Hz).
;
; You can now do the following test:
; ----------------------------------
; Same set up as above, but it must be an oscillocope with a setting of 2 ms per divion.
; Place a second oscillocope probe to the pin 1 of JP1.
; Start the tape DUMPT procedure ($EOOO) with the parameters for
; saving the entire RAM content to the tape, i.e. 32 KByte.
; With the oscillocope probe at pin 12 of IC1 (74LS04) and with a falling edge
; trigger of this measured signal you can see, how the bits are correctly detected
; by the PLL / LM311 comparator circuit.
;
; Now the PLL shouldn't choke on any of these thousands of bits, exactly 524288 bits.
; However, this procedure take 64 minutes !
;
; The below additional two test calibration procedures (PLLCALH and PLLCALL) are only
; for further tests.
;
; Set 8255 port direction:
;	CW8255_1 = $98; 8255 Control Word Case 1
;	Port A is input, Port B is output, PC0-PC3=output, PC4-PC7=input
;
; *******************************************************
; Send mid frequency calibration signal, 2865 Hz
; *******************************************************
;
PLLCAL	LDA	#29		; (168 - 23) / 5 = 29 --> 168 us + 6 us
	STA	WAIT_TIME
	JMP	PLLCAL1
;
; *******************************************************
; Send high frequency calibration signal
; *******************************************************
;
PLLCALH	LDA	#22		; (132 - 23) / 5 = 22 --> 132 us + 6 us
	STA	WAIT_TIME
	JMP	PLLCAL1
;
; *******************************************************
; Send low frequency calibration signal
; *******************************************************
;
PLLCALL	LDA	#36		; (201 - 23) / 5 = 36 --> 199 us + 6 us
	STA	WAIT_TIME
;
; *******************************************************
; Send user defined frequency calibration signal by setting
; the WAIT_TIME byte manually (valid values are 1 to 255)
; *******************************************************
;
PLLCAL1	LDA	#CW8255_1
	STA	PADD
;                                                   Values valid for mid frequency
PLLCAL2	LDA	#$01		;		--> 2 us
	STA	PCDD		; Port C0 = 1	--> 3 us
	JSR	WAIT		; 		--> 168 us
	LDA	#$00		;		--> 2 us
	STA	PCDD		; Port C0 = 0	--> 3 us
	JSR	WAIT		;		--> 168 us
	JMP	PLLCAL2		;		--> 3 us
				; --> One loop is 349 us --> 2865 Hz
;
; *******************************************************
; Load MS BASIC, KB9
; *******************************************************
;
; Load MS BASIC V1.1 KB9 by the TTY (UART) File Data Receive subroutine.
; Then make some patches in the binary code of MS BASIC to adapt to My-KIM addresses.
;
; There is actually nothing to prevent us from starting BASIC immediately.
; May be, that the stack pointer is already somewhere below $1FF, but we
; assume, that BASIC will not occupy the entire stack of 256 bytes (page 1).
;
LBASIC	JSR	RFD		; Receive MS BASIC (KB9.BIN) File Data via TTY
	JSR	BASIC_PATCH	; Make some patches within BASIC binary code
	JMP	$4065		; Start MS BASIC V1.1 KB9
;
; *******************************************************
; When we have loaded Microsoft BASIC V1.1 KB9,
; we have to make some patches in the BASIC binary code to adapt for My-KIM addresses.
; *******************************************************
;
; We have to correct two JSR vectors for working with TTY (implemented as UART)
;
;		KIM-1 Address			My-KIM Address
;	$2456	20 EA 1E	JSR GETCH	20 C6 ED	; TTY Input routine
;	$2A51	20 A0 1E	JSR OUTCH	20 40 EE	; TTY Output routine
;
BASIC_PATCH:
	LDA	#<GETCH		; Get low byte of GETCH address
	LDY	#>GETCH		; Get high byte of GETCH address
	STA	$2457
	STY	$2458
;
	LDA	#<OUTCH		; Get low byte of OUTCH address
	LDY	#>OUTCH		; Get high byte of OUTCH address
	STA	$2A52
	STY	$2A53
;
; ---------------------------------------------------------------------
;
; BASIC is checking during program execution (RUN) and 
; program listing (LIST) on so-called CONTROL-C input characters (C=Cancel).
; However, this is not done by reading input characters by the subroutine GETCH,
; which would significantly reduce the speed of the program execution.
;
; But it checks the status of the TTY (UART) input port on a low signal level.
; In My-KIM we check the UART input port line, which is Port C7 ($C002).
; If the signal level is low (either the start bit or 0 data bits), then we assume,
; that the user wants to break to program or stop the listing execution. Hence we have
; to change the code within the addresses $26DA and $26E5, see code below.
;
; To ensure, that the signal level at Port C is mostly low during a character
; transmission, we will send from PC to My-KIM a NULL character.
;
; However, only changing the code for Port C and its bit mask is not suffficient !
; We also have to exchange the "CLC" code at address 26E5 by a "NOP" code
; to make the CONTROL-C subroutine working correctly (don't ask me what's behind the
; carry flag and where it comes from in the original KIM-1 environment).
;
; The BASIC source code for CONTROL-C checking procedure is as follows:
;
; 	26D9	60		ISCRTS: RTS
;				ISCNTC:	; Is it Control C ?
; 	26DA	A9 01			LDA	#$01	; Mask bit 0
;	26DC	2C 40 17		BIT	$1740	; Check KIM-1 Port A
;	26DF	30 F8			BMI	ISCRTS	; Branch to nominal BASIC TTY operation
;	26E1	A2 08			LDX	#$08
;	26E3	A9 03			LDA	#$03
;	26E5	18			CLC
;	26E6	C9 03			CMP	#$03
;	26E8	B0 01			BCS	STOPC
;	29EA	18			CLC
;	26EB	D0 3D		STOPC:	BNE	CONTRT	; RETURN IF NOT CONT-C OR
;							; IF NO TERMINATOR FOR STOP OR END.
;							; [C]=0 SO WILL NOT PRINT "BREAK".
;	26ED	A5 C7			LDA	TXTPTR
;	26EF	A4 C8			LDY	TXTPTR+1
;	26F1	F0 0C			BEQ	DIRIS
;	26F3	85 8A			STA	OLDTXT
;	26F5	84 8B			STY	OLDTXT+1
;	26F7	A5 86		STPEND:	LDA	CURLIN
;	26F9	A4 87			LDY	CURLIN+1
;	26FB	85 88			STA	OLDLIN
;	26FD	84 89			STY	OLDLIN+1
;	26FF	68		DIRIS:	PLA		; POP OFF NEWSST ADDR.	
;	2700	68			PLA
;	2701	A9 6C		ENDCON:	LDA	BRKTXT
;	2703	A0 22			LDY	BRKTXT+1
;	2705	A2 00			LDX	0
;	2707	86 14			STX	CNTWFL
;	2709	90 03			BCC	GORDY	; CARRY CLEAR SO DON'T PRINT "BREAK".
;	270B	4C 3D 23		JMP	ERRFIN
;	270E	4C 48 23	GORDY	JMP	READY	; TYPE "READY"
;	2711	D0 17		CONT:	BNE	CONTRT	; MAKE SURE THERE IS A TERMINATOR
;	2713	A2 D2			LDX	ERRCN	; CONTINUE ERROR
;	2715	A4 8B			LDY	OLDTXT+1; A STORED TXTPTR OF ZERO IS SETUP
;							; BY STKINI AND INDICATRES THERE IS
;							; NOTHING TO CONTINUE.
;	2717	D0 03			BNE
;	2719	4C 21 23		JMP	ERROR	; "STOP", "END", TYPING CRLF TO
;							; "INPUT" AND ^C SETUP OLDTXT
;	271C	A5 A8			LDA	OLDTXT
;	271E	85 C7			STA	TXTPTR
;	2720	84 C8			STY	TXTPTR+1
;	2722	A5 88			LDA	OLDLIN
;	2724	A4 89			LDY	OLDLIN+1
;	2726	85 86			STA	CURLIN
;	2728	84 87			STY	CURLIN+1
;	272A	60		CONTRT:	RTS		; RETURN TO CALLER
;
; ---------------------------------
; We change this upper code to:
; ---------------------------------
;
; 	26D9	60		ISCRTS: RTS
;				ISCNTC:	; Is it Control C ?
; 	26DA	A9 80			LDA	#$80	; -> Mask bit 7 (UART input at Port C7)
;	26DC	2C 02 C0		BIT	SCD	; -> Bit masking with Port C
;	26DF	30 F8			BMI	ISCRTS	; Branch to nominal BASIC TTY operation
;	26E1	A2 08			LDX	#$08
;	26E3	A9 03			LDA	#$03
;	26E5	EA			NOP		; -> We MUST change CLC to NOP
;	26E6	C9 03			CMP	#$03
;
;	and so forth ...
;
; ---------------------------------
; Now do the patch according the required changes above
; ---------------------------------
	LDA	#$80
	STA	$26DB
	LDA	#$02
	STA	$26DD
	LDA	#$C0
	STA	$26DE
	LDA	#$EA
	STA	$26E5
;
; ---------------------------------------------------------------------
;
; In addition, MS BASIC allows to LOAD and SAVE a user written BASIC program
; onto or from the audio tape. In the documentation of MS BASIC from 1977 under the
; Heading "USING THE CASSETTE INTERFACE" we find the following written:
;
; Save a BASIC program onto the audio tape
; ----------------------------------------
; "To save a program on tape prepare the cassette just as though a dump command
;  where about to be issued in the KIM-1 monitor. Then type "SAVE". A tape ID
;  of "FF" is issued for all BASIC files, so only one BASIC program can be saved per tape.
;  After completion of the "SAVE" command control will be returned to the KIM-1 monitor.
;  Reenter BASIC at the "RETSAVE" location specified for your version using
;  the "G" command"
;
; On the last page of the MS BASIC documentation we find the definition of
; the "RETSAVE" location, which is $275F, see file "KIM1-KB9-BASIC Documentation.pdf"
;
; Note:
; It is not a "FF" ID, because the code shows a "FE" ID --> error made by Bill Gates!
; But when loading a BASIC program, the ID is specified as "FF", meaning:
; --> ignore ID and use start address on tape.
; This is correct and hence not a severe error made by Bill !!!
;
; Loading a BASIC program from audio tape
; ---------------------------------------
; The BASIC doumentation states the following:
;
; "To load a program from tape prepare the cassette just as though a "LOAD" command
;  where about to be issued to the KIM-1 monitor. Then type "LOAD". Any previous program
;  or variable values will be lost. Control will return to the KIM-1 monitor on the
;  completion of the load. If the load was successful "0000 xx" will be displayed
;  and simply typing "G" will return control to BASIC. Otherwise patch location
;  $0001 and $0002 to contain their previous values (what they were before the load
;  xx and xx for KB6 and xx and xx for KB9) and do a "G" to the bad load address (BLOAD)
;  specified for your version of BASIC."
;
; In order to use the audio tape "SAVE" and "LOAD" functionality within BASIC,
; we have to patch in the BASIC binary code the My-KIM addresses, which have the
; symbols "DUMPT" and LOADT".
;
; In the MS BASIC code we find the following locations:
;
;	275C	4C 00 18	JMP	DUMPT	($1800 is the KIM-1 vector)
;	27A3	4C 73 18	JMP	LOADT	($1873 is the KIM-1 vector)
;
; We have to change these vectors to the actual ones we have in My-KIM.
;
; When using the DUMPT symbol to save the BASIC program onto audio tape,
; BASIC will return to the KIM-1 operating system when finished
; and have then to restart BASIC at symbol RETSAV = $275F. This restart precedure
; has to be performed on Teletype level, which however does not
; work correctly in my BASIC edit window control environment (BasicEditor.cpp)
; 
; We therefore start dumping to audio tape at symbol DUMPT_BASIC.
; The program at this symbol address performs the following:
;
;	- save the BASIC program on audio tape (defined by BASIC in SAL/H, EAL/H and ID)
;	- save the BASIC program to save to PC (defined by BASIC in SAL/H, EAL/H and ID)
;	- jump to symbol RETSAV = $275F, which immediately will continue with BASIC
;
; Now we do the software patch in the BASIC binary code.
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Important Note: This software patch also works for saving a BASIC program on PC,
; or for loading a BASIC program from PC.
; The current implementation is for the PC (UART) interface !!
; I can't imagine that you would like to go back to the old fashion
; audio tape interface, but if you wish, you can try it. In this
; case you should have plenty of time, may be to go to get a beer or to
; dig up your garden in the meantime :-)
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;
; Note:
; - using DUMPT_BASIC will save the BASIC program on audio tape
; - using SAVE_BASIC_PC will save the BASIC program in a file on PC
; - choose what you want
;
;	LDA	#<DUMPT_BASIC	; Get low byte of DUMPT_BASIC address (Dump to Audio Tape)
;	LDY	#>DUMPT_BASIC	; Get high byte of DUMPT_BASIC address
	LDA	#<SAVE_BASIC_PC	; Get low byte of SAVE_BASIC_PC address (Dump to PC)
	LDY	#>SAVE_BASIC_PC	; Get high byte of SAVE_BASIC_PC address
	STA	$275D
	STY	$275E
;
; ---------------------------------------------------------------------
;
; Same for the LOAD of a BASIC program
; - load the BASIC program from audio tape (defined by BASIC in by SAL/H and ID)
; - load the BASIC program from PC (defined by BASIC in by SAL/H and ID)
; - Check on type of load: Good or Bad Load
; - depending on the type of Load, jump to the addresses as specified in the documentation
;
; Note:
; - using LOADT_BASIC will load the BASIC program from audio tape
; - using LOAD_BASIC_PC will load the BASIC program from a file on PC
; - choose what you want
;
;	LDA	#<LOADT_BASIC	; Get low byte of LOADT_BASIC address (Load from Audio Tape)
;	LDY	#>LOADT_BASIC	; Get high byte of LOADT_BASIC address
	LDA	#<LOAD_BASIC_PC	; Get low byte of LOAD_BASIC_PC address (Load from PC)
	LDY	#>LOAD_BASIC_PC	; Get high byte of LOAD_BASIC_PC address
	STA	$27A4
	STY	$27A5
;
; ---------------------------------------------------------------------
;
; In addition, MS BASIC fills for the audio tape functions the
; address of SAL/H, EAL/H and ID (in KIM-1 $17F5 - $17F9), which are in
; My-KIM the address $7FF4 - $7FF8, hence we have to patch the
; following STA and STY addresses.
;
; Below code is original MS BASIC source code snippet, see above at begin of page 5
;
; For SAVE
;
; 2745 8D F9 17		STA	$17F9 (ID)	STA	^O13771 ($17F9) ; STORE INTO KIM ID
; 274C 8D F5 17		STA	$17F5 (SAL)	STWD	^O13765 ($17F5/6) ; SETUP SAL,SAH
; 274F 8C F6 17		STY	$17F6 (SAH)
; 2756 8D F7 17		STA	$17F7 (EAL)	STWD	^O13767 ($17F7/8) ; SETUP EAL,EAH
; 2759 8C F8 17		STY	$17F8 (EAH)
;
; For LOAD
;
; 2790 8D F5 17		STA	$17F5 (SAL)	STWD	^O13765 ($17F5/6) ; SETUP SAL,SAH
; 2793 8C F6 17		STY	$17F6 (SAH)
; 2798 8D F9 17		STA	$17F9 (ID)	STA	^O13771 ($17F9)
;
; Now do the patches.
;
	LDA	#<SAL		; Get low byte address of SAL
	LDY	#>SAL		; Get high byte address of SAL
	STA	$274D		; For the SAVE procedure
	STY	$274E
	STA	$2791		; For the LOAD procedure
	STY	$2792
;
	LDA	#<SAH		; Get low byte address of SAH
	LDY	#>SAH		; Get high byte address of SAH
	STA	$2750		; For the SAVE procedure
	STY	$2751
	STA	$2794		; For the LOAD procedure
	STY	$2795
;
	LDA	#<EAL		; Get low byte address of EAL
	LDY	#>EAL		; Get high byte address of EAL
	STA	$2757		; For the SAVE procedure
	STY	$2758
;
	LDA	#<EAH		; Get low byte address of EAH
	LDY	#>EAH		; Get high byte address of EAH
	STA	$275A		; For the SAVE procedure
	STY	$275B
;
	LDA	#<ID		; Get low byte address of ID
	LDY	#>ID		; Get high byte address of ID
	STA	$2746		; For the SAVE procedure
	STY	$2747
	STA	$2799		; For the LOAD procedure
	STY	$279A
;
; ---------------------------------------------------------------------
;
; In addition, MS BASIC accesses the KIM-1 Volatile Execution Block (VEB)
; to get the 'LAST LOCATION'. This 'LAST LOCATION', which is in
; VEB+1, VEB+2, is the address, where the KIM-1 LOADT procedure would have
; stored the next byte from the audio tape.
; We also have to make a patch for these both locations.
;
;					Address in	Address in
;	Symbol	Task			KIM-1		My-KIM
;	-----------------------------------------------------------
;	VEB	Volatile Execution	$17EC		$7FEB
;		Block
;
; 27B8 AE ED 17	LDX	$17ED		LDXY	^O13755 ($17ED)	; GET LAST LOCATION
; 27BB AC EE 17	LDY	$17EE
;
; First we define two constants for the assembler:
;
VEB1 = VEB+1	; Note: "VEB1=VEB+1" will not be accepted by the assembler,
VEB2 = VEB+2	; it only works with spaces around '=', don't ask me why ?
;
; Now do the patch
;
	LDA	#<VEB1		; Get low byte address of VEB1
	LDY	#>VEB1		; Get high byte address of VEB1
	STA	$27B9
	STY	$27BA
	LDA	#<VEB2		; Get low byte address of VEB2
	LDY	#>VEB2		; Get high byte address of VEB2
	STA	$27BC
	STY	$27BD
;
; In order to be able to save / load different BASIC programs,
; the BASIC save / load subroutines SAVE_BASIC_PC / LOAD_BASIC_PC
; will create file names for the PC like "BASIC_XXX.BIN", where XXX is a decimal
; number from 0 to 255. To specify the XXX value, the user has to define this
; before a "SAVE" or "LOAD" command on BASIC level is typed.
; This decimal value has to be written to the zero page $F0=240 address,
; which can be performed in BASIC by "POKE 240,X" where X is the
; required decimal value.
;
; We can use this zero page $F0 address, because the MS BASIC documentation states,
; that the zero page address $DD is the first unused address by BASIC.
;
; For default, we set the X value to 1, resulting in a file name "BASIC_001.BIN"
;
	LDA	#$01
	STA	$F0
;
; ---------------------------------------------------------------------
;
; Now we do a final BASIC patch.
; Having loaded a BASIC program, the following is printed:
;
;	LOADED
;
; and the carret (cursor) stands right behind the letter 'D'.
; But we want the following:
;
;	LOADED
;	OK
;	Carret (cursor) shall now in this new line
;
; In the BASIC source code we find a PDP-10 instruction "PATSAV: BLOCK 20",
; where 20 NULL characters are inserted in the binary file.
; It is assumed, that these 20 NULLs are never used for other purposes !
; We use this BLOCK at address 0x2778, where we patch the required string of
; "LOADED" CRLF "OK" CRLF"
; ---> 0x4C,0x4F,0x41,0x44,0x45,0x44,0x0D,0x0A,0x4F,0x4B,0x0D,0x0A,0x00
; This is a NULL terminated string, which can be printed out by the
; BASIC soubroutine STROUT. The string now begins at address 0x2778
;
; First we patch the string to the "BLOCK 20"
;
	LDA	#$4C	; L
	STA	$2778
	LDA	#$4F	; O
	STA	$2779
	LDA	#$41	; A
	STA	$277A
	LDA	#$44	; D
	STA	$277B
	LDA	#$45	; E
	STA	$277C
	LDA	#$44	; D
	STA	$277D
	LDA	#$0D	; CR
	STA	$277E
	LDA	#$0A	; LF
	STA	$277F
	LDA	#$4F	; O
	STA	$2780
	LDA	#$4B	; K
	STA	$2781
	LDA	#$0D	; CR
	STA	$2782
	LDA	#$0A	; LF
	STA	$2783
	LDA	#$00	; in general not necessary, because there is already a NULL
	STA	$2784
;
; Then we patch the pointer in A,Y to this new string in the "BLOCK 20"
; The BASIC original source code is:
;
;	27B1	A9 69		LDA #$69	LDWDI	GLOAD ; pointer to "LOADED"
;	27B3	A0 27		LDY #$27
;	27B5	20 18 2A	JSR $STROUT	($2A18)
;
; and we change it to:
;
;	27B1	A9 78		LDA #$78	LDWDI	MyNewString
;	27B3	A0 27		LDY #$27
;	27B5	20 18 2A	JSR $STROUT	($2A18)
;
	LDA	#$78
	STA	$27B2	; the Y-register has already the correct value !
;
; Now we have completely done all necessary BASIC binary file patches !
;
	RTS
;
; *******************************************************
; *******************************************************
; MS BASIC "LOAD" and "SAVE" via TTY / UART
; To be used in the KIM-1 BASIC Mode
;
; Here are two subroutines to:
; - Save a user written BASIC program to a file on PC
; - Load a user written BASIC program from a file on PC
; *******************************************************
; *******************************************************
; ----------------------------
; MS BASIC "SAVE" via TTY / UART
; ----------------------------
; Having typed "SAVE" in the BASIC input mode, BASIC prepares the following:
; - The start address of the user written program is stored in variables SAL/H
; - The end address +1 of the user written program is stored in variables EAL/H
; - The ID value of $FE=254 is stored in variable ID
;
; In the original BASIC code, BASIC will then jump to the original KIM-1 address $1800
; (symbol DUMPT) to save the program on audio tape.
; We will make a patch in the BASIC binary code to jump to our below defined
; SAVE_BASIC_PC program.
;
; The procedure of this BASIC program SAVE program is as follows.
; Send a command to the PC, having the following entries:
; - Byte 0	DC1 = $11 to indicate a Device Control command
; - Byte 1	content of $F0, user defined ID value, do not take the one provided by BASIC
; - Byte 2	SAL
; - Byte 3	SAH
;
; Based on these information, the PC will create a binary file named "BASIC_XXX.BIN",
; where XXX is the decimal number of the ID. If such a file already exists,
; it will be overwritten. The PC then sends to My-KIM the confirmation byte which is $00.
;
; In case of an error on PC, the PC then sends to My-KIM the confirmation byte which is $FF.
; - When My-KIM has received an $FF byte, the SAVE procedure will be stopped.
; - When My-KIM has received an $00 byte, a file transfer procedure is started.
;
; The file transfer procedure sends the entire BASIC program in records of up to 64 bytes
; to the PC, which are then saved in the created file. The records hold a checksum byte
; to allow a correct transmission:
; - When the PC has correctly received a record (Checksum=0), PC is confirming the
;   receiption by a $00 byte.
; - When the PC has NOT correctly received a record (Checksum!=0), PC is confirming the
;   receiption by a $FF byte and My-KIM is sending the same record once more.
;
; When all data bytes of the BASIC program have been transmitted, an End Of File
; record is sent from My-KIM to PC, indicating, that the PC file can be closed.
; Also this record receiption has to be confirmed by the PC by sending a confirmation byte.
;
; Having completed the file transfer procedure, My-KIM jumps back to the BASIC
; RETSAV symbol, which is defined as $275F and BASIC prints "SAVED".
;
; First we do some constant definitions:
; We equate the zero page address symbol CHKHI to NUMDB (NUMber of Data Bytes).
; This makes the reading of the source code a little bit easier.
; It is assumed, that this zero page address is not used by receiving or
; transmitting a byte from or to TTY.
;
DC1 = $11	; Device Control 1, used to annouce transmitting data to PC
DC2 = $12	; Device Control 2, used to annouce receiving data from PC
NUMDB = CHKHI	; CHKHI is not used in BASIC mode, we therefore can use it
;
SAVE_BASIC_PC:
	LDA	#DC1	; Device Control 1
	JSR	OUTCH	; Send content of accu to PC
	LDA	$F0	; We take the user defined ID value, and not the one from BASIC
	JSR	OUTCH
	LDA	SAL	
	JSR	OUTCH
	LDA	SAH	
	JSR	OUTCH
	JSR	GETCH	; Get the command confirmation byte from PC
	BNE	SAVE_BASIC_PC	; When not $00, repeat procedure, otherwise start Transfer
	JSR	TFD	; Transmit file data to PC as specified by ID_BAS, SAL/H and EAL/H
	JMP	$275F	; Jump back to BASIC symbol RETSAV
;
; ----------------------------
; MS BASIC "LOAD" via TTY / UART
; ----------------------------
; Having typed "LOAD" in the BASIC input mode, BASIC prepares the following:
; - The start address, where the user program shall be written,
;   is saved in variables SAL/H
; - The ID value of $FF=255 is stored in variable ID
;
; In the original BASIC code, BASIC will then jump to the original KIM-1 address $1873
; (symbol LOADT) to load the program from audio tape.
; We will make a patch in the BASIC binary code to jump to our below defined
; LOAD_BASIC_PC program.
;
; The procedure of this BASIC program LOAD program is as follows.
; Send a command to the PC, having the following entries:
; - Byte 0	DC2 = $12 to indicate a Device Control command
; - Byte 1	content of $F0, user defined ID value, do not take the one provided by BASIC
; - Byte 2	SAL
; - Byte 3	SAH
;
; Based on these information, the PC will open the existing binary file named
; "BASIC_XXX.BIN", where XXX is the decimal number of the ID. If such a file does
; not exist, the procedure will be terminated.
; The PC then sends to My-KIM the confirmation byte which is $00.
;
; In case of an error on PC, the PC then sends to KIM the confirmation byte which is $FF.
; - When My-KIM has received an $FF byte, the LOAD procedure will be stopped.
; - When My-KIM has received an $00 byte, a file transfer procedure is started.
;
; The file transfer procedure sends the entire BASIC program from the PC in records of up
; to 64 bytes to My-KIM into those addresses, which are specified in the record header.
; The records hold a checksum byte to allow a correct transmission:
; - When My-KIM has correctly received a record (Checksum=0), My-KIM is confirming the
;   receiption by a $00 byte.
; - When My-KIM has NOT correctly received a record (Checksum!=0), My-KIM is confirming the
;   receiption by a $FF byte and the PC is sending the same record once more.
;
; When all data bytes of the BASIC program have been transmitted, an End Of File
; record is sent from PC to My-KIM, indicating the end of the data transfer.
; Also this record receiption has to be confirmed by My-KIM by sending a confirmation byte.
;
LOAD_BASIC_PC:
	LDA	#DC2	; Device Control 2
	JSR	OUTCH	; Send content of accu to PC
	LDA	$F0	; We take the user defined ID value, and not the one from BASIC
	JSR	OUTCH
	LDA	SAL	
	JSR	OUTCH
	LDA	SAH	
	JSR	OUTCH
	JSR	GETCH	; Get the command confirmation byte from PC
	BNE	LOAD_BASIC_PC	; When not $00, repeat procedure, otherwise start Transfer
	JSR	RFD	; Receive file data from PC as specified by ID_BAS and SAL/H
;
; Having completed the file transfer procedure, we have to indicate to BASIC
; the last address, where the load routine has stored a data byte.
; In the original KIM-1 LOADT software this address was in
; VEB+1 and VEB+2 (VEB = Volatile Execution Block, $17ED/E).
;
; In My-KIM the last address, where LOAD_BASIC_PC have written a byte is in the
; pointer SPPL/H + the number of bytes transmitted in the last record.
; This number is in variable NUMDB, hence we have to add the content of SPPL/H with
; the content of NUMDB. The result has to be written to the address
; pointer VEB+1 / VEB+2, so that BASIC can pick it up.
; Compute the last address, where we have stored a byte and save it
; in VEB+1 and VEB+2, because BASIC needs it here.
;
; Correctly speaking, this is not the last address, where we have written a byte.
; The address in VEB+1/2 is the one, where we would write the next program byte from
; audio tape. Apparently this should not be a problem, because also the original MS BASIC
; source code takes this into account, see below code snippet, indicated by
; comment: "ITS ONE TOO BIG"
;
; In any way, the code does not make any sense to me, in particular
; the branch to DECVRT and the NOP operations. Only god knows, what Bill (or was it Paul)
; thought, when writing this code.
;
;  ------------------------------------------------------------------------------
; 			JMP	^O14163 ($1873)	; GO READ THE DATA
;   	RTLOAD: 	LDXI	STKEND-256	; RESET THE STACK
;			TXS
;   			LDWDI	READY		
;			STWD	^O1 ($0001)
;			LDWDI	GLOAD		; TELL HIM IT WORKED
;			JSR	STROUT
;			LDXY	^O13755 ($17ED)	; GET LAST LOCATION (VEB+1/2)
;			TXA			; ITS ONE TOO BIG
;			BNE	DECVRT		; DECREMENT [X,Y]
;			NOP
;	DECVRT: 	NOP
;			STXY	VARTAB		; SETUP NEW VARIABLE LOCATION
;			JMP	FINI		; RELINK THE PROGRAM
;  ------------------------------------------------------------------------------
;
;	CLC
;	LDA	SPPL	; see subroutine RFDR below, after symbol RFDR1
;	ADC	NUMDB	; see subroutine RFDR below, after symbol RFDR1
;	STA	VEB+1
;	LDA	SPPH	; see subroutine RFDR below, after symbol RFDR1
;	ADC	#0
;	STA	VEB+2
;
; Then the load procedure jumps back to BASIC which is simply performed by
; jumping to addess $0000, because BASIC has already written there the following code:
;	$0000	4C
;	$0001	xx
;	$0002	yy
; where xx and yy is the "Return from Good Load" vector to BASIC.
; Jump back to BASIC "Good Load" return vector.
; We could also directly jump to $27A6, which is the BASIC "Good Load" return
;
	JMP	$0000
;
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
; TTY - UART - File Transfer Procedures
; To be used in the Teletype Mode !
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
;
; My-KIM allows to transfer data file contents from RAM to PC and vice versa.
; The transfer is nearly identical with the KIM-1 audio tape dump and load procedures.
; The user has to specify the the type of transfer in the following KIM-1 addresses:
;
; $7FF4	SAL - Start address of data to be transmitted or received (low byte of address)
; $7FF5	SAH - Start address of data to be transmitted or received (high byte of address)
; $7FF6	EAL - End address of data to be transmitted or received (low byte of address)
; $7FF7	EAH - End address of data to be transmitted or received (high byte of address)
; $7FF8	ID - File number, can be $00 up to $FF
;
; Note: the end address shall be one byte higher than the last byte to be transmitted,
; see also page 14 in KIM-1 User Manual, lower part of page.
;
; Transmitting My-KIM RAM content to PC and save it in a file:
;------------------------------------------------------------
; The user has to specify the following variables:
; - SAL/H	Defines the start address of the data to transmit
; - EAL/H	Defines the end address of the data to transmit (+1 as for audio tape)
; - ID		Defines the file number and can be within 0x00 - 0xFF
;
; When the user types 'T' in the TTY mode, the subroutine TFILE will be called, 
; performing the following procedure:
;
; - Send DC1 = 0x11 to PC
; - Send ID to PC
; - Send SAL/H to PC, low byte first, high byte second
;
; Then KIM is expecting from PC a confirmation byte, that sent command has been received:
; - Confirmation Byte = $00 --> command correctly received,
;   KIM is allowed to start to transmit the file data to PC
; - Confirmation Byte = $FF --> command not correctly received, send command once more
;
; Then the PC is creating two files named: KIM_DATA_XXX.BIN and KIM_ADDR_XXX.BIN with:
; - XXX is the decimal ID number, e.g. ID = 0x24 --> XXX = "036"
; - hence the file names on PC are "KIM_DATA_036.BIN" and "KIM_ADDR_036.BIN"
;
; If the files already exist, they will be overwritten.
; The 'DATA' file holds the binary data sent from KIM to PC.
; The 'ADDR' file only holds the two start address SAL and SAH bytes.
;
; Then the subroutine uses subroutine TFD to send the data via UART to the PC
; and the PC writes the sent data in binary form into the created data file.
;
; Transfering a PC data file to My-KIM RAM:
;------------------------------------------
; The user has to specify the following variables:
; - SAL/H	Defines the start address, where to write into the RAM
;		If the start address is 0xFFFF, then the address in the
;		address file name shall be used
; - ID		Defines the file number and shall be within 0x00 - 0xFF
;
; When the user types 'R' in the TTY mode, the subroutine RFILE will be called, 
; performing the following procedure:
;
; - Send DC2 = 0x12 to PC
; - Send ID to PC
; - Send SAL/H to PC, low byte first, high byte second
;
; Then KIM is expecting from PC a confirmation byte, that the sent command has been received:
; - Confirmation Byte = $00 --> command correctly received,
;   PC will start to transmit the file data
; - Confirmation Byte = $FF --> command not correctly received, send command once more
;
; The PC looks for an existing file "KIM_DATA_XXX.BIN", having exactly the ID=XXX value.
; - If the file exists, then the PC sends the content of the file to My-KIM via UART.
; - If the file does NOT exist, then the PC sends a NULL content file to My-KIM via UART,
;   which does not write any data to the KIM RAM.
;
; The handling of the specified SAL/H is as follows:
; - If the SAL/H content is 0xFFFF, the PC sends Intel Hex File data records using
;   the start address, which is defined in the file named "KIM_ADDR_XXX.BIN".
; - If the SAL/H content is NOT 0xFFFF, the PC sends Intel Hex File data records using
;   the start address, as defined by SAL/H.
;
; Then My-KIM uses subroutine RFD to receive the data via UART from the PC
; and will store the data to the RAM area as specified by the Intel Hex File records.
; 
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
; TFILE: Transmit a data file to PC via TTY / UART
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
;
TFILE	LDA	#DC1	; Device Control 1
	JSR	OUTCH	; Send content of accu to PC
	LDA	ID	
	JSR	OUTCH
	LDA	SAL	
	JSR	OUTCH
	LDA	SAH	
	JSR	OUTCH
	JSR	GETCH	; Get the command confirmation byte from PC
	BNE	TFILE	; When not $00, repeat procedure, otherwise start Transfer
	JSR	TFD	; Transmit file data to PC as specified by ID, SAL/H and EAL/H
	RTS
;
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
; RFILE: Receive a data file from PC via TTY / UART
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++
;
RFILE	LDA	#DC2	; Device Control 2
	JSR	OUTCH	; Send content of accu to PC
	LDA	ID	
	JSR	OUTCH
	LDA	SAL	
	JSR	OUTCH
	LDA	SAH	
	JSR	OUTCH
	JSR	GETCH	; Get the command confirmation byte from PC
	BNE	RFILE	; When not $00, repeat procedure, otherwise start Transfer
	JSR	RFD	; Receive file data from PC as specified by ID and SAL/H
	RTS
;
; *******************************************************
; *******************************************************
; TTY Receive File Data Functionality
; *******************************************************
; *******************************************************
;
; The file data is transmitted from PC to KIM-1 RAM in form of records,
; which hold up to 64 data bytes.
; The data bytes sent in the record are stored to the memory locations,
; as specified within the record.
;
; The transmission format nearly follows the Intel Hex File Format and
; is described below:
;
;	Byte 0: Number of data bytes in the record to be stored to memory
;	Byte 1: Low address starting byte, where to store the following data bytes
;	Byte 2: High address starting byte, where to store the following data bytes
;	Byte 3: Record Type: $00 = data byte record, $01 = End Of File (EOF) record
;	Byte 4: 1. data byte
;	Byte 5: 2. data byte
;	:
;	Byte n-1: Last data byte
;	Byte n:   Checksum
;
; The checksum is computed as follows:
; 	- Sum the values of all bytes from Byte 0 to Byte n (as modulo 256, without carry)
;	- The sum shall now be 0, if not, then there was a transmission error
;
; In addition we expect an End Of File (EOF) record, which format is as follows:
;
;	Byte 0: = $00	Number of data bytes in record to be stored to memory
;	Byte 1: = $00	Low address byte, where to store the data bytes
;	Byte 2: = $00	High address byte, where to store the data bytes
;	Byte 3: = $01	End Of File record
;	Byte 4: = $FF   Checksum, Summing all bytes of the EOF record will give $00
;
; Note: Each file transmission MUST have an EOF record !
;
; Having received a record of the file and having stored it to memory,
; then send back a confirmation byte to the PC via UART (TTY).
; The confirmation byte is defined as follows:
;
; - $00 --> record received correctly, expecting next record, if it was not the EOF record
; - $FF --> record not correctly received, expecting same record once more from PC
;
; *******************************************************
; Subroutine to Receive File Data from TTY (RFD)
;
; Read all file data records from TTY,
; until we have received the End Of File record (type $01).
; The data received in these records are saved to the memory locations,
; as specified in the record.
;
; Having received a record, we always confirm this by sending to PC:
; - 0x00 --> record received correctly, expecting next record, if it was not the EOF record
; - 0xFF --> record NOT received correctly, send same record once more
;
; If we have a checksum error in the received file data record,
; request from PC to send the record once more.
; *******************************************************
;
RFD	JSR	RFDR	; Read a file data record from PC via TTY (UART)
	BNE	RFD1	; If not received correctly, request record once more
	LDA	#$00	; Record received correctly
	JSR	OUTCH	; Send $00 to PC to request next record
;
	LDA	ID	; Check received record type
	CMP	#$01	; Is it an EOF record ?
	BNE	RFD	; If not an EOF record, then receive the next record
	RTS		; otherwise successfully return from this subroutine.
;
RFD1	LDA	#$FF	; Send to PC the request to send the record once more
	JSR	OUTCH	; Send content of accu to PC
	JMP	RFD	; Jump back to get record once more
;
; *******************************************************
; Subroutine to Receive a File Data Record from TTY (RFDR)
;
; The received record type is saved in symbol ID:
;	- $00 --> nominal data record
;	- $01 --> EOF record
;
; On return, the accu holds the checksum of the record,
; which shall be $00, if received correctly.
; All other values in accu indicate a transmission error.
; *******************************************************
;
RFDR	LDA	#$00
	STA	CHKSUM	; set checksum initially to 0
;
	JSR	GETCH	; get byte from TTY, byte is in accumulator
	STA	NUMDB	; save number of data bytes to receive
	JSR	RFDRCS	; Compute Checksum
	JSR	GETCH
	STA	SPPL	; save to store address low, in zero page
	JSR	RFDRCS	; Compute Checksum
	JSR	GETCH
	STA	SPPH	; save to store address high, in zero page
	JSR	RFDRCS	; Compute Checksum
	JSR	GETCH
	STA	ID	; Save record type
	JSR	RFDRCS	; Compute Checksum
;
; If number of data bytes to be received = 0, skip the receiving of data bytes.
; Same as for the EOF record
;
	LDA	NUMDB
	BEQ	RFDR2	; branch, because number data bytes to be received is 0
	LDA	ID
	CMP	#$01	; EOF record type = $01
	BEQ	RFDR2	; branch, because it is the EOF record
;
; Now the loop over all data bytes to be received within this record.
; Note: GETCH will overwrite Y-register, we therefore have to save it temporarily.
;
	LDY	#$00	; Begin with y-index = 0
RFDR1	STY	TMPY	; Save Y-register
	JSR	GETCH	; received byte is now in accu
	LDY	TMPY	; restore Y-register
	STA	(SPPL),Y ; store the received byte to memory at specified address
	JSR	RFDRCS	; Compute Checksum
	INY		; increment Y-register
	CPY	NUMDB	; do we have received all bytes of this record ?
	BNE	RFDR1	; No --> branch back
	JSR	RFDRVEB	; Save the last address, where data was written,
			; we need this when loading a BASIC program.
;
; All bytes received.
; Receive the checksum byte, which is the last byte of the record
; Compute the final checksum, which is then in the accu and will be
; returned to the calling program.
;
RFDR2	JSR	GETCH
	JSR	RFDRCS	; Compute Checksum
	RTS		; Checksum now in accu, if $00 then OK, otherwise error
;
; Checksum Computation
;
RFDRCS	CLC
	ADC	CHKSUM
	STA	CHKSUM
	RTS
;
; Because we use the subroutine RFDR also for loading a BASIC program,
; we have to make some preparations.
; This subroutine RFDRVEB computes the last address+1, where we have written a
; received byte and we place this address to pointer VEB+1 / VEB+2.
; We have to do this, because BASIC needs it in order to know,
; the size of the received BASIC program.
; The reason for this is, that BASIC assumes, that we load a program
; from the original KIM-1 audio tape routine named LOADT.
; This LOADT routine holds the last written received byte address in the
; VEB vector (Volatile Execution Block) at address VEB1 and VEB2.
;
RFDRVEB:
	CLC
	LDA	SPPL	; Get the base address
	ADC	NUMDB	; add the Y-Index, based on content of NUMDB
	STA	VEB1	; VEB1 = VEB+1
	LDA	SPPH
	ADC	#0	; high address byte of NUMDB is 0
	STA	VEB2	; VEB1 = VEB+2
	RTS
;
; *******************************************************
; *******************************************************
; TTY Transmit File Data Functionality via UART
; *******************************************************
; *******************************************************
;
; This subroutine allows to transmit a complete binary file
; from KIM-1 RAM to the PC via TTY (UART)
;
; The file data is transmitted from KIM-1 to PC in form of records,
; holding up to 64 data bytes.
;
; The format of a record is identical with the one for receiving data
; from PC and is not repeated here, see description above before symbol RFD.
;
; The start and end address of the RAM area to be transmitted shall be specified
; in the locations SAL/H and EAL/H. The file ID is specified in symbol ID.
; Note: None of these locations will be modified by the procedure.
; The end address EAL/H shall be 1 byte larger than the last address to be transmitted.
; This is the same notation as for the KIM-1 DUMPT subroutine, dumping data on audio tape.
; If the end address is not larger than the start address, then send an EOF record to PC.
;
; *******************************************************
; Subroutine to Transmit File Data from KIM-1 RAM to PC via TTY (TFD)
;
; Transmit the file in format of Intel Hex File data records via TTY to the PC
; *******************************************************
;
; Because we will modify the SAL/H values,
; we save them on stack and restore them at the end of the procedure.
;
TFD	LDA	SAL
	PHA
	LDA	SAH
	PHA
;
; First we check, whether the specified end address is larger than the start address.
; We do this by computing EAL/H - SAL/H:
; - When carry is set, then EAL/H is larger than SAL/H
; - When carry is cleared, then EAL/H is NOT larger than SAL/H
; In the latter case we only send an EOF record to the PC,
; hence no data will be transmitted to the PC.
;
	SEC
	LDA	EAL
	SBC	SAL
	LDA	EAH
	SBC	SAH
	BCC	TFD4	; branch, because end address is NOT larger than start address	
;
; Here we start the transmission procedure.
;
TFD1	LDA	#64
	STA	NUMDB	; We are prepared to send 64 data bytes in one record
;
; We have to compute the number of data bytes we really HAVE to send in the next record.
; First we compute {Start address + 64} and save result in SPPL/H.
;	
	CLC
	LDA	SAL
	ADC	NUMDB
	STA	SPPL
	LDA	SAH
	ADC	#$00
	STA	SPPH
;
; Is content in SPPL/H larger than End address ?
; We do here a subtraction of:
; 	{End address - (Start address + 64)} = {End address - SPPL/H}
; and save the result in SPPL/H.
;
; If carry flag is 1, then it is NOT larger, we can transmit 64 bytes.
; If carry flag is 0, then it is larger, we have to transmit less than 64 bytes,
; which also could be 0.
;
	SEC
	LDA	EAL
	SBC	SPPL
	STA	SPPL
	LDA	EAH
	SBC	SPPH
	STA	SPPH
	BCC	TFD3	; branch, because we have to transmit less than 64 bytes
;
; Now we have in NUMDB the correct value of number of data bytes to transmit.
; Place start address for next record to SPPL/H vector, then send the record.
;
TFD2	LDA	SAL
	STA	SPPL
	LDA	SAH
	STA	SPPH
;
; Send the record with:
; - start address in SPPL/H
; - number of data bytes in NUMDB
;
; Note: SPPL/H and NUMDB will not be modified by subroutine TFDR
;
	JSR	TFDR
;
; Now we wait for the confirmation from PC, that it has correctly received the record
; based on its checksum computation. The PC is sending:
; - $00 -> correctly received record
; - $FF -> NOT correctly received, meaning "please send record once more"
;
	JSR	GETCH
	CMP	#$FF
	BEQ	TFD2	; Send data record once more, NUMDB is not modified by TFDR
;
; PC has correctly received the record.
; Increase start address by number of bytes just sent.
;
	CLC
	LDA	SAL
	ADC	NUMDB
	STA	SAL
	LDA	SAH
	ADC	#$00
	STA	SAH
;
; Jump back to prepare for next record to send.
;
	JMP	TFD1
;
; We have to transmit less than 64 bytes. This means it could also be 0.
; Now compute how many bytes we really have to transmit.
; In SPPL/H we have:
; 	{End address + (Start address + 64)} as a negative value
; We do an addition by {64 + negative Value}.
; Store the value of the low order byte of subtraction in NUMDB.
;
; If the result is 0 in NUMDB, we shall not transmit any byte, then we send an EOF record.
; Otherwise send a data record with NUMDB data bytes.
;
TFD3	CLC
	LDA	#64
	ADC	SPPL
	STA	SPPL
	STA	NUMDB
	LDA	#0
	ADC	SPPH
	STA	SPPH
;
	LDA	NUMDB	; if NUMDB not 0, then transmit a data record
	BNE	TFD2
;
; Send EOF record whose type is $01
;
TFD4	LDA	#$00
	JSR	OUTCH	; send number of data bytes to transmit
	LDA	#$00
	JSR	OUTCH	; send low address byte
	LDA	#$00
	JSR	OUTCH	; send high address byte
	LDA	#$01
	JSR	OUTCH	; send record type, here $01
	LDA	#$FF
	JSR	OUTCH	; send checksum
;
; Now we wait for the confirmation from PC, that it has correctly received the record
; based on its checksum computation. The PC is sending:
; - $00 -> correctly received record
; - $FF -> NOT correctly received, meaning "please send record once more"
;
	JSR	GETCH
	CMP	#$FF
	BEQ	TFD4	; Send EOF record once more
;
; Restore the SAL/H values from stack.
; Return from subroutine.
;
	PLA
	STA	SAH
	PLA
	STA	SAL
;
	RTS
;
; *******************************************************
; Subroutine to Transmit one record of data from RAM to PC via TTY (RTFD)
; It is assumed, that:
; - the number of data bytes to send is in NUMDB
; - the begin address of the data bytes to send is in SPPL/H
; *******************************************************
;
TFDR	LDA	#$00
	STA	CHKSUM	; set checksum initially to 0
	LDA	NUMDB	; get the number of bytes to transmit
	JSR	TFDRCS	; Compute Checksum
	JSR	OUTCH	; send num data bytes to PC via TTY (UART)
	LDA	SPPL
	JSR	TFDRCS	; Compute Checksum
	JSR	OUTCH	; send low address byte
	LDA	SPPH
	JSR	TFDRCS	; Compute Checksum
	JSR	OUTCH	; send high address byte
	LDA	#$00	; record type ($00 = data record, $01 = EOF record)
	JSR	TFDRCS	; Compute Checksum
	JSR	OUTCH	; send record type
;
; Now do the loop over all data bytes to send to PC.
; Save Y-register temporarily in TMPY.
;
	LDY	#$00
TFDR1	LDA	(SPPL),Y	; get the byte to send
	JSR	TFDRCS	; Compute Checksum
	STY	TMPY
	JSR	OUTCH	; send data byte
	LDY	TMPY
	INY		; inc Y-index
	CPY	NUMDB	; have we sent all bytes ?
	BNE	TFDR1	; branch back if not
;
; Here we have sent all data bytes.
;
; Now send the checksum and return.
; At the receiver side the checksum computation is as follows:
; - sum up all received data bytes as a checksum
; - add to this the now sent checksum byte
; - then the value shall become NULL, if no error in the transmission
; This means we have to subtract here the computed checksum from NULL !
;
TFDR2	SEC
	LDA	#$00
	SBC	CHKSUM
	JSR	OUTCH	; send final checksum byte
	RTS
;
; Checksum Computation, save accu on stack.
;
TFDRCS	PHA
	CLC
	ADC	CHKSUM
	STA	CHKSUM
	PLA
	RTS
;
; *******************************************************
; Test: Write page 2 with ascending byte numbers
; Used to test audio tape writing and reading
; *******************************************************
;
WBYTE	LDX	#0
WBYTE1	TXA
	STA	$0200,X
	INX
	BNE	WBYTE1
;
; Done and back to operating system and show $0200
;
	LDA	#0
	STA	POINTL
	LDA	#2
	STA	POINTH
	JMP	START
;
; *******************************************************
; Test: Compare content of page 2 with page 3
; Used to test audio tape writing and reading
; *******************************************************
;
CMPP	LDX	#0
CMPP1	LDA	$0200,X
	CMP	$0300,X
	BNE	CMPP2
	INX
	BNE	CMPP1
	JMP	CMPP3
;
; In case of error, show error byte position
;
CMPP2	STX	POINTL
	LDA	#2
	STA	POINTH
	JMP	START
;
; If OK, show $0200
;
CMPP3	LDA	#0
	STA	POINTL
	LDA	#2
	STA	POINTH
	JMP	START
;
; *******************************************************
; Test the subroutine RDBIT (Read tape input bit from port C6).
; The read bit is in accumulator at bit 7 (sign bit).
; Output the read bit to port C2.
; *******************************************************
;
; Do the loop over reading the bits from tape
;
T_RD1	JSR	RDBIT	; The read bit is now in accumulator at bit 7 (sign bit)
	AND	#$80
	BNE	T_RD2	; Branch if Bit is 1
	LDA	#$04	; Set port C2 to 0
	STA	PADD
	JMP	T_RD1
;
T_RD2	LDA	#$05	; Set port C2 to 1
	STA	PADD
	JMP	T_RD1
;
; *******************************************************
; Test for sending data to PC via TTY (UART)
; -----------------------------------------------------------
;
; UART TX output line is at port PC1.
; We define a baud rate of ca. 340 = 34 CPS.
; To simulate this, we set the start bit counter to 200:
; - CNTL30 = 200
; - CNTH30 = 0
;
; You can define different baud rates by manually setting CNTL30/CNTH30
; and then starting at symbol T_SEND1.
; The byte to send to UART shall be provided in zero page address $0000.
; We use the UART output subroutine with symbol "OUTCH"
; This test routine is an endless loop and can only be stopped by pressing "ST"
; on the KIM-1 keyboard !
;
; A special note:
; ---------------
; This test routine does not always work correctly in the KIM-1 keyboard Single Step mode,
; because the 8255 PIA ports will continuously be changed from control word
; CW8255_1 to CW8255_2 and by this the TTY (UART) Port C1 will always be reset
; to 0, which represents a start bit.
; Depending on the bit status, the routine sometimes give correct readings on
; the UART serial input of the PC, sometimes not.
; See additional comments before symbol GETCH.
;
; However, when working with the KIM-1 keyboard WITHOUT the Single Step mode,
; the test function should give correct results on the PC.
;
; Basically the TTY (UART) functionality works correct,
; when My-KIM ist set to the TTY mode (bridge: PA0 is connected to ROW3 of 74145).
; In this mode, the control word of the 8255 PIA is never changed.
; 
; *******************************************************
;
T_SEND	LDA	#200
	STA	CNTL30
	LDA	#0
	STA	CNTH30
;
T_SEND1	LDA	$00
	JSR	OUTCH	; Send the byte in accu
	JMP	T_SEND1
;
; *******************************************************
; Test for receiving data from the UART sent by the PC.
; -----------------------------------------------------
; UART RX input is at port PC7.
; We measure at the first received data byte the duration of the start bit,
; which is saved in CNTL30/CNTH30.
; Therefore, the first transmitted byte shall have in bit position 0 a 1 to
; clearly identify the start bit length. A suitable byte is the rubout (DEL) byte,
; which is defined as $7F. The rest of this byte is ignored.
;
; We use the UART input subroutine with symbol "GET1"
;
; We store the first 256 received bytes to address $0200 and increasing.
; Having done this, we go back to KIM-1 keyboard mode.
; *******************************************************
;
T_REC	LDA	#$FF	; Define start bit count high byte
	STA	CNTH30
	LDA	#$F8	; We start here with 8 counts less than 0
			; to compensate for overhead
;
; When receiving the first byte from the UART, we measure the duration of the
; start bit time, which is saved in CNTL30 / CNTH30.
; The source code is copied from label DETCPS (DETect Characters Per Second)
;
T_REC1	LDY	SCD	; Read port C
	BMI	T_REC1	; branch as long as bit 7 is 1 --> start bit not started
; 
T_REC2	CLC		; Here starts the loop to 'measure' the start bit time
	ADC	#$01
	BCC	T_REC3
	INC	CNTH30	; increment the high byte of the start bit counter
T_REC3	LDY	SCD	; we use LDY, because we do not want to destroy the accu
	BPL	T_REC2	; branch as long as bit 7 is 0 --> start bit not ended
;
; Start bit has ended here
;
	STA	CNTL30	; Save the low byte of the start bit counter
	LDX	#$08
	JSR	GET5	; Get the remaining 8 bits of the byte from UART and forget them
;
; Now we receive 256 bytes and save them in $0200 and ascending
;
	LDX	#$00
T_REC4	STX	TMPX	; Save X-register
	LDX	#$08	; We need this X value, because we start with GET1
	JSR	GET1	; received byte is now in accu
			; Note: GET1 is endless loop until a start bit is detected
	LDX	TMPX	; Restore X-register, although already done in GET1
	STA	$0200,X	; store received byte in page 2 at ascending index positions
	INX
	BNE	T_REC4
;
; Back to KIM keyboard mode
;
	LDA	#$00
	STA	POINTL
	STA	POINTH
	JMP	START
;
; ===============================================================================
;
; ---------------------------------------------------------------- PAGE 15 ------
;
;
;
;
;		666666   555555   333333   000000
;		6        5             3   0    0
;		6        5             3   0    0
;		666666   555555   333333   0    0
;		6    6        5        3   0    0
;		6    6        5        3   0    0
;		666666   555555   333333   000000
;
;
;		         000000   000000   222222
;		         0    0   0    0        2
;		------   0    0   0    0        2
;		------   0    0   0    0   222222
;		------   0    0   0    0   2
;		         0    0   0    0   2
;		         000000   000000   222222
;
;
;
;
; COPYRIGHT
; MOS TECHNOLOGY, INC
; DATE OCT 13 1975 REV E
;
; KIM	: TTY INTERFACE
;	: KEYBOARD INTERFACE
;	: 7 SEG 6 DIGIT DISPLAY
;
;
; TTY Commands:
;	G	GOEXEC, small letter also works, see subroutine SCAN
;	CR	OPEN NEXT CLELL
;	LF	OPEN PREV. CELL
;	.	MODIFY OPEN CELL
;	SP	OPEN NEW CELL (space character)
;	L	LOAD Paper Tape (OBJECT FORMAT)
;	Q	DUMP Paper Tape FROM OPEN CELL ADDR TO EAL/EAH
;	RO	RUB OUT KEY ($7F) - RETURN TO START (KIM)
;		(ALL ILLEGAL CHARS ARE IGNORED)
;
;	Note: Paper Tape format is a kind of Intel Hex File format, but not identical.
;	The format is described in the "KIM-1 User Manual" on appendix pages F1 and F2.
:
; KIM-1 KEYBOARD Commands:
;	ADDR	(AD) SETS MODE TO MODIFY CELL ADDRESS
;	DATA	(DA) SETS MODE TO MODIFY DATA IN OPEN CELL
;	STEP	(+) INCREMENTS TO NEXT CELL
;	RST	(RS) SYSTEM RESET
;	RUN	(GO) GOEXEC
;	STOP	(ST) Stop a user program
;	PC	(PC) DISPLAY Program Counter and Memory Content of stoped user program
;
;	CLOCK IS NOT DISABLED IN SIGMA 1 (only god knows, what SIGMA 1 means !)
;
; The following subroutines are available for TTY (UART) operations:
;
; - GETCH: Get a byte from the UART
; - OUTCH: Send a byte to the UART
;
; These are also used by Microsoft BASIC, Vers. KB9, 1977.
; These address vectors must be set in the KB9 bin file, otherwise MS BASIC will not run !
; The 
;
; ===============================================================================
;
;
; ---------------------------------------------------------------- PAGE 16 ------
	*=$EC00		; In KIM-1 this was address $1C00
;
; This is the procedure for the Single Step Mode, when a user program has
; fetched an Op-code from RAM, initiated by a NMI interrupt
;
SAVE    STA     ACC	; KIM ENTRY VIA hardware interrupt NMI and IRQ (also BRK opcode)
	PLA
	STA     PREG
SAVE1   PLA		; KIM ENTRY VIA JSR (ACCUMULATOR LOST)
	STA     PCL	; store program counter also here to allow to recall by key "PC"
	STA     POINTL	; store here to show PC at display
	PLA
	STA     PCH	; store program counter also here to allow to recall by key "PC"
	STA     POINTH	; store here to show PC at display
SAVE2   STY     YREG
	STX     XREG
	TSX
	STX     SPUSER
; ############## CHANGE BEGIN #############################
;
; We do the INITS function ONLY when we are in the Keyboard mode, but NOT in the TTY mode.
; The reason is, that the INITS procedure changes the control word of the 8255 I/O IC.
; Setting the control word will set all output ports to NULL, which is
; equivalent with an UART serial start bit, hence the PC IDE will receive garbage bytes.
; We do this by checking, whether we are in the TTY mode.
;
	LDA	#$01	; Check whether we are in TTY mode
	BIT	SAD
	BEQ	SAVE3	; Jump when in TTY mode
	JSR	INITS	; When in keyboard mode, do the normal INITS function
	JMP     START
;
SAVE3	LDX	#$01	; here we are in TTY mode
	STX	MODE	; Set keyboard mode to Address type
	CLD		; Clear decimal mode
	SEI		; Enable interrupts
	JSR     CRLF
	JMP	START1	; We use modified START, with CRLF and print next op-code
; ############## CHANGE END #############################
;
; *******************************************************
; KIM-1 operating system reset and interrupt vectors:
;
;	- NMIT --> By an NMI interrupt, which is generated by:
;	           * either in single step keyboard mode,
;	             see IC U26 pin 6 open collector NAND gate output, driven by the
;	             SYNC signal and the 74LS145 address decoder output line "07" (pin 9)
;	           * or by a low pulse, generated by the STOP monoflop (U25 ST key pressed)
;	           Details on handling this interrupt is shown in KIM-1 Hardware Manual, page 27
;	           NMI is edge sensitive (falling endge at NMI input)
;	- IRQT --> By an IRQ interrupt: before using it, set the processor I flag to 0,
;	           which is set to 1 at the end of subroutine INITS
;	           IRQ is NOT edge sensitive !
;	           Details on handling this interrupt is shown in KIM-1 Hardware Manual, page 26
;	- RST  --> By a RESET, i.e. power ON or a low pulse,
;	           generated by the RESET monoflop (U25, RS key pressed)
;
; Details see page 26 of KIM-1 User Manual.
; *******************************************************
;
NMIT    JMP     (NMIV)	; NON MASKABLE INTERRUPT TRAP, is set in INITS to $EC00 = SAVE
IRQT    JMP     (IRQV)	; INTERRUPT TRAP, is set in INITS to $EC00 = SAVE
;
; ********************
; KIM Entry via Reset
; ********************
;
RST	LDX     #$FF	; Set stack pointer
	TXS
	STX     SPUSER
	LDA	#$00	; Set 7-Segment LED Display to address $0000
	STA	POINTL
	STA	POINTH
;
; After RESET (RS key) the LED display shall now show "0000 xx"
;
; ############## CHANGE BEGIN ###########################
;	JSR     INITS
	JSR     INITSA	; Initiation of entire system, also setting audio tape defaults
; ############## CHANGE END #############################
;
; PREPARE TO DETERMINE CHARACTERS PER SECOND IN CASE OF TTY,
; i.e. measure time of start bit
;
DETCPS  LDA     #$FF	; COUNT STARTBIT
	STA     CNTH30	; ZERO CNTH30, TEST FIRST KEYBOARD OR TELEPRINTER (TTY)
;
; We check here port A0, whether there is a bridge for TTY operation.
; (Port A0 <--> ROW3 of 74LS145)
; ROW3 signal of output line number 3 (pin 4) of 74LS145 is pulled to low level
;
; Old source code
; ---------------
;	LDA     #$01	; MASK BIT 0
;DET1    BIT     SAD     ; TEST FOR TTY (TELETYPE), read TTY/KB bridge
;	BNE     START   ; Branch to START, because Bridge is not present, hence not TTY mode
;	BMI     DET1    ; Yes we are in TTY mode, check immediately
;			; whether a start bit (low level) is present at PA7
;			; If not, wait for begin of start bit
;
; New source code
; ---------------
; Check on TTY mode (PA0=0). If not, continue with label START
;
	LDA	#$01	; mask bit 0 of port A
	BIT	SAD	; check if still in TTY mode, bridge PA0 <--> ROW3 of 74LS145
	BNE	START	; If not, continue with START
;
; Wait for TTY start bit (low level) at port C7.
; We are awaiting a RubOut ($7F) character, for checking the length of the start bit.
; It MUST be a character, where bit 0 is high level to clearly measure the 
; start bit time, which has a low level. This means, any character can be used
; instead of the RubOut key, which has a 1 in bit 0.
; See also TTY (UART) graphics above subroutine GETCH.
;
DET1	LDA	SCD	; get port C and analyse bit 7
	BMI	DET1	; wait for start bit (0 Volt) as long as C7 is high level
;
; We got the start of the bit (low level), now count the duration of the start bit
;
	LDA     #$F8	; Start bit detected, we start here with 8 counts less than 0
			; to compensate for overhead
;
; Here we start the loop to 'measure' the start bit duration.
; Neglecting the "INC CNTH30" instruction, the loop takes 14 us.
;
; Example 1:
; With a baud rate of 240, the loop will be performed 297 times.
; 297 * 14 us = 4.158 ms = 240.5 baud, which is very correct -> 0.02% error.
; Because we started with -8 counts, the counter will show 289 = $0121.
;
; Example 2:
; With a baud rate of 340, the loop will be performed 210 times.
; 210 * 14 us = 2.940 ms = 340.13 baud, which is very correct -> 0.03% error.
; Because we started with -8 counts, the counter will show 202 = $00CA.
;
; Example 3:
; With a baud rate of 440, the loop will be performed 160 times.
; 160 * 14 us = 2.240 ms = 446.42 baud, which is nearly correct -> 1.4% error.
; Because we started with -8 counts, the counter will show 152 = $009A.
;
; Example 4:
; With a baud rate of 540, the loop will be performed 132 times.
; 132 * 14 us = 1.848 ms = 541.12 baud, which is correct -> 0.2% error.
; Because we started with -8 counts, the counter will show 124 = $007C.
;
; Example 5:
; With a baud rate of 800, the loop will be performed 89 times.
; 89 * 14 us = 1.246 ms = 802.56 baud, which is nearly correct -> 0.3% error.
; Because we started with -8 counts, the counter will show 81 = $0051.
;
; Example 6:
; With a baud rate of 1200, the loop will be performed 59 times.
; 59 * 14 us = 0.826 ms = 1210.65, which is mostly correct -> 0.9% error.
; Because we started with -8 counts, the counter will show 51 = $0033.
;
; I tried 2400 baud, but this failed with reading rubbish with a loop counter of 30.
; And 2400 baud is equal to 240 Characters Per Second (CPS).
;
; This is not a disaster, because the KIM-1 User Manual states on page 17:
;   "... since the KIM-1 system automatically adjusts for a wide variety of
;    data rates (10 CPS, 15 CPS, 30 CPS, etc.)"
;
; Hence with 1200 baud = 120 CPS in example 6 we are far beyond of this,
; what KIM-1 states to do.
;
; Important:
; ----------
; The above performed test only consider the correct reception of data by KIM
; via the TTY (UART) from PC.
; However, when KIM is transmitting data with baud rates higher than 700 to
; the PC via TTY (UART), the baud timing is not accurate enough, because
; the PC will show rubbish on the data transmitted by KIM.
; It is therefore suggested, to define the maximum possible baud rate to 600,
; which is still far beyond that, what the original KIM-1 could do !!
;
; Here starts the loop for measuring the start bit time.
; Neglecting the "INC CNTH30" instruction, the loop takes 14 us.
;
DET3    CLC 
	ADC     #$01	; Add 1 to the low counter
	BCC     DET2    ; if no carry, then branch
	INC     CNTH30	; increment the high byte of the start bit counter
DET2    LDY     SCD	; Read port C7, we use LDY, because we do not want to destroy the accu
	BPL     DET3    ; Continue with DET3, if still low level
;
; Start bit ended.
;
	STA     CNTL30	; save the low counter in CNTL30
;
; Get rest of the byte, but we do not analyse it
;
	LDX     #$08
	JSR     GET5
; ---------------------------------------------------------------- PAGE 17 ------
;
; MAKE TTY/KB SELECTION (KB = Keyboard)
;
; Explanation on detecting TTY or Keyboard mode:
; By reading Port A0 we identify, in which mode we are currently:
;
; - PA0 = 0 --> TTY mode
; - PA0 = 1 --> Keyboard mode
;
; When we are in TTY mode and switch back to keyboard mode, i.e. removing TTY/KB bridge,
; the 7-Segment LEDs will not immediately light up, because the TTY mode
; is still running in an internal loop for receiving data via TTY (UART),
; see subroutine GETCH and further on at label READ.
; Only when we then press a key on the TTY, then the processor will
; jump to the KIM-1 keyboard mode and the LEDs light up.
;
; When you switch back to TTY mode, you can directly continue in this mode,
; without any interruption or the need to restart the system (Rubout key).
;
; However, starting in the KIM-1 keyboard mode and then switching to the
; TTY mode does not work, because the processer has not performed the
; TTY start bit time measuring code at symbol DET1.
; To use the TTY mode you have to press the RS button and the PC has to send a
; Rubout character ($7F) to begin the start bit time measuring procedure.
; This is a convenient way and all the data in the RAM is still preserved.
;
; Important message:
; The original KIM-1 softawre always start the START procedure by calling INIT1,
; which among others also sets the 8255 ports. This however gives
; unpredictable results on the 7-segment LEDs and also in TTY mode on the UART.
; In My-KIM we never change the ports direction, with the only exception in the
; SCAND subroutine, where we temporarily change port A to input to read
; the My-KIM keyboard and then immediately switch back to output to drive
; the 7-segment LEDs.
; We therefore skip here the subroutine INIT1 !!!
;
; ############## CHANGE BEGIN ###########################
;START	JSR     INIT1
; ############## CHANGE END #############################
START	LDA     #$01	; READ TTY/KB bridge (Port A0 <--> ROW3 of 74LS145)
	BIT     SAD	; if PA0 is low level, then TTY mode
	BNE     TTYKB	; branch to TTYKB symbol, entering the Keyboard mode
	JSR     CRLF	; we are in TTY mode: PRINT CARRIAGE RETURN / LINE FEED
; ############## CHANGE BEGIN ###########################
;	LDX     #$0A
	LDX     #$04	; see changed table symbol "TOP"
; ############## CHANGE END #############################
	JSR     PRTST   ; PRINT 'KIM\r\n'
START1	JMP     SHOW1	; Show address in POINTL/H an show its content, e.g. "0000 xx"
;
CLEAR   LDA     #$00
	STA     INL     ; CLEAR TTY INPUT BUFFER
	STA     INH
READ    JSR     GETCH   ; GET CHAR FROM TTY and save temporarily in INL/H
	CMP     #$01    ; 1 HAS NO MEANING FOR TTY
	BEQ     TTYKB   ; 1 = KB-MODE -> continue with keyboard mode
	JSR     PACK    
	JMP     SCAN	; Analyse the received byte
;
; MAIN ROUTINE FOR KEYBOARD
; AND LED DISPLAY
;
TTYKB   JSR     SCAND   ; WAIT UNTIL NO KEY PRESSED
	BNE     START   ; IF PRESSED, WAIT AGAIN ->
TTYKB1  LDA     #$01    ; CHECK Keyboard/TTY MODE
	BIT     SAD     ; TTY mode ?
	BEQ     START   ; YES ->
	JSR     SCAND   ; WAIT FOR keyboard KEY...
	BEQ     TTYKB1  ; NO KEY ->
	JSR     SCAND   ; DEBOUNCE KEY
	BEQ     TTYKB1  ; NO KEY ->
;
; The KIM-1 keyboard has the following keys:
;
; 'GO' 'AD' 'DA' 'PC' '+' '0-9' 'A-F'
; Here we check, which one has been pressed.
;
GETK    JSR     GETKEY
	CMP     #$15    ; >= $15 = ILLEGAL
	BPL     START   ; YES ->
	CMP     #$14
	BEQ     PCCMD   ; Key "PC" --> DISPLAY PROGRAM COUNTER
	CMP     #$10
	BEQ     ADDRM   ; Key "AD" --> Set Keyboard to ADDRESS MODE
	CMP     #$11
	BEQ     DATAM   ; Key "DA" --> Set Keyboard to DATA MODE
	CMP     #$12
	BEQ     STEP    ; Key "+" --> Increment address
	CMP     #$13
	BEQ     GOV	; Key "GO" --> EXECUTE PROGRAM
DATA    ASL	A	; ONE OF THE BUTTONS {0-9, A-F} HAS BEEN PUSHED
	ASL	A	; SHIFT CHAR INTO HIGH ORDER NIBBLE
	ASL	A     
	ASL	A
	STA     TEMP    ; STORE FOR DATA MODE
	LDX     #$04
DATA1   LDY     MODE    ; PART OF ADDRESS ?
	BNE     ADDR    ; YES ->
; ---------------------------------------------------------------- PAGE 18 ------
	LDA     (POINTL),Y   ; GET DATA
	ASL     TEMP
	ROL	A       ; MSB-TEMP = MSB-KEY -> A
	STA     (POINTL),Y   ; STORE NEW DATA
	JMP     DATA2
;
ADDR    ASL	A       ; TEMP NOT NEEDED HERE
	ROL     POINTL  ; MSB-KEY -> POINTL
	ROL     POINTH  ; POINTL -> POINTH
DATA2   DEX             ; 4 TIMES = COMPLETE NIBBLE ?
	BNE     DATA1   ; NO ->
	BEQ     DATAM2  ; -> ALWAYS do a branch !
;
ADDRM   LDA     #$01	; SWITCH TO ADDRESS MODE
	BNE     DATAM1  ; -> ALWAYS do a branch
;
DATAM   LDA     #$00
DATAM1  STA     MODE
DATAM2  JMP     START
;
STEP    JSR     INCPT	; Key "+" INCREMENT ADDRESS ON DISPLAY
	JMP     START
;
GOV     JMP     GOEXEC	; Key "GO"
;
; DISPLAY Program counter BY MOVING PC TO POINTL/H
; Key "PC" pressed, restore program counter at time of interrupt, e.g by "ST" key
;
PCCMD   LDA     PCL
	STA     POINTL
	LDA     PCH
	STA     POINTH
	JMP     START
;
; ############## CHANGE BEGIN ###########################
;
; The functions for Paper Tape Dump (DUMP) and Load (LOAD) have been deleted
;
; ############## CHANGE END #############################
; -----------------------------------------
; TTY mode: User has pressed SPACE bar 
; - user has entered a new address.
; - show this memory location and its content to TTY
; -----------------------------------------
SPACE   JSR     OPEN    ; Copy content in I/O buffer to POINTL/H --> open new cell
; -----------------------------------------
; Send CRLF and show memory location in POINTL/H and content to TTY
; -----------------------------------------
SHOW    JSR     CRLF
; -----------------------------------------
; Show memory location in POINTL/H and its content to TTY
; -----------------------------------------
SHOW1   JSR     PRTPNT	; Print the address in POINTL/H to TTY
	JSR     OUTSP   ; Print a space to TTY
	LDY     #$00
	LDA     (POINTL),Y      ; Get byte in the address
	JSR     PRTBYT	; Print byte to TTY
	JSR     OUTSP   ; Print space
	JMP     CLEAR	; clear the I/O buffer and wait for next input from TTY
; -----------------------------------------
; TTY mode: User has pressed CR
; -----------------------------------------
RTRN    JSR     INCPT   ; NEXT ADDRESS
	JMP     SHOW
; -----------------------------------------
; TTY mode: User has pressed 'G'
; -----------------------------------------
GOEXEC  LDX     SPUSER  ; USER DEFINED STACK
	TXS		; START A PROGRAM AT DISPLAYED ADDRESS.
	LDA     POINTH  ; PROGRAM RUNS FROM
; ---------------------------------------------------------------- PAGE 21 ------
	PHA             ; DISPLAYED ADDRESS
	LDA     POINTL
	PHA
	LDA     PREG    ; USER DEFINED FLAG REGISTER
	PHA
	LDX     XREG
	LDY     YREG
	LDA     ACC
	RTI             ; START PROGRAM 
;
; *******************************************************
; Analyse the character keys, which come from TTY.
;
; Very important:
; ---------------
; The procedure SCAN will be called, when in TTY mode and user has typed
; something in below symbol READ:
;
;	JSR 	GETCH	; get user typed character in accu
;	CMP	#$01
;	BEQ	TTYKB --> continue with KIM-1 keyboard mode
;	JSR	PACK
;	JMP	SCAN
;
; But before the SCAN procedure is called, the subroutine PACK will be called,
; doing the following:
;
; - The user typed character is in accu
; - If accu is '0'...'9' or 'A'...'F', shift it into INL/H as 4-bit nibbles
;   in form of 0x0 to 0xF and set accu = $00
; - For all other typed characters, PACK does not change the accu
;
; The consequence is:
; -------------------
; The procedure SCAN will only be performed correctly, when user has typed in
; characters others than '0'...'9' or 'A'...'F'.
;
; Let's assume the following case:
; - user has typed 'B'
; - 'B' will be saved in INL/H and accu is set to $00
; - When in SCAN we will check on typed character 'B' = 0x42 we do:
;
;	CMP 	#'B'	--> Processor does (A-M) = 0x00 - 0x42 = 0xBE
;	BEQ	To my Vector
; 
; *******************************************************
;
SCAN    CMP	#$20	; ' ' SPACE bar key: OPEN NEW CELL
	BEQ	SPACE
	CMP	#$7	; 'RUB OUT' key: RESTART KIM
	BEQ	STARTV
	CMP	#$0D	; 'CR' key: NEXT CELL
	BEQ	RTRN
	CMP	#$0A	; 'LF' key: PREV CELL
	BEQ	FEED
	CMP	#'.'	; '.' key: MODIFY CELL
	BEQ	MODIFY
	CMP	#'G'	; 'G' key: EXEC PROGRAM
	BEQ	GOEXEC
; ############## CHANGE BEGIN ###########################
;
	CMP	#'W'	; 'W' key: Write (Dump) data / program onto audio tape
	BEQ	DUMPV
	CMP	#'L'	; 'L' key: Load data / program from audio tape
	BEQ	LOADV
;
; Added here functions to transmit and receive data via UART to and from PC
;
	CMP	#'T'	; 'T' key: Transmit data via TTY to PC (UART)
	BEQ	TFILEV
	CMP	#'R'	; 'R' key: Receive data via TTY from PC (UART)
	BEQ	RFILEV
;
; Added here function load MS BASIC KB9 via UART in TTY mode
;
	CMP	#'M'	; 'M' key: Load MS BASIC
	BEQ	BASICV	; We will never jump back from BASIC to KIM operating system
;
; Added here function to show memory content via UART in TTY mode
;
	CMP	#'S'	; 'S' key: List  content SAL/H - EAL/H
	BEQ	LIST_M
;
; Added here function to print software version via UART in TTY mode
;
	CMP	#'V'	; 'V' key: Print software version
	BEQ	PSWVV
;
; ############## CHANGE END #############################
;
	JMP     READ	; IGNORE ILLEGAL CHAR
;
; -----------------------------------------
; TTY mode: User has pressed Rubout key
; -----------------------------------------
STARTV	JMP     START	; KIM-1 restart
; ############## CHANGE BEGIN ###########################
; -----------------------------------------
; TTY mode: User has pressed 'W' or 'L' key (Write / Load audio tape)
; -----------------------------------------
DUMPV	JSR     DUMPT_BODY	; Dump to audio tape
	JMP	SHOW	; Back to receiving new commands via TTY
LOADV	JSR     LOADT_BODY	; Load from audio tape
	JMP	SHOW	; Back to receiving new commands via TTY
;DUMPV	JMP     DUMPT	; Dump to audio tape
;LOADV	JMP     LOADT	; Load from audio tape
; -----------------------------------------
; TTY mode: User has pressed 'T' or 'R' key (Transmit / Receive File Data)
; -----------------------------------------
TFILEV	JSR	TFILE	; Transmit file data via TTY to PC (UART)
	JMP	SHOW	; Back to receiving new commands via TTY
RFILEV	JSR     RFILE	; Read file data via TTY from PC (UART)
	JMP	SHOW	; Back to receiving new commands via TTY
; -----------------------------------------
; TTY mode: User has pressed 'M' key (Load Microsoft BASIC)
; -----------------------------------------
BASICV	JMP     LBASIC	; Load MS BASIC via TTY from PC (UART) and start BASIC,
			; we will never return from BASIC back to KIM operating system
; -----------------------------------------
; TTY mode: Print software version
; -----------------------------------------
PSWVV	JSR	PSWV	; Print Software version to TTY as NULL terminated string
	JMP	SHOW	; Send CRLF and show current address and its value via TTY
; -----------------------------------------
; TTY mode: Show memory content SAL/H - EAL/H
; -----------------------------------------
LIST_M	JSR	DUMP	; Show memory content based on SAL/H - EAL/H
	JMP	SHOW	; Send CRLF and show current address and its value via TTY
;
; ############## CHANGE END #############################
;
FEED    SEC
	LDA     POINTL  ; DECREMENT POINTL/H
	SBC     #$01
	STA     POINTL
	BCS     FEED1
	DEC     POINTH
FEED1   JMP     SHOW
;
MODIFY  LDY     #$00    ; GET CONTENTS OF INPUT BUFFER
	LDA     INL     ;  INL AND STORE IN LOCATION
	STA     (POINTL),Y      ;  SPECIFIED BY POINT
	JMP     RTRN
; ---------------------------------------------------------------- PAGE 22 ------
;
; *******************************************************
; SUBROUTINES FOLLOWS
; *******************************************************
;
; SUBROUTINE TO PRINT POINT = ADDRESS
;
PRTPNT  LDA     POINTH
	JSR     PRTBYT
	JSR     CHK
	LDA     POINTL
	JSR     PRTBYT
	JSR     CHK
	RTS
; -----------------------------------------
; Print character string in table "TOP"+X to TTY
; -----------------------------------------
; ############## CHANGE BEGIN ###########################
;CRLF    LDX     #$07    ; OUTPUT <CR> AND <LF>
CRLF    LDX     #$01    ; OUTPUT <CR> AND <LF>	; changes because of changed symbol "TOP"
; ############## CHANGE END   ###########################
PRTST   LDA     TOP,X
	JSR     OUTCH
	DEX             ; EVERYTHING ?
	BPL     PRTST   ; NO ->
PRT1    RTS
; -----------------------------------------
; Print Byte in Accu as two ASCII characters to TTY
; -----------------------------------------
PRTBYT  STA     TEMP    ; SAVE A
	LSR	A       ; SHIFT Accu 4 TIMES to the right
	LSR	A
	LSR	A
	LSR	A
	JSR     HEXTA   ; Convert low nibble Hex to ASCII and print
	LDA     TEMP
	JSR     HEXTA   ; Convert low nibble Hex to ASCII and print
	LDA     TEMP    ; Restore Accu
	RTS
;
HEXTA   AND     #$0F    ; Mask low nibble
	CMP     #$0A    ; > 9 ?
	CLC
	BMI     HEXTA1  ; NO ->
	ADC     #$07    ; Yes: >9, add 7 to achieve A ... F
HEXTA1  ADC     #$30    ; Add 30 to convert to ASCII
	JMP     OUTCH   ; Print ASCII character
; -----------------------------------------
; GET CHAR FROM TTY IN A
; RETURN FROM SUB WITH CHAR IN A
; X IS PRESERVED and
; Y RETURNED = $FF, because of used DELAY and DEHALF subroutines
; -----------------------------------------
;
; ---------------------------------------------------------------- PAGE 23 ------
;
; TTY (UART) protocol is 8N1 and resulting signal level at port C7 is shown below:
;
; Example byte = 75 = $4B = 01001011b, least significant bit 0 transmitted first
;
;             1  1  0  1   0  0  1   0  = $4B
; +5V ----  --------  ----      ----   ----
;        |  |      |  |  |      |  |   |  | Next Byte
;  0V    ----      ----  --------  ----   -------
;       Start 0  1  2  3   4  5  6   7  Stop  Start
;
; If only one byte is transmitted, the signal level after the stop bit remains high !
;
; Read port A to check TTY/KB bridge between PA0 <--> ROW3, see page 18 KIM-1 User Manual
; When no bridge is present, PA0 is not pulled down by 74LS145 ROW3 --> NOT TTY mode.
; The reason is, that PA0 is connected to input of pin 1 of 7406 (U17A), pulling PA0 up.
; We therefore can check it by the BIT op-code with mask $01 in accumulator:
; - when result is zero, we are in TTY mode.
; - when result is NOT zero, we are in KIM keyboard mode.
;
; The KIM-1 software uses the BIT instruction, which allows the following:
; - The addessed byte is logically AND with the mask in the accumulator
; - Depending on the AND instruction, the Z flag is set or not set
; - This BIT instruction does NOT affect the content of the accumulator
; - In addition, the BIT instruction, independent of the mask in the accumulator,
;   is testing bit 7 (Negative) and bit 6 (Overflow) of the addressed byte
;   and places the result to the N and V flag of the status register.
;
; With this BIT procedure the KIM-1 software can test simultaneously three cases:
; - Check, whether A^M is zero or not		--> Z flag
; - Check, whether A^M is negative or positive	--> N flag (bit 7)
; - Check, whether A^M is has overflow or not	--> V flag (bit 6)
;
; Having this not in mind, the original KIM-1 software cannot be understood !
; In particular the two below instructions after the BIT instruction:
;
;		LDA     #$01    ; MASK BIT 0
;	DET1    BIT     SAD     ; TEST FOR TTY (TELETYPE), READ JUMPER
;		BNE     START   ; NO -> check Z flag
;		BMI     DET1    ; NO STARTBIT, WAIT FOR IT -> check N flag
;
; In My-KIM we have a different situation:
; - checking the TTY/KB mode is done by testing port pin PA0
; - checking the TTY input signal is done by reading port pin PC7
;
; GETCH is the TTY input routine, also used by Microsoft BASIC, (KB9), 1977
;
; ############## CHANGE BEGIN #########################
;
GETCH   STX     TMPX	; Save X-register
	LDX     #$08	; We have to read 8 bits
	LDA     SAD	; Read port A
	AND     #$01	; Mask bit 0 --> check if still in TTY mode
	BNE     GET6	; If not in TTY mode (bit 0 = 1), do nothing and return
			; with $01 in accumulator
;
; Wait for begin of start bit --> port pin C7 shall become 0
;
GET1    LDA     SCD	; Read port C
	BMI     GET1	; wait for begin of start bit (branch if C7 is still 1)
;
; Delay 1 + 1/2 bit time, so that we start to read the first bit in its mid position
;
	JSR     DELAY	; DELAY 1 BIT (skip the start bit time)
GET5    JSR     DEHALF	; DELAY 1/2 BIT TIME, to read first bit in its mid position
;
; Start to read the 8 bits
;
GET2    LDA     SCD	; Read port C
	AND     #$80	; MASK BIT 7
	LSR     CHAR	; SHIFT LAST RESULT to the right
	ORA     CHAR	; OR it with new bit
	STA     CHAR	; and STORE IT AGAIN
	JSR     DELAY	; Delay 1 bit to read the next bit of the byte
	DEX
	BNE     GET2	; branch for NEXT BIT to read
	JSR     DEHALF	; DELAY 1/2 BIT TIME to be sure to reach the begin of the stop bit
			; (high level), before we can start to scan for the next
			; start bit (low level), see TTY signal diagram above.
;
; ############## CHANGE END #########################
;
; We have done it
;
	LDX     TMPX	; Restore X-register
	LDA     CHAR	; place the received byte in accumulator and return
GET6    RTS
;
; INITIALIZATION of Modes and Ports
;
; ############## CHANGE BEGIN #########################
;
; Initiation of system and modes, including setting audio tape dump defaults
;
; We initiate the audio tape dump start and end addresses (SAL/H EAL/H)
; as well as the ID value to default values in RAM:
;
; - SAL/H = $0200
; - EAL/H = $0280	--> hence allowing to save half of page 2, Note: end address 
; - ID = $01		    must be 1 byte higher than the last byte to send !
;
INITSA	LDA	#$00	; Set audio tape dump start address
	STA	SAL
	LDA	#$02
	STA	SAH
	LDA	#$80	; Set audio tape dump end address
	STA	EAL
	LDA	#$02
	STA	EAH
	LDA	#$01	; Set program ID to 1
	STA	ID
;
; ############## CHANGE END ###########################
;
; Nominal Initiation of system and modes, also called from SAVE procedure
;
INITS   LDX     #$01    ; SET DISPLAY TO ADDRESS MODE
	STX     MODE
; ############## CHANGE BEGIN #########################
;
; Place the "SAVE" address ($EC00) to the NMIV ($7FFA+B) and IRQV ($7FFE+F) vectors in RAM.
; This is easier for the user, who else has always to manually enter this address
; to the NMIV vector after power ON, before he can use the "ST" = stop key,
; see KIM-1 User Manual, page 45/46.
;
; This correct NMIV vector is also necessary, before executing a user program in the
; single step (SST) mode, i.e. the SST slide switch is in position ON and
; then pressing the "GO" key, see page 46, item 7 of KIM-1 User Manual.
;
; It shall be noted, that the single step mode can only be performed for user programs,
; which are NOT located in the ROM address range $E000 - $FFFF,
; because of the specific hardware implementation by NANDing the K7 line of the 74LS145
; address selector with the SYNC line of the 6502 processor. The NAND output line
; is then connected via the SST slide switch in ON position to the
; NMI interrupt input line of the 6502 processor.
;
; Special remark 1:
; -----------------
; Here I violate (assault) the 74LS00 NAND gate output line (in My-KIM it is IC3B),
; because it should be an open collector NAND gate, as implemented in KIM-1 by
; IC U26 (input pin 4&5, output pin 6). Originally I used an 74LS38 IC with open collector,
; but it turned out, that this open collector output does not switch so quickly as required.
; I therefore decided to use a standard 74LS00 IC, whose output transistor is also
; capable of pulling down the NMI input line of the processor, which is connected to
; Vcc via a 3K3 resistor.
;
; Special remark 2:
; -----------------
; For the generation of the NMI interrupt in the single step mode the NAND gate IC3B
; (in KIM-1 it is U26 with output pin 6) is driven by two signals:
; - the SYNC signal of the 6502 processor and
; - the 74LS145 decoder output line 7 (pin 9) for selecting the ROM area
;
; The logic of this NAND gate is as follows:
; - When a non-ROM address is present (pin 9 of IC1=74LS145 is high) and
; - the SYNC signal becomes high because fetching an opcode
; then the output of the NAND gate shall generate a NMI interrupt by
; jumping to the KIM-1 Address $1C1C with label NMIT (in My-KIM it is address $EC1C).
;
; Because the 74LS145 has open collector outputs, it does not switch so fast
; as the standard TTL ICs with a totem pole transistor output network.
; In addition, the 74LS145 has a propagation delay time of up to 50 ns.
;
; The result is, that the SYNC high signal reaches the NAND gate ca. 40 ns earlier
; than the output of pin 9 of 74LS145. This creates for EVERY opcode fetch
; (independent of the current address) a small negative spike with a duration of 10-15 ns
; at the output of the NAND gate, hence the NMI interrupt is executed also
; when the processor is loading an opcode from the ROM -> this is not what we want !
; Because the NMI interrupt shall only be executed for non-ROM addresses !
;
; It was therefore necessary to delay the SYNC signal by adding two 74LS04 inverters
; as shown below:
;
;                  |\              |\
;    SYNC    =-----| o------+------| o---= delayed SYNC signal to NAND gate input
;    from          |/       |      |/       - in KIM-1 it is pin 5 of U26 (7438)
;    6502                  === 1 nF         - in My-KIM it is pin 5 of IC3B (74LS00)
;                           |
;                          ---
;
; The inserted capacitor of 1 nF delays the SYNC signal by appr. > 25 ns, which is
; sufficient to avoid the negative spike at the output of the NAND gate IC3B.
; With this design modification the 6502 processor runs absolutely stable
; in the single step mode !
;
; This however does not answer the question, why this did not happen in the
; original KIM-1 hardware design, although I used the same parts and the same schematics.
;
; Now we place the "SAVE" address ($EC00) to the NMIV ($7FFA+B) and IRQV ($7FFE+F) vectors in RAM.
;
	LDX	#<SAVE	; Take low byte of "SAVE" address
	STX	NMIV
	STX	IRQV
	LDX	#>SAVE	; Take high byte of "SAVE" address
	STX	NMIV+1
	STX	IRQV+1
;
; ############## CHANGE END ###########################
;
; ############## CHANGE BEGIN #########################
;INIT1   LDX     #$00
INIT1   LDX     #CW8255_1 ; Control word for: Port A = Input, Port B = Output,
;			; Port C = mixed (C0-C3 = Output; C4-C7 = Input)
; ############## CHANGE END ###########################
	STX     PADD    ; Set the 8255 control word
; ############## CHANGE BEGIN #########################
;
; We do not need the following instructions to set the port B separately,
; because already done by writing the 8255 control word CW8255_1
;	LDX     #$3F
;	STX     PBDD    ; PB0..PB5 = OUTPUT
;			; PB6, PB7 = INPUT
;
; But we have to set the two Port C output bits
; - PC0 = Tape output to 0
; - PC1 = TTY output to 1 (because TTY Start bit is level 0)
;
	LDA	#$00	; Control word to set Port C0 to 0 (Bit Set/Reset command)
	STA	PCDD	; Write to 8255 Port Control
	LDA	#$03	; Control word to set Port C1 to 1 (Bit Set/Reset command)
	STA	PCDD	; Write to 8255 Port Control
;
; ############## CHANGE END ###########################
;
; For Port B:
; Set 74LS145 open-colletor output line Q3 (row 3) to low to check either KB or TTY mode.
; In KIM-1 it was:
; 	TTY mode when expansion connector pins A-V & A-21 are connected.
; 	This connects 74LS145 open collector output Q3 with port PA0, acting as input.
; In My-KIM it is:
;	TTY mode, when JP2 switch of Key-Board is connected.
;
	LDX     #$07	; Define Byte to write to Port B
	STX     SBD
	CLD		; Clear decimal mode of processor
	SEI		; Do NOT allow any interrupt from IRQ (SEI = set interrupt disable flag to 1=TRUE)
	RTS		; Call CLI to allow interrupts at IRQ, see KIM-1 Hardware Manual, page 25.
			; However, this may be dangerous, because it could burn the 7-Segment LEDs !
;
; OUTPUT CHAR IN A TO TTY       At KIM-1 = $1E9E
; Transmission uses 8N1 format:
;	1 start bit (low level)
;	8 data bits (logical 0 = low level, logical 1 = high level)
;	no parity bit,
;	1 stop bit (high level)
;
OUTSP   LDA     #$20    ; PRINT SPACE
;
; OUTCH is the TTY output routine, also used by Microsoft BASIC, (KB9), 1977
;
OUTCH   STA     CHAR
	STX     TMPX
	JSR     DEHALF	; Delay 1/2 Bit time to ensure, that previous stop bit
			; is completed
; ############## CHANGE BEGIN #########################
;
; The My-KIM TTY (UART) output is at port C1.
; To set the 8255 output port C1 we use the following control words (Bit Set/Reset feature):
;	C1 to 0 --> Write $02 to PCDD (Control Word Register)
;	C1 to 1 --> Write $03 to PCDD (Control Word Register)
; Therefore some changes in the code.
;
; Send start bit: port C1 = 0
;
	LDA	#$02	; Port C1 = 0 => Start bit
	STA	PCDD	; Write to 8255 Port Control
	JSR     DELAY	; Delay 1 Bit time
;
; Loop over all 8 bits, least significant bit first
;
	LDX     #$08
OUT1	LSR     CHAR    ; shift byte to right, bit 0 then goes to carry bit
	BCS	OUT2	; if carry bit is 1, then jump
	LDA	#$02	; Carry = 0 -> Set Port C1 = 0
	STA	PCDD	; Write to 8255 Port Control
	JMP	OUT3
OUT2	LDA	#$03
	STA	PCDD	; Carry = 1 -> Set Port C1 = 1
; ---------------------------------------------------------------- PAGE 24 ------
OUT3	JSR     DELAY	; Delay 1 Bit time
	DEX             ; All bits done ?
	BNE     OUT1    ; No -> OUT1
;
; Send stop bit: port C1 = 1
;
	LDA	#$03	; Set Port C1 = 1
	STA	PCDD	; Write to 8255 Port Control
;
; ############## CHANGE END ###########################
	JSR     DELAY	; Delay 1 Bit time
	LDX     TMPX	; Restore x-register
	RTS
;
; DELAY 1 BIT TIME AS DETERMINED BY DETCPS
;
DELAY   LDA     CNTH30
	STA     TIMH
	LDA     CNTL30
DE2     SEC
DE4     SBC     #$01
	BCS     DE3     ; A <> $FF ->
	DEC     TIMH
DE3     LDY     TIMH    ; TIMH > 0 ?
	BPL     DE2     ; YES ->
	RTS
;
; DELAY HALF A BIT TIME
;
DEHALF  LDA     CNTH30
	STA     TIMH
	LDA     CNTL30
	LSR	A
	LSR     TIMH
	BCC     DE2
	ORA     #$80
	BCS     DE4     ; Always branch to DE4 !
;
; Subroutine to determine if key is depressed or condition of KB / TTY mode:
; - Key not depressed or TTY mode --> A = 0
; - Key depressed or KB mode      --> A not 0
;
AK      LDY     #$03    ; 3 ROWS
	LDX     #$01    ; SELECT 74LS145 OUTPUT Q0

ONEKEY  LDA     #$FF    ; INITIAL VALUE
;
AK1     STX     SBD     ; ENABLE OUTPUT = SELECT ROW
	INX
	INX             ; PREPARE FOR NEXT ROW
	AND     SAD     ; A := A && (PA0..PA7)
	DEY             ; ALL ROWS ?
	BNE     AK1     ; NO ->
; ---------------------------------------------------------------- PAGE 25 ------
; Restore Port B for detecting KB / TTY mode only
;
	LDY     #$07
	STY     SBD     ; SELECT 74LS145 OUTPUT line 3
;
	ORA     #$80    ; MASK BIT 7 OF A
	EOR     #$FF    ; IF A STILL IS $FF -> A := 0
	RTS
;
; OUTPUT TO 7-SEGMENT-DISPLAY
;
; We have 6 LED digits (4 for Address, 2 for Data Byte).
; Each digit will be enabled by one of the 6 open collector outputs of the 74LS145
; 1 out of 10 decoder IC with:
;
;	4567 89 = 74LS145 decoder output line numbers
;	FFFF FF = left 4 digits = address, right two digits = byte value in address
;
; The selection is done by the inputs A,B,C,D of this IC, where Port B is connected to
; these four inputs as follows:
;	B1 -> A
;	B2 -> B
;	B3 -> C
;	B4 -> D
;
; Examples:
;  Enable high nibble of high Address Byte: Decoder = 4 --> Port B = $08 --> $09 (*)
;  Enable low  nibble of high Address Byte: Decoder = 5 --> Port B = $0A --> $0B
;  Enable high nibble of low  Address Byte: Decoder = 6 --> Port B = $0C --> $0D
;  Enable low  nibble of low  Address Byte: Decoder = 7 --> Port B = $0E --> $0F
;  Enable high nibble of Byte value:        Decoder = 8 --> Port B = $10 --> $11
;  Enable low  nibble of Byte value:        Decoder = 9 --> Port B = $12 --> $13
;
; (*) Because in KIM-1 the port B0 pin has always to stay at high level,
; the value, which will be written to port B is increased by 1,
; e.g. $08 -> $09 for a decoder output of 4.
; Therefore the display loop starts with an X-Index value of $09 and will be
; increased with each loop by 2 (two times INX in subroutine CONVD), see below.
;
; But in this My-KIM implementation, port pin B0 is not used !
;
SCAND   LDY     #$00     ; POINTL/POINTH = ADDRESS ON DISPLAY
	LDA     (POINTL),Y      ; GET DATA FROM THIS ADDRESS
	STA     INH     ; STORE addresss content to INH, see comment below before SCAND1.
;
; ############## CHANGE BEGIN #########################
;
; We have to change Port A from input to output to drive the 7-Segment LEDs.
; Changing the control word of 8255 will reset all bits of all other output ports to 0,
; very nasty.
;
; Recovery procedure for Port A: nothing to do, because it was input
;
; Recovery procedure for Port B:
;	- We set the bits so that all output lines (0-9) of the
;	  74LS145 (1 out of 10 decoder) are high, resulting in the following:
;	  - No 7-Seg. LED is active
;	  - All row lines of the keyboard matrix are not active (high)
;	  The associated 74LS145 (A-D) input value is 10 = $0A, which has to be shifted by
;	  1 bit to the left, hence it is 20 = $14, see above examples before symbol SCAND.
;
; Recovery procedure for Port C:
; 	- We read the actual output bits of Port C before we
; 	  change the control word of 8255 and restore Port C outputs (PC0 - PC3)
;	  after having changed the control word.
;
SCANDS:			; Note: this symbol will never be called and could be deleted.
	LDA	SCD	; Read port C output status
	PHA		; save it
	LDA     #CW8255_2 ; Control word for: Port A = Output, Port B = Output, Port C = mixed
	STA     PADD	; Set port A to output
	PLA
	STA	SCD	; Restore port C
	LDA	#$14
	STA	SBD	; Set port B, so that all output pins of 74LS145 are high
;
; ############## CHANGE END ###########################
;
	LDX     #$09    ; START WITH DISPLAY AT 74LS145 OUTPUT line number 4 up to 9
	LDY     #$03    ; 3 BYTES TO BE SHOWN
;			; (2 bytes for address, 1 byte for data in address)
;
; Here the KIM-1 software author is doing a nasty trick:
; - He wants to display the address, stored in POINTL and POINTH
; - He wants to display the byte value at this address.
; - He knows, that the address pointer is defined in the zero page RAM in this sequence:
;
;	INL	*=*+1	; RAM location only used in TTY mode
;	INH	*=*+1	; RAM location only used in TTY mode
;	POINTL	*=*+1
;	POINTH	*=*+1
;
; His trick is now, that he stores the byte value of the address to memory
; location INH. Then he can address these 3 bytes (INH, POINTL, POINTH) by a loop,
; beginning with an Y-Index value of 3. In this loop he loads these bytes by the
; instruction:
;
;	LDA INL,Y
;
; where Y-Index begins with 3 and will be decreased by 1 for each loop.
; The result is, that he can first fetch the high byte of the address (POINTH),
; then the low byte of the address (POINTL) and then the RAM
; content at this address, because already stored in INH, see above instruction STA INH.
; Therefore NEVER change the sequence of the RAM address locations from INL to POINTH !!!
; 
SCAND1  LDA     INL,Y   ; GET BYTE (Y=3->POINTH, Y=2->POINTL, Y=1->INH)
	LSR	A       ; GET high nibble of byte BY SHIFTING A four times to the right
	LSR	A
	LSR	A
	LSR	A
	JSR     CONVD	; show the value on the LED display
	LDA     INL,Y   ; GET BYTE AGAIN
	AND     #$0F    ; GET low nibble of byte
	JSR     CONVD	; show the value on the LED display
	DEY             ; ALL ?
	BNE     SCAND1  ; NO ->
; ############## CHANGE BEGIN #########################
;
; We have to change Port A back from output to input.
; Changing the control word will reset all bits of all output ports, very nasty.
;
; Because we immediately set Port B (STX SBD), we need nothing to do here.
; But we have to reset the four Port C output bits (PC0 - PC3).
; We therefore read the actual output bits of Port C before we
; change the control word of 8255 and restore Port C after having changed the control word.
;
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++
;	LDA	SCD	; Read Port C
;	PHA		; and save it
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++
	LDA     #CW8255_1 ; Control word for: Port A = Input, Port B = Output, Port C = mixed
	STA     PADD    ; Set port A to input
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++
;	PLA
;	STA	SCD	; restore Port C
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++
	STX     SBD     ; Set ALL DIGITS to OFF, because X-Register is now $15
			; ($13 and two times INX), which results in 74LS145 decoder
			; input as 10 ($A). With $A as input to decoder, none of the
			; open collector output lines will go low.
;
; ############## CHANGE END ###########################
;
; Now we continue with scanning the keyboard
;
	JMP     AK
;
; CONVERT DIGIT INTO 7-SEGMENT-VALUE
;
; With a ON time of the selected LED digit of 508 us, it takes 3 ms to illuminate
; the 6 LED digits. Hence we have an ON duty cycle for each digit of 1/6, which
; allows a higher current through the LED segments, which is about 35 mA,
; because of:
;	(5V - 3*0.7V) / 82 Ohm = 35 mA
;
; -> 3*0.7 because of two transistors and the LED segment itself.
;
; In a 100% ON duty cycle the maximum current should be less than 20 mA.
; If necessary, the brightness of the LEDs can of course be changed
; by choosing another delay time than the 508 PHI2 cycles.
; However, the mentioned value of 508 cycles is wrong, because $7F * 5 = 635 cycles !
;
CONVD   STY     TEMP
	TAY
	LDA     TABLE,Y
	LDY     #$00
	STY     SAD     ; TURN OFF SEGMENTS
	STX     SBD     ; SELECT 7 SEGMENT DISPLAY
	STA     SAD     ; OUTPUT CODE ON DISPLAY
	LDY     #$7F    ; DELAY 508 PHI2 CLOCK CYCLES (508 MICROSEC)
CONVD1  DEY
	BNE     CONVD1
	INX             ; NEXT DISPLAY with two times INX
	INX		; see comments above
; ---------------------------------------------------------------- PAGE 26 ------
	LDY     TEMP
	RTS
;
; INCREMENT POINTL/H = ADDRESS ON DISPLAY
;
INCPT   INC     POINTL
	BNE     INCPT2
	INC     POINTH
INCPT2  RTS
;
; GET KEY FROM KEYBOARD. RETURN WITH A=KEY. A > 15 -> ILLEGAL OR NO KEY
;
GETKEY  LDX     #$21    ; ROW 0 / DISABLE INPUT TTY
GETKE5  LDY     #$01    ; ONLY ONE ROW IN THE TIME
	JSR     ONEKEY  ; KEY?
	BNE     KEYIN   ; YES ->
	CPX     #$27    ; LAST ROW?
	BNE     GETKE5  ; NO, NEXT ONE ->
	LDA     #$15    ; 15 = NO KEY
	RTS
;
KEYIN   LDY     #$FF    ; Y := KEY NUMBER
KEYIN1  ASL	A       ; SHIFT A UNTIL
	BCS     KEYIN2  ;  BIT = 1 ->
;
; COMMENT: BIT 7 IS ALWAYS 0 SO CARRY IS ALWAYS 0 THE FIRST TIME
;  AND ALLOWING Y TO BECOME 0 (KEY $FF DOES NOT EXIST)
;
	INY     
	BPL     KEYIN1  ; ALWAYS ->

KEYIN2  TXA
	AND     #$0F    ; STRIP BIT4..7
	LSR	A       ; A := ROW+1
	TAX             ; X := ACTUAL ROW+1
	TYA
	BPL     KEYIN4  ; ALWAYS, BECAUSE Y<7 -> 
;
; ADD 7 TO A FOR EVERY ROW ABOVE 0 TO GET ACTUAL KEY NUMBER
;
KEYIN3  CLC
	ADC     #$07    ; ADD (X-1) TIMES 7 TO A
KEYIN4  DEX             ; COUNTDOWN TO 0
	BNE     KEYIN3  
	RTS             ; A IS ALWAYS < 21 EG. < $15
;
; COMPUTE CHECKSUM, USED FOR PAPERTAPE and Tape LOAD/DUMP
;
CHK     CLC
	ADC     CHKSUM
	STA     CHKSUM
	LDA     CHKHI
	ADC     #$00
	STA     CHKHI
	RTS
; ---------------------------------------------------------------- PAGE 27 ------
; GET 2 HEX-CHARS AND PACK INTO INL AND INH
;  NON HEX CHAR WILL BE LOADED AS NEAREST HEX EQUIVALENT
;
GETBYT  JSR     GETCH
	JSR     PACK
	JSR     GETCH
	JSR     PACK
	LDA     INL
	RTS
;
; SHIFT CHAR IN A INTO INL AND INH
; This subroutine is only called in TTY mode and performes the following:
;
; - The user typed character from GETCH is in accu
; - If accu is '0'...'9' or 'A'...'F', shift it into INL/H as 4-bit nibbles
;   in form of 0x0...0x9 and 0xA...0xF and set accu = $00
; - For all other typed characters, INL/H and accu is not changed
;
PACK    CMP	#$30	; If smaller than '0'
	BMI	UPDAT2	; then do nothing and return
	CMP	#$47	; if larger than 'F'
	BPL	UPDAT2	; then do nothing and return
	CMP	#$40	; If smaller then '@', hence '0'...'?' or 0x30 to 0x3F
	BMI	UPDATE	; then branch to UPDATE
	CLC		; when we are here, then accu is '@'...'F'
	ADC	#$09	; add 9 results in 0x49 to 0x4F
UPDATE  ROL	A	; shift low nibble to high nibble (to the left)
	ROL	A	; case 0x30 to 0x39 will result in high nibble to 0x0...0x9
	ROL	A	; case 0x49 to 0x4F will result in high nibble to 0x9...0xF
	ROL	A	; here is an error: should be A...F instead of '9'...'F'
			; to correct for this error, change "CMP #$40" to "CMP #$41"
	LDY	#$04	; now shift high nibble into INL/INH
UPDAT1  ROL	A	; allowing to save 4 nibbles for forming an address
	ROL	INL	; using only INL will allow to change RAM with exactly this value
	ROL	INH
	DEY		; 4 times ?
	BNE	UPDAT1	; branch if not
	LDA	#$00	; Having received '0'...'9' or 'A'...'F', put 0x00 into accu
UPDAT2  RTS
;
OPEN    LDA	INL	; MOVE I/O-BUFFER TO POINT
	STA	POINTL
	LDA	INH
	STA	POINTH
	RTS
; -------------------------------------------------
; Subroutine Show Memory Content (only in TTY mode)
; Show RAM content from SAL/H to EAL/H
; -------------------------------------------------
;
DUMP    LDA     SAL
	STA	POINTL
	LDA	SAH
	STA	POINTH
;
; Check whether we are ready.
;
DUMP1   JSR     CRLF
	LDA     POINTL	; Check if POINTL/H > EAL/H
	CMP     EAL
	LDA     POINTH
	SBC     EAH
	BCC     DUMP2	; branch, when not ready
;
; We are ready.
; Set content of POINTL/H back to SAL/H
;
	LDA     SAL
	STA	POINTL
	LDA	SAH
	STA	POINTH
	RTS
;
; Print a complete line.
;
DUMP2   LDA     #16	; Print 16 data bytes in one line
	TAX		; Save as X index
	JSR     PRTPNT	; Print address in POINTL/H
	LDA	#':'	; Print character ':'
	JSR	OUTCH
	JSR	OUTSP
;
; Print 16 data bytes.
;
DUMP3   LDY     #$00	; Y index is always 0
	LDA     (POINTL),Y
	JSR     PRTBYT	; Print the data byte
	JSR	OUTSP	; Print a space
	CPX	#$09
	BNE	DUMP4
	JSR	OUTSP	; one additional blank after 8 data bytes
DUMP4	JSR     INCPT	; Increment POINTL/H
	DEX		; Printed line completed ?
	BNE     DUMP3	; No ->
	JMP     DUMP1
;
; ---------------------------------------------------------------- PAGE 28 ------
; TABLES
;
; ############## CHANGE BEGIN ###########################
;
; I do not see any reason for transmitting 0x00 bytes to TTY, see symbol "TOP" ????
; In addition, I hate 0x00 bytes in the mid of receiving strings (C language).
; Therefore we have to get rid of them.
; But then we have to change the source code at symbol "CRLF" and right behind
; the symbol "START" calling "PRTST" for printing "KIM\r\n"
;
; TOP     .BYTE     $00,$00,$00,$00,$00,$00,$0A,$0D,"MIK" ; ONLY USED IN FUNCTION CRLF/PRTST
	.BYTE     $00,$00,$00,$00,$00,$00	; we keep the NULLs, but we do not use it
TOP     .BYTE     $0A,$0D,"MIK"			; This is the new string to allow for sending:
						; - "CRLF" = "\r\n" with X-Index $01 and
						; - "KIM\r\n" with X-Index $04
;	.BYTE     " ",$13,"RORRE"," ",$13 ; ONLY USED AT LABEL LOADER WITH LDX = #$11 (LOAD PAPERTAPE)
	.BYTE     $0A,$0D,"RORRE"	; use LDX #$0B to print "Error CRLF KIM CRLF"
					; used at label LOADER with LDX = #$0B (LOAD PAPERTAPE)
; ############## CHANGE END ###########################
;
; TABLE HEX TO 7 SEGMENT
;		0   1   2   3   4   5   6   7
TABLE   .BYTE	$BF,$86,$DB,$CF,$E6,$ED,$FD,$87
;		8   9   A   B   C   D   E   F
	.BYTE	$FF,$EF,$F7,$FC,$B9,$DE,$F9,$F1
;
; *******************************************************
; Subroutine Wait
;
; This subroutine realises accurate time delays, which are used for
; writing and reading data to/from the tape because we do not have a timer, as in KIM-1.
;
; We assume a clock frequency of 1 MHz for the 6502.
;
; The duration for the delay shall be provided in byte address WAIT_TIME
; and is defined as follows:
;
; 	WAIT_TIME = 1 --> 13 +  5 + 10 = 23 cycles = 23 micro seconds
; 	WAIT_TIME = 2 --> 13 + 10 + 10 = 28 cycles = 28 micro seconds
; 	WAIT_TIME = 3 --> 13 + 15 + 10 = 33 cycles = 33 micro seconds
;	:
; 	WAIT_TIME = 255 --> 13 + 1275 + 10 = 1298 cycles = 1298 micro seconds
;
;	Based on this we can write:
;	WAIT_TIME = (Delay time in micro seconds - 23) / 5   with 256 > WAIT_TIME > 0
;
; For realising delay times longer than 1298 us, use multiple calls of WAIT subroutine.
; When using a 2 MHz clock frequency (only for 65C02), then the formula is as follows:
;
;	WAIT_TIME = (Delay time in micro seconds - 11.5) / 2.5   with 256 > WAIT_TIME > 0
;
; Then the maximum possible delay time is 649 micro seconds with WAIT_TIME = 255.
;
; *******************************************************
;
				; JSR WAIT = 6 cycles
WAIT	PHA			; 3 cycles, sum = 9
	LDA	WAIT_TIME	; 4 cycles, sum = 13
;
; Loop over decrementing accumulator content, we assume a branch to same page !
;
WAIT1	SBC	#$01		; 2 cycles
	BNE	WAIT1		; 3 cycles, sum = 5 for every subtraction
;
; Restore accumulator and return
;
	PLA			; 4 cycles
	RTS			; 6 cycles, sum = 10
;
; *******************************************************
; RAM Memory Test Program
; Jim Butterfield, 1977
; The First Book of KIM
; Page 122 - 123
; *******************************************************
;
; Used zero page variables
;
MT_BEG = $00	; Place to $0000 the page number, where to begin with memory test, e.g. $02 = 2
MT_END = $01	; Place to $0001 the page number, where to end with memory test, e.g. $7E = 126
;
MT_FLAG	= $02
MT_FLIP	= $03
MT_MOD	= $04
;
; Start of program at symbol MT_START.
; The program uses the KIM-1 start address symbol "START", originally at $1C4F, here at $EC54.
; It also uses the zero page variables POINTL ($FA) and POINTH ($FB)
; to finally display the result.
; Details on code and how to use it, see the above mentioned book.
;
; In short:
; Set the addresses of the first and last memory page you wish to test
; to locations $0000 and $0001.
; Start the program at symbol MT_START. It will halt with a memory address on the display.
; If no faults were found, the address will be one location past the last address tested.
; If a fault is found, its address will be displayed.
;
; WARNING:
;	Do NOT test the zero page, because there are our variables and some KIM-1 variables.
;	Do NOT test page 1, because there is the KIM-1 stack.
;	Do NOT test page 127, because there are some KIM-1 operating system variables.
;
; To test the 32 KByte RAM (excluding page 0, 1 and 127), define the following:
; 	$0000 = $02
;	$0001 = $7E	(page 126)
;
; Then start the memory test program at symbol MT_START. Then the LED display goes OFF.
; The test program takes about 8 seconds.
;
; The result of the RAM test can be as follows:
; - The LED display shows the address $7F00, which is
;   the first not tested RAM address, hence the test was successful.
;
; - If the display shows another RAM address,
;   then the RAM at this address does not work correctly.
;
; It is worth to mention, that this also works in the TTY mode !
;
MT_START:
	LDA	#0		; set zero pointers for low order addresss
	TAY
	STA	POINTL
MT_BIGLP:
	STA	MT_FLAG		; =$00 first pass, =$FF second pass
	LDX	#2
	STX	MT_MOD		; set 3 tests each pass
MT_PASS	LDA	MT_BEG		; set pointer to start of test area
	STA	POINTH
	LDX	MT_END
	LDA	MT_FLAG
	EOR	#$FF		; reverse FLAG
	STA	MT_FLIP		; =$FF first pass, =$00 second pass
MT_CLEAR:
	STA	(POINTL),Y	; write above FLIP value into all locations
	INY
	BNE	MT_CLEAR
	INC	POINTH
	CPX	POINTH
	BCS	MT_CLEAR
;
; FLIP value in all locations - now change 1 in 3
;
	LDX	MT_MOD
	LDA	MT_BEG		; set pointer back to start
	STA	POINTH
MT_FILL	LDA	MT_FLAG		; change value
MT_TOP	DEX
	BPL	MT_SKIP		; skip 2 out of 3
	LDX	#2		; restore 3-counter
	STA	(POINTL),Y	; change 1 out of 3
MT_SKIP	INY
	BNE	MT_TOP
	INC	POINTH		; new page
	LDA	MT_END		; have we passed end of test area ?
	CMP	POINTH
	BCS	MT_FILL		; no, keep going
;
; Memory is set up - now test it
;
	LDA	MT_BEG		; set pointer back to start
	STA	POINTH
	LDX	MT_MOD		; set up 3-counter
MT_POP	LDA	MT_FLIP		; test for FLIP value 2 out of 3 times ..
	DEX
	BPL	MT_SLIP		; or ..
	LDX	#2		; 1 out of 3 test for FLAG value
	LDA	MT_FLAG
MT_SLIP	CMP	(POINTL),Y	; here is the memory test
	BNE	MT_OUT		; branch if test failed
	INY
	BNE	MT_POP
	INC	POINTH
	LDA	MT_END
	CMP	POINTH
	BCS	MT_POP
;
; Above test was OK - change and repeat
;
	DEC	MT_MOD		; change 1/3 position and do third
	BPL	MT_PASS
	LDA	MT_FLAG		; invert FLAG for pass 2
	EOR	#$FF
	BMI	MT_BIGLP
MT_OUT	STY	POINTL		; put low order address to display pointer
	JMP	START		; and exit to "START" symbol
;
; *********************************
; Second RAM Test program (TR), 10.1.2021
;
; Fill entire RAM range with value $00
; Then set one byte at address TE_AD to $FF (TE_AD = $0201)
; You can define any other address, if necessary for testing,
; however it should be within the RAM test address range.
; Then check that only this byte has changed in the entire RAM.
;
; WARNING:
;	Do NOT test the zero page, because there are our variables and some KIM-1 variables.
;	Do NOT test page 1, because there is the KIM-1 stack.
;	Do NOT test page 127 ($7E), because there are some KIM-1 operating system variables.
;
; Hence we can check in My-KIM from $0200 to $7EFF (page 2 to page 126).
;
; Usage:
; - Enter to the RAM addresses $0001 and $0002 the RAM page range you want to test:
; - e.g. 0000 = $02, 0001 = $7E
; - Then start the program at label "TR".
;
; At the end of the test we can have two situations:
; - if NO error occured, then the LED display shows $0000
; - if an error occured, then the error address is shown on the LED display.
;
; It is worth to mention, that this also works in the TTY mode !
;
; *********************************
;
TE_AD = $0201
;
; Used zero page variables
;
TR_BEG = $00	; Place to $0000 the page number, where to begin with memory test, e.g. $02 = 2
TR_END = $01	; Place to $0001 the page number, where to end with memory test, e.g. $7E = 126
;
; Set up test begin page
;
TR	LDA	TR_BEG
	STA	POINTH
	LDA	#$00
	STA	POINTL
	LDY	#$00
;
; Write this page with $00 values
;
TR_1	STA	(POINTL),Y
	INY
	BNE	TR_1
;
; Increment page.
; When we are at end test page, then continue by setting the $FF value to the RAM
; into a single memory address (TE_AD)
;
	LDY	#$00
	LDX	POINTH
	CPX	TR_END
	BEQ	TR_11
	INC	POINTH
	JMP	TR_1
;
; When we are here, then all pages are written with a $00 value.
; Write a $FF byte to defined test address, which is TE_AD = $0201
;
TR_11	LDA	#$FF
	STA	TE_AD
;
; Now check, that no other byte in RAM has changed.
;
	LDA	TR_BEG
	STA	POINTH
	LDA	#$00
	STA	POINTL
	LDY	#$00
;
; Do the comparision.
; If it is address TE_AD, then skip it.
;
TR_2	LDA	POINTH
	CMP	#>TE_AD
	BNE	TR_3
	CPY	#<TE_AD
	BNE	TR_3
	JMP	TR_4		; Yes, it is address TE_AD, skip comparison
;
TR_3	LDA	(POINTL),Y
	CMP	#$00
	BNE	TR_ERROR	; here we have detected an error !
TR_4	INY
	BNE	TR_2
;
; Increment page.
; When we are at test end page, then stop with success.
;
	LDY	#$00
	LDX	POINTH
	CPX	TR_END
	BEQ	TR_SUCCESS
	INC	POINTH
	JMP	TR_2
;
TR_SUCCESS:
	LDA	#$00
	STA	POINTL
	STA	POINTH
	JMP	START

TR_ERROR:
	STY	POINTL
	JMP	START
;
; *********************************
; Test WAIT Subroutine, 9.1.2021
; *********************************
	LDX	#$FF
	STX	WAIT_TIME
WAIT2	JSR	WAIT
	DEX
	BNE	WAIT2
	JMP	START
;
; *********************************
; User Port Application
; *********************************
;
; The My-KIM design has four port pins for user applications. These are:
;
;	Port C2	working as output
;	Port C3	working as output
;
;	Port C4	working as input
;	Port C5	working as input
;
; The following subroutines offer an easy way to set the output ports
; or to read the input ports.
;
; - For setting an output port, we use the Bit Set/Reset Function of the 8255 PIA
;   The accumulator is temporarily saved on the stack
; - For reading an input port, we read the entire port C,
;   mask the associated bit and return the value in accumulator
;
; Set Port C2 to 0	--> Use JSR SPC2_0
; Set Port C2 to 1	--> Use JSR SPC2_1
; Set Port C3 to 0	--> Use JSR SPC3_0
; Set Port C3 to 1	--> Use JSR SPC3_1
; Read Port C4		--> Use JSR RPOC4, returns 0 in accu, when PC4 is 0, otherwise not 0
; Read Port C5		--> Use JSR RPOC5, returns 0 in accu, when PC5 is 0, otherwise not 0
;
; It shall be noted, that the port output functions do not work
; in the single step mode, because we always alter the 8255 PIA control word
; in this mode, which will reset all output pins to 0.
;
SPC2_0	PHA
	LDA	#$04
	BNE	SPORTC
SPC2_1	PHA
	LDA	#$05
	BNE	SPORTC
SPC3_0	PHA
	LDA	#$06
	BNE	SPORTC
SPC3_1	PHA
	LDA	#$07
SPORTC	STA	PCDD
	PLA
	RTS
RPOC4	LDA	SCD
	AND	#$10
	RTS
RPOC5	LDA	SCD
	AND	#$20
	RTS
;
; *********************************
; Print Software Version to UART
; *********************************
;
PSWV	LDA	#<SWVT	; Take low byte of begin address of string
	STA	SPPL	; String printing pointer low
	LDA	#>SWVT	; Take high byte of begin address of string
	STA	SPPH	; String printing pointer high
;
; *********************************
; Print NULL Terminated String (PNTS) to UART.
; Strings shall never be longer than 255 chars (because using Y-register as index).
; Start address of string shall be in SPPL/H (zero page).
; Save accumulator and Y-register.
; X-register will be saved in subroutine OUTCH,
; but Y-register will be overwritten in OUTCH, therefore save temporarily in TMPY
; *********************************
;
PNTS	PHA		;  Save accu
	TYA		;  Save Y-register
	PHA
;
	LDY     #$00
PNTS1	LDA     (SPPL),Y
	BEQ	PNTS2	; Branch because terminating NULL loaded
	STY	TMPY
	JSR     OUTCH	; Send accumulator to UART, Y-reg. will be overwritten
	LDY	TMPY
	INY
	JMP     PNTS1
;
PNTS2	PLA
	TAY		;  Restore Y-register
	PLA		;  Restore accu
	RTS
;
; Software Version NULL Terminated String, to be sent by subroutine PSWV
; via PNTS (Print Null Terminated String). Address of SWVT shall be in SPPL/H.
;
SWVT	.byte	$0D,$0A,"SW Ver. 22.02.2021",0
;
	*=$FFF0
;
SWVHEX	.byte	$22,$02,$21	; Hex values of software version for LED display
;
;
; -------------------------------------------------------
; INTERRUPT VECTORS
; -------------------------------------------------------
;
	*=$FFFA
NMIENT  .WORD     NMIT
RSTENT  .WORD     RST
IRQENT  .WORD     IRQT
;
; END OF SOURCE CODE
;