/*******************************************************
 Author:					Manfred Langemann
 mailto:					Manfred.Langemann@t-online.de
 Begin of project:			21.11.2020
 Latest version generated:	12.12.2020
 Filename:					Commands.c
 Description:				Command functions
 *******************************************************/
#include <stdio.h>
#include <string.h>
#include <avr/io.h>

#include "General.h"
#include "Delay.h"
#include "RS232.h"
#include "Commands.h"
#include "ProgProc.h"
/*
** Defines for the programming LED.
*/
#define LED_PROG_DDR	SBIT (DDRD, 6)
#define LED_PROG_PORT	SBIT (PORTD, 6)
/*
** Private Function Prototypes.
*/
void Commands_Return (uint8_t iCommandReturn, uint8_t iNumDataBytes);
uint32_t ASCII2Hex (uint8_t *ascii);
/*
** Global variables.
*/
COMMAND_DATA_BLOCK cdb;

/*******************************************************
 Function: Commands_Init

 Purpose: Init the command functionality by init the RS232 link.

 Input parameters: void

 Return value: void
 *******************************************************/
void Commands_Init (void)
	{
/*
** Reset the UART receive buffer.
*/
	RS232_Reset_UART ();
/*
** Configure port for programming LED.
*/
	LED_PROG_DDR = 1;
	LED_PROG_PORT = 0;

	return;
	}
/*******************************************************
 Function: Commands_Process

 Purpose: Get via the RS232 link a command from the PC, analyse and process it

 The function can process the following commands:

	EEPROM_COMMAND_PING,					// Check that programmer is alive
	EEPROM_COMMAND_WRITE_DATA,				// Send max 128 data bytes to be written to EEPROM, based on Intel hex file format
	EEPROM_COMMAND_READ_DATA,				// Request to read up to 128 bytes from EEPROM
	EEPROM_COMMAND_ERASE					// Erase EEPROM with user defined byte value
	
 The command details are defined in the received command and data block structure (cdb)

 Input parameters: void

 Return value:	void
 *******************************************************/
void Commands_Process (void)
	{
	uint8_t				iret;
/*
** Check, whether we have received a command via RS232.
** If no command received, then return.
*/
	if (!RS232_RX (&cdb)) return;
	LED_PROG_PORT = 1;	// Set Programming LED to ON
/*
** Analyse the command and perform the required action.
** The command is in the command and data block structure variable cdb->iCommand.
**
** ------------------------------------------------------------
** We have received a PING command, confirm the command.
** ------------------------------------------------------------
*/
	if (cdb.iCommand == EEPROM_COMMAND_PING)
		{
		Commands_Return (EEPROM_RETURN_COMMAND_CONFIRMATION, 0);
		goto exit;
		}
/*
** ------------------------------------------------------------
** We have received a EEPROM_COMMAND_WRITE_DATA command.
** ------------------------------------------------------------
*/
	if (cdb.iCommand == EEPROM_COMMAND_WRITE_DATA)
		{
/*
** Write the content of the received Intel hex file record to the EEPROM, as specified in the cdb struct.
*/
		iret = Commands_DoWriteToEEPROM ();
		if (iret == FALSE)
			{
			Commands_Return (EEPROM_RETURN_ERROR, 0);			
			}
		else
			{
			Commands_Return (EEPROM_RETURN_COMMAND_CONFIRMATION, 0);
			}
		goto exit;
		}
/*
** ------------------------------------------------------------
** We have received a EEPROM_COMMAND_READ_DATA command.
** ------------------------------------------------------------
*/
	if (cdb.iCommand == EEPROM_COMMAND_READ_DATA)
		{
/*
** Read the bytes from the EEPROM, as specified in the cdb struct.
** The read bytes will be placed to the cdb.Data[] array.
** On success, return in cdb.iNumDataBytes the number of data bytes in the array cdb.Data[]
** For this purpose we use the iPara1 value, as transmitted by the command,
** which has not been changed in function Commands_DoReadFromEEPROM().
*/
		iret = Commands_DoReadFromEEPROM ();
		if (iret == FALSE)
			{
			Commands_Return (EEPROM_RETURN_ERROR, 0);
			}
		else
			{
			Commands_Return (EEPROM_RETURN_COMMAND_CONFIRMATION, cdb.iPara1);
			}
		goto exit;
		}
/*
** ------------------------------------------------------------
** We have received a EEPROM_COMMAND_ERASE command.
** ------------------------------------------------------------
*/
	if (cdb.iCommand == EEPROM_COMMAND_ERASE)
		{
/*
** We immediately confirm the command, because the erasing procedure takes longer
** and the PC program might no wait so long for the command confirmation,
** see timeout statement "commt.ReadTotalTimeoutConstant = 1000;" in function T_COMM::OpenCom() in file Comm.cpp in PC program.
*/
		Commands_Return (EEPROM_RETURN_COMMAND_CONFIRMATION, 0);
		iret = Commands_DoEraseEEPROM ();
/*
** Now confirm once more.
*/
		if (iret == FALSE)
			{
			Commands_Return (EEPROM_RETURN_ERROR, 0);
			}
		else
			{
			Commands_Return (EEPROM_RETURN_COMMAND_CONFIRMATION, 0);
			}
		goto exit;
		}
/*
** When we are here, we have received a wrong command.
** Send back an error message with EEPROM_RETURN_WRONG_COMMAND.
*/
	Commands_Return (EEPROM_RETURN_WRONG_COMMAND, 0);
/*
** Nominal exit of function.
*/	
exit:
	LED_PROG_PORT = 0;	// Set Programming LED to OFF
	return;
	}
/*******************************************************
 Function: Commands_DoWriteToEEPROM

 Purpose: Writes data to the EEPROM.

 Input parameters:
	Global variable:			COMMAND_DATA_BLOCK cdb
	uint16_t iBaseAddress		The base address
	
	The bytes to be written are in the struct array cbd.Data[] in Intel hex file format (excluding the colon)

	The received data in cdb.Data[] has the following format:
	
	Byte 0:		Number of code data bytes in record
	Byte 1+2:	16 bit address (High byte at 1, low byte at 2)
	Byte 3:		Record type
	Byte 4:		First code data byte
	Byte 5:		Second code data byte
	and so on ... until
	Byte n:		Last code data byte
	
	Note: The 16 bit address supplied in byte 1+2 is the address
	for the EEPROM and NOT of the code location itself. See related software in PC program.

 Return value: uint8_t
	TRUE:	Write operation was successfully
	FALSE:	Write operation failed
 *******************************************************/
uint8_t Commands_DoWriteToEEPROM (void)
	{
	uint8_t		i;
	uint8_t		iNumCodeBytes;
	uint16_t	iAddress;
	uint16_t	iAddressLow, iAddressHigh;
/*
** Extract the number of code data bytes we have in the array.
*/
	iNumCodeBytes = cdb.Data[0];
/*
** If number of code data bytes is 0, then do nothing and return (see above shown example record).
*/
	if (iNumCodeBytes == 0) return TRUE;
/*
** Extract the address, where to start to write to the EEPROM.
**	- Index 1 = high byte of address
**	- Index 2 = low byte of address
*/
	iAddressHigh = cdb.Data[1];
	iAddressLow = cdb.Data[2];
	iAddress = (iAddressHigh << 8) + iAddressLow;
/*
** Do the loop over all code data bytes to write to the EEPROM. 
** We begin extracting from index 4, where the first code data byte is written.
** Always increment the write address.
*/
	for (i=0; i<iNumCodeBytes; i++)	
		{
		ProgProc_WriteByte (iAddress, cdb.Data[i+4]);
		iAddress++;
		}
		
	return TRUE;
	}
/*******************************************************
 Function: Commands_DoReadFromEEPROM

 Purpose: Read data from the EEPROM.

 Input parameters:
	Global variable: COMMAND_DATA_BLOCK cdb
	uint16_t iBaseAddress		The base address
	
		cdb.iPara1			// we shall read iNumBytes data bytes
		cdb.iNumDataBytes	// number of data bytes in Comm.cdb.Data is 2 (defining the start address from where to read)
		cdb.Data[0]			// Low order start address to read from
		cdb.Data[1]			// High order start address to read from
			
	The read bytes will be placed to the struct element cbd.Data[]
	
	Note: The 16 bit start address supplied in cdb.Data[0] and cdb.Data[1] is the address
	of the EEPROM and NOT of the code location itself. See related software in PC program.

 Return value: uint8_t
	TRUE:	Read operation was successfully
	FALSE:	Read operation failed
 *******************************************************/
uint8_t Commands_DoReadFromEEPROM (void)
	{
	uint8_t		i;
	uint8_t		iNumBytes;
	uint16_t	iAddress;
/*
** First we check, whether we have received the data correctly.
*/
	iNumBytes = cdb.iPara1;
	if (iNumBytes < 1 || iNumBytes > 128) return FALSE;
	if (cdb.iNumDataBytes != 2) return FALSE;
/*
** Compute the start address, where to read from.
** The address is stored in the first two bytes of the array.
** LSB first, MSB second.
*/
	iAddress = cdb.Data[0] + (cdb.Data[1]<<8);
/*
** Do the loop over all bytes to read.
*/
	for (i=0; i<iNumBytes; i++)	
		{
/*
** Read the byte from the EEPROM and place it to the cdb buffer.
** Then increase the address by 1.
*/		
		ProgProc_ReadByte (iAddress, &cdb.Data[i]);
		iAddress++;
		}
	
	return TRUE;
	}
/*******************************************************
 Function: Commands_DoEraseEEPROM

 Purpose: Erase the EEPROM.

 Input parameters: none

 Return value: uint8_t	--> always TRUE
 *******************************************************/
uint8_t Commands_DoEraseEEPROM (void)
	{
	uint32_t	iAddress;
	uint8_t		iEraseValue;
/*
** Do the loop over all bytes to write to the EEPROM.
*/
	iEraseValue = cdb.iPara1;
	for (iAddress=0; iAddress<8192; iAddress++)
		{
		ProgProc_WriteByte ((uint16_t)iAddress, iEraseValue);
		}
	
	return TRUE;
	}
/*******************************************************
 Function: Commands_Return

 Purpose: Return the command via the RS232 link.

 Input parameters:
	uint8_t		iCommandReturn	EEPROM_RETURN_COMMAND_CONFIRMATION or EEPROM_RETURN_WRONG_COMMAND
	uint8_t		iNumDataBytes	Number of data bytes to be returned

 Return value: void
 *******************************************************/
void Commands_Return (uint8_t iCommandReturn, uint8_t iNumDataBytes)
	{
	uint8_t		i;
/*
** Fill the cdb with the command response to send to the PC.
*/
	cdb.iCommand = iCommandReturn;
	cdb.iNumDataBytes = iNumDataBytes;
/*
** Send the Command and Data Block content to the PC.
** First send the 4 command header bytes of the Command and Data Block to RS232.
*/
	RS232_PutByte (cdb.iCommand);
	RS232_PutByte (cdb.iPara1);
	RS232_PutByte (cdb.iPara2);
	RS232_PutByte (cdb.iNumDataBytes);
/*
** If we have to transmit data array bytes, then do it here.
*/	
	for (i=0; i<cdb.iNumDataBytes; i++)
		{
		RS232_PutByte (cdb.Data[i]);
		}

	return;
	}
/************************************************************************
	Function: ASCII2Hex
	
	Convert an ASCII Hex string to a number
	Any string length is accepted, but the number shall not exceed the uint32_t range
*************************************************************************/
uint32_t ASCII2Hex (uint8_t *ascii)
	{
	uint8_t		ch;
	uint32_t	nval = 0;
	uint16_t	mul = 16;
	uint16_t	i=0;

	while ((ch = ascii[i++] - '0') >= 0)
		{
		if (ch > 9)
			{
			ch -= ('A' - '9' - 1);
			if (ch > 15) ch -= ('a' - 'A');
			if (ch > 15) break;
			if (ch < 10) break;
			}
		if (ch >= mul) break;
		nval = (nval * mul) + ch;
		}

	return nval;
	}	