/*******************************************************
 Author:					Manfred Langemann
 mailto:					Manfred.Langemann@t-online.de
 Begin of project:			21.11.2020
 Latest version generated:	12.12.2020
 Filename:					ProgProc.c
 Description:				Low Level Programming Procedure functions
 *******************************************************/
#include <stdio.h>
#include <string.h>
#include <avr/io.h>

#include "General.h"
#include "Delay.h"
#include "RS232.h"
#include "ProgProc.h"
/*
** Defines for the data ports D0 to D7, to write to and read from the EEPROM.
*/
#define DATA_D0_DDR		SBIT (DDRC, 2)		// D0 DDR
#define DATA_D1_DDR		SBIT (DDRD, 2)		// D1 DDR
#define DATA_D2_DDR		SBIT (DDRC, 4)		// D2 DDR
#define DATA_D3_DDR		SBIT (DDRC, 5)		// D3 DDR
#define DATA_D4_DDR		SBIT (DDRD, 4)		// D4 DDR
#define DATA_D5_DDR		SBIT (DDRC, 3)		// D5 DDR
#define DATA_D6_DDR		SBIT (DDRD, 5)		// D6 DDR
#define DATA_D7_DDR		SBIT (DDRD, 3)		// D7 DDR

#define DATA_D0_OUT		SBIT (PORTC, 2)		// D0 output
#define DATA_D1_OUT		SBIT (PORTD, 2)		// D1 output
#define DATA_D2_OUT		SBIT (PORTC, 4)		// D2 output
#define DATA_D3_OUT		SBIT (PORTC, 5)		// D3 output
#define DATA_D4_OUT		SBIT (PORTD, 4)		// D4 output
#define DATA_D5_OUT		SBIT (PORTC, 3)		// D5 output
#define DATA_D6_OUT		SBIT (PORTD, 5)		// D6 output
#define DATA_D7_OUT		SBIT (PORTD, 3)		// D7 output

#define DATA_D0_IN		(PINC & 1<<PC2)		// D0 input
#define DATA_D1_IN		(PIND & 1<<PD2)		// D1 input
#define DATA_D2_IN		(PINC & 1<<PC4)		// D2 input
#define DATA_D3_IN		(PINC & 1<<PC5)		// D3 input
#define DATA_D4_IN		(PIND & 1<<PD4)		// D4 input
#define DATA_D5_IN		(PINC & 1<<PC3)		// D5 input
#define DATA_D6_IN		(PIND & 1<<PD5)		// D6 input
#define DATA_D7_IN		(PIND & 1<<PD3)		// D7 input
/*
** Defines for the /CE, /WE and /OE control pins of the EEPROM.
*/
#define CNTR_CE_DDR		SBIT (DDRC, 1)		// CE Pin
#define CNTR_WE_DDR		SBIT (DDRC, 0)		// WE Pin
#define CNTR_OE_DDR		SBIT (DDRB, 1)		// OE Pin

#define CNTR_CE_PORT	SBIT (PORTC, 1)		// CE Pin
#define CNTR_WE_PORT	SBIT (PORTC, 0)		// WE Pin
#define CNTR_OE_PORT	SBIT (PORTB, 1)		// OE Pin
/*
** Defines for 2 times cascaded 74HC595 shift registers,
** where the parallel outputs are acting as 16 bit address lines for the EEPROM,
** but only address lines A0 - A12 are used for the 8 KByte EEPROM.
** Note: the SCL pin is connected to high level, the G pin is connected to low level.
*/
#define ADDR_SER_DDR	SBIT (DDRB, 2)		// Serial In
#define ADDR_SCK_DDR	SBIT (DDRB, 0)		// Clock
#define ADDR_RCK_DDR	SBIT (DDRD, 7)		// Register Clock

#define ADDR_SER_PORT	SBIT (PORTB, 2)		// Serial In
#define ADDR_SCK_PORT	SBIT (PORTB, 0)		// Clock
#define ADDR_RCK_PORT	SBIT (PORTD, 7)		// Register Clock
/*
** Private function prototypes.
*/
uint8_t ProgProc_GetDataLines (void);
void ProgProc_SetDataLines (uint8_t iByte);
void ProgProc_SetAddressLines (uint16_t iAddress);
void ProgProc_SetDataPortToInput (void);
void ProgProc_SetDataPortToOutput (void);

/*******************************************************
 Function: ProgProc_Init

 Purpose: Init the programmer functionality.

 Input parameters: void

 Return value: void
 *******************************************************/
void ProgProc_Init (void)
	{
/*
** Set /CE and /WE and /OE ports to output and set outputs to High Level (EEPROM disabled)
*/	
	CNTR_CE_DDR = 1;
	CNTR_WE_DDR = 1;
	CNTR_OE_DDR = 1;

	CNTR_CE_PORT = 1;
	CNTR_WE_PORT = 1;
	CNTR_OE_PORT = 1;
/*
** Set ATmega8 data ports (D0-D7) to input.
*/
	ProgProc_SetDataPortToInput ();
/*
** Set 74HC595 shift register related ports to output and defined levels.
*/
	ADDR_SER_DDR = 1;
	ADDR_SCK_DDR = 1;
	ADDR_RCK_DDR = 1;

	ADDR_SER_PORT = 1;
	ADDR_SCK_PORT = 1;	// latches from low to high level
	ADDR_RCK_PORT = 1;	// latches from low to high level

	return;
	}
/*******************************************************
 Function: ProgProc_ReadByte

 Purpose: Read a byte from the EEPROM (Size = 8 KByte)

 Input parameters:
	uint16_t iAddress	The address, from where to read the byte from the EEPROM
						Valid range is: 0x0000 to 0x1FFF, i.e. A0 - A12
	uint8_t *iByte		Pointer to byte, where to place the read byte

 Return value: void
 *******************************************************/
void ProgProc_ReadByte (uint16_t iAddress, uint8_t *iByte)
	{
/*
** Set data (D0-D7) port pins to input.
*/
	ProgProc_SetDataPortToInput ();
/*
** Set the address lines, A0-A12.
*/
	ProgProc_SetAddressLines (iAddress);
/*
** Enable the EEPROM by low level at pin /CE and /OE.
** Wait period, before we can read the EEPROM data.
*/
	CNTR_CE_PORT = 0;
	CNTR_OE_PORT = 0;
	Delay_us (1);
/*
** Read the EEPROM data byte at the controller input ports.
 */
	*iByte = ProgProc_GetDataLines ();
/*
** Disable the EEPROM by high level at pin /OE and /CE.
*/
	CNTR_OE_PORT = 1;
	CNTR_CE_PORT = 1;

	return;
	}
/*******************************************************
 Function: ProgProc_WriteByte

 Purpose: Write the byte to the EEPROM (8 KByte)
			The procedure uses the /WE controlled programming mode !

 Input parameters:
	uint16_t iAddress	The address, where to write the byte to the EEPROM
						Valid range is: 0x000 to 0x1FFF, i.e A0 - A12.
	uint8_t iByte		The byte to write to the EEPROM

 Return value: void
 *******************************************************/
void ProgProc_WriteByte (uint16_t iAddress, uint8_t iByte)
	{
/*
** To be on the sure side that nothing collides,
** disable the EEPROM by a high level at the /CE, /WE and /OE pins.
*/
	CNTR_OE_PORT = 1;
	CNTR_CE_PORT = 1;
	CNTR_WE_PORT = 1;
/*
** Set controller data ports to output.
*/
	ProgProc_SetDataPortToOutput ();
/*
** Set the data line (D0-D7) and the address lines (A0-A12).
*/	
	ProgProc_SetDataLines (iByte);
	ProgProc_SetAddressLines (iAddress);
/*
** Enable the EEPROM by low level at pin /CE.
*/
	CNTR_CE_PORT = 0;
/*
** Write pulse to EEPROM with low level at pin /WE with a duration of min 200 ns.
** This procedure uses the /WE controlled programming mode !
*/	
	CNTR_WE_PORT = 0;
	Delay_us (1);
	CNTR_WE_PORT = 1;
/*
** We have to wait for 3 (6) milli seconds, because the EEPROM has to complete its write cycle !
** Disable the EEPROM by high level at pin /CE.
*/
	Delay_ms (6);		// This delay time is necessary for Atmel AT28C64B, manufactured 2020
	//Delay_ms (3);		// Other devices may need less time, e.g. M28C64C-15PI from ST Microelectronics, manufactured 1997
	CNTR_CE_PORT = 1;
/*
** Set controller data port pins to input for security.
*/	
	ProgProc_SetDataPortToInput ();
	
	return;
	}
/*******************************************************
 Function: ProgProc_GetDataLines

 Purpose: Get data (D0-D7) lines

 Input parameters: void

 Return value: uint8_t
	The read data byte from the EEPROM
 *******************************************************/
uint8_t ProgProc_GetDataLines (void)
	{
	uint8_t		iByte = 0;
/*
** Read the controller input ports, which are connected to the data lines D0-D7 of the EEPROM and retrieve the byte.
** Warning: never test on e.g. (DATA_D0_IN == 1), because DATA_D0_IN is defined as (PINC & 1<<PC2).
** Only test on not equal or equal 0. In earlier projects this cost me several hours of debugging !?!
*/
	if (DATA_D0_IN != 0) iByte = iByte | 0x01;
	if (DATA_D1_IN != 0) iByte = iByte | 0x02;
	if (DATA_D2_IN != 0) iByte = iByte | 0x04;
	if (DATA_D3_IN != 0) iByte = iByte | 0x08;
	if (DATA_D4_IN != 0) iByte = iByte | 0x10;
	if (DATA_D5_IN != 0) iByte = iByte | 0x20;
	if (DATA_D6_IN != 0) iByte = iByte | 0x40;
	if (DATA_D7_IN != 0) iByte = iByte | 0x80;	

	return iByte;
	}
/*******************************************************
 Function: ProgProc_SetDataLines

 Purpose: Set data (D0-D7) lines

 Input parameters:
	uint8_t		iByte	the byte value to write to the EEPROM

 Return value: void
 *******************************************************/
void ProgProc_SetDataLines (uint8_t iByte)
	{
/*
** Set the controller data output ports according to the byte to write.
*/
	if (iByte & 0x01) {DATA_D0_OUT = 1;} else {DATA_D0_OUT = 0;}
	if (iByte & 0x02) {DATA_D1_OUT = 1;} else {DATA_D1_OUT = 0;}
	if (iByte & 0x04) {DATA_D2_OUT = 1;} else {DATA_D2_OUT = 0;}
	if (iByte & 0x08) {DATA_D3_OUT = 1;} else {DATA_D3_OUT = 0;}
	if (iByte & 0x10) {DATA_D4_OUT = 1;} else {DATA_D4_OUT = 0;}
	if (iByte & 0x20) {DATA_D5_OUT = 1;} else {DATA_D5_OUT = 0;}
	if (iByte & 0x40) {DATA_D6_OUT = 1;} else {DATA_D6_OUT = 0;}
	if (iByte & 0x80) {DATA_D7_OUT = 1;} else {DATA_D7_OUT = 0;}		

	return;
	}
/*******************************************************
 Function: ProgProc_SetAddressLines

 Purpose: Set the address lines (A0-A15) by shifting the address value
			to the 16-bit shift register of the two cascaded 74HC595.

 Input parameters:
	uint16_t	iAddress	the address where to write the byte to the EEPROM

 Return value: void
 *******************************************************/
void ProgProc_SetAddressLines (uint16_t iAddress)
	{
	uint8_t		i;
/*
** Shift the 16 bit address value into the two cascaded 74HC595 shift registers. Most significant bit first.
** Because we can program only 8 KByte EEPROMs, the meaningful address lines are A0 - A12,
** i.e. QA1-QH1, QA2-QE2 --> A0 is at QA1 up to QE2 is at A12.
** It is assumed, that the QH* output pin of the first shift register is connected to the SER input pin of the cascaded shift register.
** Place the actual bit to the serial SER input line of the first 74HC595.
** Do the loop over the 16 bits of the variable iAddress.
*/
	for (i=0; i<16; i++)
		{
		ADDR_SER_PORT = 0;
		if (iAddress & 0x8000) ADDR_SER_PORT = 1;
		ADDR_SCK_PORT = 0;
		Delay_us (1);			// low pulse shall be 1 us
		ADDR_SCK_PORT = 1;		// bit will be clocked in by this rising signal
		iAddress = iAddress << 1;
		}
/*
** Store the value in the shift register to the 74HC595 internal storage register by a low pulse on the RCK pin.
** Then the shifted address value will appear on the parallel output pins 2 x (QA - QH)
*/	
	ADDR_RCK_PORT = 0;
	Delay_us (1);				// low pulse shall be 1 us
	ADDR_RCK_PORT = 1;			// data will be clocked into storage register by this rising signal
/*
** Wait for address line settling time.
*/
	Delay_us (1);

	return;
	}
/*******************************************************
 Function: ProgProc_SetDataPortToInput

 Purpose: Set data (D0-D7) lines to EEPROM to input

 Input parameters: void

 Return value: void
 *******************************************************/
void ProgProc_SetDataPortToInput (void)
	{
	DATA_D0_DDR = 0;
	DATA_D1_DDR = 0;
	DATA_D2_DDR = 0;
	DATA_D3_DDR = 0;
	DATA_D4_DDR = 0;
	DATA_D5_DDR = 0;
	DATA_D6_DDR = 0;
	DATA_D7_DDR = 0;
	return;
	}
/*******************************************************
 Function: ProgProc_SetDataPortToOutput

 Purpose: Set data (D0-D7) lines to EEPROM to output

 Input parameters: void

 Return value: void
 *******************************************************/
void ProgProc_SetDataPortToOutput (void)
	{
	DATA_D0_DDR = 1;
	DATA_D1_DDR = 1;
	DATA_D2_DDR = 1;
	DATA_D3_DDR = 1;
	DATA_D4_DDR = 1;
	DATA_D5_DDR = 1;
	DATA_D6_DDR = 1;
	DATA_D7_DDR = 1;
	return;
	}
