/*******************************************************
 Author:					Manfred Langemann
 mailto:					Manfred.Langemann t t-online.de
 Begin of project:			21.11.2020
 Latest version generated:	21.11.2020
 Filename:					RS232.c
 Description:				Implementation of RS232 Interface
 *******************************************************/
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "General.h"
#include "Commands.h"
#include "RS232.h"
/*
** Baud rate for serial RS232 link
*/
#define BAUD_RATE		9600
/*
** Define a stream for the printf function, by using the function RS232_putchar
*/
FILE uart_str = FDEV_SETUP_STREAM(RS232_putchar, NULL, _FDEV_SETUP_RW);

#define UART_RECEIVE_BUFFER_SIZE	MAX_DATA_BYTES+4
/*
** Global variables
*/
volatile uint8_t	UART_Buffer[UART_RECEIVE_BUFFER_SIZE];
volatile uint8_t	UART_NumBytes;
volatile uint8_t	UART_NumBytesToReceive;
volatile uint8_t	UART_CommandValid;

/*******************************************************
 Function: RS232_Init

 Purpose: Initialise the RS232 serial link

 Input Parameter: None

 Return Value: None
*******************************************************/
void RS232_Init (void)
	{
/*
** Enable TXEN and RXEN in register UCSRB.
*/
	UCSRB |= (1 << TXEN);
	UCSRB |= (1 << RXEN);
/*
** Enable UART Receive Interrupt.
*/
	UCSRB |= (1 << RXCIE);
/*
** Set transmission type, Asynchron 8N1.
*/
	UCSRC |= (1 << URSEL)|(3<<UCSZ0);
/*
** Set baud rate.
** With F_CPU = 10MHz and Baud rate = 9600, the error is 0.2%
*/
    UBRRH = (uint8_t) ((F_CPU / (BAUD_RATE * 16L) - 1)>>8);
    UBRRL = (uint8_t) (F_CPU / (BAUD_RATE * 16L) - 1);
/*
** Open a channel for printf.
*/
	stdout = &uart_str;
/*
** Set the default UART buffer receive parameters.
*/
	RS232_Reset_UART ();
/*
** Enable interrupts.
*/	
	sei ();
	}
/*******************************************************
 Function: RS232_Reset_UART

 Purpose:
	Reset the UART buffer parameters.
	
 Input Parameter: void

 Return Value:	void
*********************************************************/
void RS232_Reset_UART (void)
	{
/*
** Set the UART buffer receive parameters to default start values.
*/
	UART_CommandValid = FALSE;
	UART_NumBytesToReceive = 0;
	UART_NumBytes = 0;
	
	return;
	}
/*******************************************************
 Function: RS232_RX

 Purpose:
	Get data via RS232 by the PC.
	Whether data has been received is flagged by UART_CommandValid.
	
 Input Parameter:
	COMMAND_DATA_BLOCK		*cdb	The received bytes have to be copied to this structure

 Return Value:	uint8_t
	TRUE:	Command correctly received
	FALSE:	No command received, or less than 4 bytes received
*********************************************************/
uint8_t RS232_RX (COMMAND_DATA_BLOCK *cdb)
	{
	uint8_t		i;
/*
** Is command valid ?
*/
	if (!UART_CommandValid) return FALSE;
/*
** Disable all interrupts.
*/
	cli ();
/*
** If we have received less than 4 bytes, then this is an error.
** Reset the UART parameters.
*/
	if (UART_NumBytes < 4)
		{
		RS232_Reset_UART ();
		sei ();
		return FALSE;
		}
/*
** Copy the received UART bytes to the Command and Data Block structure.
** Note: we can only accept in total UART_RECEIVE_BUFFER_SIZE bytes because of defined size of UART_RECEIVE_BUFFER_SIZE
*/
	cdb->iCommand = UART_Buffer[0];			
	cdb->iPara1 = UART_Buffer[1];			
	cdb->iPara2 = UART_Buffer[2];			
	cdb->iNumDataBytes = UART_Buffer[3];		

	for (i=0; i<cdb->iNumDataBytes; i++)
		{
		if (i<UART_RECEIVE_BUFFER_SIZE-4) cdb->Data[i] = UART_Buffer[i+4];
		}
/*
** Invalidate the RS232 buffer and associated global variables.
** This allows the PC to send new data via the RS232 link.
*/
	RS232_Reset_UART ();
/*
** Enable all interrupts.
*/
	sei ();

	return TRUE;
	}

/*******************************************************
 Function: RS232_putchar

 Purpose: Initialise function "printf "

 Input Parameter: char c

 Return Value:
 	Value = 0
*******************************************************/
int RS232_putchar (char c, FILE *stream)
	{
/*
** Check on NEWLINE.
*/
	if (c == '\n')
		{
		RS232_putchar ('\r', stream);
		}
/*
** Wait until previous character is sent.
*/
	loop_until_bit_is_set(UCSRA, UDRE);
/*
** Send character to UART.
*/
	UDR = c;

	return 0;
	}
/*******************************************************
 Function: RS232_PutString

 Purpose: Send a NULL terminating string to the UART

 Input Parameter: char*		s		The string

 Return Value: void
*******************************************************/
void RS232_PutString (const char* s)
	{
/*
** When the string is empty, then do nothing.
*/
	if (*s == 0) return;
/*
** Otherwise start to send the characters of the NULL terminating string.
** Do it until we reach the NULL terminating character.
*/
	do
		{
		RS232_PutByte ((uint8_t)*s);
		}
		while (*s++);
	
	return;
	}
/*******************************************************
 Function: RS232_PutByte

 Purpose: Send to the UART a data byte

 Input Parameter: the byte

 Return Value: None
 	Value = 0
*******************************************************/
void RS232_PutByte (uint8_t Byte)
	{
/*
** Wait until previous byte is sent. Then send byte to UART.
*/
	while (!(UCSRA & (1<<UDRE)));
	UDR = Byte;

	return;
	}
/*******************************************************
 Function: Interrupt Vector Routine USART RXC complete

 Purpose:
	Will be called by interrupt, when a byte has been received by the UART
 *******************************************************/
ISR (USART_RXC_vect)
	{
/*
** Disable all interrupts.
*/
	cli ();
/*
** If it is the first byte we receive, i.e. UART_NumBytesToReceive == 0,
** then this byte is the number of bytes, which will follow.
** Save it and return from here because nothing else to do.
*/
	if (UART_NumBytesToReceive == 0)
		{
		UART_NumBytesToReceive = UDR;
		sei ();
		return;
		}		
/*
** Otherwise write the received byte to the UART_Buffer.
** Do not write the byte to the UART buffer, when we have exceeded the UART buffer size.
*/
	if (UART_NumBytes < UART_RECEIVE_BUFFER_SIZE) UART_Buffer[UART_NumBytes] = UDR;
/*
** Increment the UART buffer byte counter.
*/
	UART_NumBytes++;
/*
** If we have received all data bytes, then the transmission is completed
** and indicate that the command is valid.
*/
	if (UART_NumBytesToReceive == UART_NumBytes) UART_CommandValid = TRUE;
/*
** Enable all interrupts.
*/
	sei ();

	return;
	}